"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array-buffer.slice.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.typed-array.uint8-array.js");

require("core-js/modules/es.typed-array.copy-within.js");

require("core-js/modules/es.typed-array.every.js");

require("core-js/modules/es.typed-array.fill.js");

require("core-js/modules/es.typed-array.filter.js");

require("core-js/modules/es.typed-array.find.js");

require("core-js/modules/es.typed-array.find-index.js");

require("core-js/modules/es.typed-array.for-each.js");

require("core-js/modules/es.typed-array.includes.js");

require("core-js/modules/es.typed-array.index-of.js");

require("core-js/modules/es.typed-array.iterator.js");

require("core-js/modules/es.typed-array.join.js");

require("core-js/modules/es.typed-array.last-index-of.js");

require("core-js/modules/es.typed-array.map.js");

require("core-js/modules/es.typed-array.reduce.js");

require("core-js/modules/es.typed-array.reduce-right.js");

require("core-js/modules/es.typed-array.reverse.js");

require("core-js/modules/es.typed-array.set.js");

require("core-js/modules/es.typed-array.slice.js");

require("core-js/modules/es.typed-array.some.js");

require("core-js/modules/es.typed-array.sort.js");

require("core-js/modules/es.typed-array.subarray.js");

require("core-js/modules/es.typed-array.to-locale-string.js");

require("core-js/modules/es.typed-array.to-string.js");

require("core-js/modules/es.array-buffer.constructor.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/web.timers.js");

require("core-js/modules/es.typed-array.int32-array.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.array.slice.js");

require("core-js/modules/es.data-view.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var flatOptions = require('flat-options');

var PromiseController = require('promise-controller');

var isNull = require('lodash.isnull');

var forEach = require('lodash.foreach');

var isEqual = require('lodash.isequal');

var isUndefined = require('lodash.isundefined');

var concat = require('lodash.concat');

var formatMessage = require('format-message');

var sodium = require('libsodium-wrappers');

var Mutex = require('async-mutex');

var defaultOptions = require('./channel-options');

var edge_protocol = require('./protocol');

var zitiConstants = require('../constants');

var ZitiConnections = require('./connections');

var ZitiWebSocket = require('../websocket/websocket');

var Header = require('./header');

var Messages = require('./messages');

var _require = require('../utils/throwif'),
    throwIf = _require.throwIf;

var utils = require('../utils/utils');

var ZitiTLSConnection = require('./tls-connection');
/**
 * @typicalname channel
 */


module.exports = /*#__PURE__*/function () {
  /**
   *
   * @param {Options} [options]
   */
  function ZitiChannel(options) {
    _classCallCheck(this, ZitiChannel);

    this._options = flatOptions(options, defaultOptions);
    this._ctx = this._options.ctx;
    this._id = this._ctx.getNextChannelId();
    this._data = this._options.data;
    this._timeout = this._options.timeout;
    this._helloTimeout = this._options.helloTimeout;
    this._state = edge_protocol.conn_state.Initial;
    this._msgSeq = -1;
    this._edgeRouter = this._options.edgeRouter;
    this._edgeRouterHost = this._edgeRouter.hostname;
    this._connections = new ZitiConnections();
    this._zws = new ZitiWebSocket(this._edgeRouter.urls.ws + '/ws', {
      ctx: this._ctx
    });
    this._callerId = "ws:";

    this._zws.onMessage.addListener(this._recvFromWire, this);

    this._zws.onClose.addListener(this._recvClose, this);

    this._zws.onSend.addListener(this._recvSend, this);

    this._createHelloController(); // Set the maximum timestamp


    this._helloCompletedTimestamp = new Date(8640000000000000); // http://www.ecma-international.org/ecma-262/5.1/#sec-15.9.1.1

    this._messages = new Messages({
      ctx: this._ctx,
      channel: this
    });
    this._view_version = new Uint8Array(new ArrayBuffer(4));

    this._view_version.set(edge_protocol.VERSION, 0);

    this._mutex = new Mutex.Mutex();
  }

  _createClass(ZitiChannel, [{
    key: "getCtx",
    value: function getCtx() {
      return this._ctx;
    }
  }, {
    key: "getId",
    value: function getId() {
      return this._id;
    }
  }, {
    key: "getData",
    value: function getData() {
      return this._data;
    }
  }, {
    key: "getState",
    value: function getState() {
      return this._state;
    }
  }, {
    key: "setState",
    value: function setState(state) {
      this._state = state;
    }
  }, {
    key: "getAndIncrementSequence",
    value: function getAndIncrementSequence() {
      return this._msgSeq++;
    }
  }, {
    key: "getEdgeRouterHost",
    value: function getEdgeRouterHost() {
      return this._edgeRouterHost;
    }
    /**
     * 
     */

  }, {
    key: "helloCompletedTimestamp",
    get: function get() {
      return this._helloCompletedTimestamp;
    }
    /**
     * Has this Channel already completed the 'hello' sequence?
     *
     * @returns {Boolean}
     */

  }, {
    key: "isHelloCompleted",
    get: function get() {
      return Boolean(this._helloCompleted);
    }
  }, {
    key: "_createHelloController",
    value: function _createHelloController() {
      var helloTimeout = this._helloTimeout || this._timeout;
      this._helloing = new PromiseController({
        timeout: helloTimeout,
        timeoutReason: "Can't complete 'Hello' within allowed timeout: ".concat(helloTimeout, " ms.")
      });
    }
    /**
     * Remain in lazy-sleepy loop until tlsConn has completed its TLS handshake.
     * 
     */

  }, {
    key: "awaitTLSHandshakeComplete",
    value: function () {
      var _awaitTLSHandshakeComplete = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
        var self;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                self = this;
                return _context.abrupt("return", new Promise(function (resolve) {
                  (function waitForTLSHandshakeComplete() {
                    if (!self._tlsConn.isTLSHandshakeComplete()) {
                      self._ctx.logger.trace('awaitTLSHandshakeComplete() tlsConn for ch[%d] TLS handshake still not complete', self.getId());

                      setTimeout(waitForTLSHandshakeComplete, 100);
                    } else {
                      self._ctx.logger.trace('tlsConn for ch[%d] TLS handshake is now complete', self.getId());

                      return resolve();
                    }
                  })();
                }));

              case 2:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function awaitTLSHandshakeComplete() {
        return _awaitTLSHandshakeComplete.apply(this, arguments);
      }

      return awaitTLSHandshakeComplete;
    }()
    /**
     * Do Hello handshake between this channel and associated Edge Router.
     * 
     */

  }, {
    key: "hello",
    value: function () {
      var _hello = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
        var _this = this;

        var headers, sequence, msg;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return this._zws.open();

              case 2:
                if (!this.isHelloCompleted) {
                  _context4.next = 5;
                  break;
                }

                this._ctx.logger.trace('Hello handshake was previously completed');

                return _context4.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(resolve) {
                    return regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            resolve({
                              channel: _this,
                              data: null
                            });

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  return function (_x) {
                    return _ref.apply(this, arguments);
                  };
                }()));

              case 5:
                if (!isEqual(this._callerId, "ws:")) {
                  _context4.next = 17;
                  break;
                }

                this._tlsConn = new ZitiTLSConnection({
                  type: 'edge-router',
                  ctx: this._ctx,
                  ws: this._zws,
                  ch: this,
                  datacb: this._recvFromWireAfterDecrypt
                });

                this._ctx.logger.debug('starting  ZitiTLSConnection.pullKeyPair for: [%o]', this._tlsConn.getUUID());

                _context4.next = 10;
                return this._tlsConn.pullKeyPair().catch(function (e) {
                  _this._ctx.logger.error(e);

                  throw new Error(e);
                });

              case 10:
                this._ctx.logger.debug('completed ZitiTLSConnection.pullKeyPair for: [%o]', this._tlsConn.getUUID());

                this._tlsConn.create();

                this._ctx.logger.debug('initiating TLS handshake');

                this._tlsConn.handshake();

                _context4.next = 16;
                return this.awaitTLSHandshakeComplete();

              case 16:
                this._ctx.logger.debug('TLS handshake completed');

              case 17:
                this._ctx.logger.debug('initiating message: edge_protocol.content_type.HelloType');

                headers = [new Header(edge_protocol.header_id.SessionToken, {
                  headerType: edge_protocol.header_type.StringType,
                  headerData: this._options.session_token
                }), new Header(edge_protocol.header_id.CallerId, {
                  headerType: edge_protocol.header_type.StringType,
                  headerData: this._callerId
                })];
                sequence = this.getAndIncrementSequence();
                _context4.next = 22;
                return this.sendMessage(edge_protocol.content_type.HelloType, headers, null, {
                  sequence: sequence
                });

              case 22:
                msg = _context4.sent;
                this._helloCompletedTimestamp = Date.now();
                this._helloCompleted = true;
                this.setState(edge_protocol.conn_state.Connected);

                this._ctx.logger.debug('ch[%d] Hello handshake to Edge Router [%s] completed at timestamp[%o]', this._id, this._edgeRouterHost, this._helloCompletedTimestamp);

                return _context4.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(resolve) {
                    return regeneratorRuntime.wrap(function _callee3$(_context3) {
                      while (1) {
                        switch (_context3.prev = _context3.next) {
                          case 0:
                            resolve({
                              channel: _this,
                              data: null
                            });

                          case 1:
                          case "end":
                            return _context3.stop();
                        }
                      }
                    }, _callee3);
                  }));

                  return function (_x2) {
                    return _ref2.apply(this, arguments);
                  };
                }()));

              case 28:
              case "end":
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function hello() {
        return _hello.apply(this, arguments);
      }

      return hello;
    }()
    /**
     * Connect specified Connection to associated Edge Router.
     * 
     */

  }, {
    key: "connect",
    value: function () {
      var _connect = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(conn) {
        var _this2 = this;

        var self;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                self = this;
                return _context6.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(resolve, reject) {
                    var keypair, sequence, headers, msg;
                    return regeneratorRuntime.wrap(function _callee5$(_context5) {
                      while (1) {
                        switch (_context5.prev = _context5.next) {
                          case 0:
                            self._ctx.logger.debug('initiating Connect to Edge Router [%s] for conn[%d]', _this2._edgeRouterHost, conn.getId());

                            _context5.next = 3;
                            return sodium.ready;

                          case 3:
                            keypair = sodium.crypto_kx_keypair();
                            conn.setKeypair(keypair);
                            sequence = _this2.getAndIncrementSequence();
                            headers = [new Header(edge_protocol.header_id.ConnId, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: conn.getId()
                            }), new Header(edge_protocol.header_id.SeqHeader, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: 0
                            })];

                            if (conn.getEncrypted()) {
                              // if connected to a service that has 'encryptionRequired'
                              headers.push(new Header(edge_protocol.header_id.PublicKey, {
                                headerType: edge_protocol.header_type.Uint8ArrayType,
                                headerData: keypair.publicKey
                              }));
                            }

                            conn.setState(edge_protocol.conn_state.Connecting);

                            self._ctx.logger.debug('about to send Connect to Edge Router [%s] for conn[%d]', conn.getChannel().getEdgeRouterHost(), conn.getId());

                            _context5.next = 12;
                            return self.sendMessage(edge_protocol.content_type.Connect, headers, self._options.network_session_token, {
                              conn: conn,
                              sequence: sequence
                            });

                          case 12:
                            msg = _context5.sent;

                            self._ctx.logger.debug('connect() calling _recvConnectResponse() for conn[%d]', conn.getId());

                            _context5.next = 16;
                            return self._recvConnectResponse(msg.data, conn);

                          case 16:
                            resolve();

                          case 17:
                          case "end":
                            return _context5.stop();
                        }
                      }
                    }, _callee5);
                  }));

                  return function (_x4, _x5) {
                    return _ref3.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function connect(_x3) {
        return _connect.apply(this, arguments);
      }

      return connect;
    }()
    /**
     * Close specified Connection to associated Edge Router.
     * 
     */

  }, {
    key: "close",
    value: function () {
      var _close = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(conn) {
        var _this3 = this;

        var self;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                self = this;
                return _context8.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(resolve, reject) {
                    var sequence, headers, msg;
                    return regeneratorRuntime.wrap(function _callee7$(_context7) {
                      while (1) {
                        switch (_context7.prev = _context7.next) {
                          case 0:
                            self._ctx.logger.debug('initiating Close to Edge Router [%s] for conn[%d]', _this3._edgeRouterHost, conn.getId());

                            sequence = conn.getAndIncrementSequence();
                            headers = [new Header(edge_protocol.header_id.ConnId, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: conn.getId()
                            }), new Header(edge_protocol.header_id.SeqHeader, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: sequence
                            })];

                            self._ctx.logger.debug('about to send Close to Edge Router [%s] for conn[%d]', conn.getChannel().getEdgeRouterHost(), conn.getId());

                            _context7.next = 6;
                            return self.sendMessage(edge_protocol.content_type.StateClosed, headers, self._options.network_session_token, {
                              conn: conn,
                              sequence: sequence
                            });

                          case 6:
                            msg = _context7.sent;

                            self._ctx.logger.debug('close() completed with response[%o]', msg);

                            conn.setState(edge_protocol.conn_state.Closed);
                            resolve();

                          case 10:
                          case "end":
                            return _context7.stop();
                        }
                      }
                    }, _callee7);
                  }));

                  return function (_x7, _x8) {
                    return _ref4.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function close(_x6) {
        return _close.apply(this, arguments);
      }

      return close;
    }()
    /**
     * Receives response from Edge 'Connect' message.
     * 
     */

  }, {
    key: "_recvConnectResponse",
    value: function () {
      var _recvConnectResponse2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(msg, expectedConn) {
        var buffer, contentTypeView, contentType, sequenceView, sequence, connId, conn;
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _context9.next = 2;
                return msg.buffer;

              case 2:
                buffer = _context9.sent;
                contentTypeView = new Int32Array(buffer, 4, 1);
                contentType = contentTypeView[0];
                sequenceView = new Int32Array(buffer, 8, 1);
                sequence = sequenceView[0];
                _context9.next = 9;
                return this._messageGetConnId(msg);

              case 9:
                connId = _context9.sent;
                conn = this._connections._getConnection(connId);
                throwIf(isUndefined(conn), formatMessage('Conn not found. Seeking connId { actual }', {
                  actual: connId
                }));

                if (!isEqual(conn.getId(), expectedConn.getId())) {
                  this._ctx.logger.error("_recvConnectResponse() actual conn[%d] expected conn[%d]", conn.getId(), expectedConn.getId());
                }

                this._ctx.logger.debug("ConnectResponse contentType[%d] seq[%d] received for conn[%d]", contentType, sequence, conn.getId());

                _context9.t0 = contentType;
                _context9.next = _context9.t0 === edge_protocol.content_type.StateClosed ? 17 : _context9.t0 === edge_protocol.content_type.StateConnected ? 20 : 34;
                break;

              case 17:
                this._ctx.logger.warn("conn[%d] failed to connect on ch[%d]", conn.getId(), this.getId());

                conn.setState(edge_protocol.conn_state.Closed);
                return _context9.abrupt("break", 35);

              case 20:
                if (!(conn.getState() == edge_protocol.conn_state.Connecting)) {
                  _context9.next = 32;
                  break;
                }

                this._ctx.logger.debug("conn[%d] connected", conn.getId());

                if (!conn.getEncrypted()) {
                  _context9.next = 29;
                  break;
                }

                _context9.next = 25;
                return this._establish_crypto(conn, msg);

              case 25:
                this._ctx.logger.debug("establish_crypto completed for conn[%d]", conn.getId());

                _context9.next = 28;
                return this._send_crypto_header(conn);

              case 28:
                this._ctx.logger.debug("send_crypto_header completed for conn[%d]", conn.getId());

              case 29:
                conn.setState(edge_protocol.conn_state.Connected);
                _context9.next = 33;
                break;

              case 32:
                if (conn.getState() == edge_protocol.conn_state.Closed || conn.getState() == edge_protocol.conn_state.Timedout) {
                  this._ctx.logger.warn("received connect reply for closed/timedout conne[%d]", conn.getId()); // ziti_disconnect(conn);

                }

              case 33:
                return _context9.abrupt("break", 35);

              case 34:
                this._ctx.logger.error("unexpected content_type[%d] conn[%d]", contentType, conn.getId());

              case 35:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function _recvConnectResponse(_x9, _x10) {
        return _recvConnectResponse2.apply(this, arguments);
      }

      return _recvConnectResponse;
    }()
    /**
     * 
     */

  }, {
    key: "_establish_crypto",
    value: function () {
      var _establish_crypto2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(conn, msg) {
        var result, peerKey, keypair, results;
        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                this._ctx.logger.debug("_establish_crypto(): entered for conn[%d]", conn.getId());

                _context10.next = 3;
                return this._messageGetBytesHeader(msg, edge_protocol.header_id.PublicKey);

              case 3:
                result = _context10.sent;
                peerKey = result.data;

                this._ctx.logger.debug("_establish_crypto(): peerKey is: ", peerKey);

                if (!(peerKey == undefined)) {
                  _context10.next = 10;
                  break;
                }

                this._ctx.logger.debug("_establish_crypto(): did not receive peer key. connection[%d] will not be encrypted: ", conn.getId());

                conn.setEncrypted(false);
                return _context10.abrupt("return");

              case 10:
                if (conn.getState() == edge_protocol.conn_state.Connecting) {
                  keypair = conn.getKeypair();
                  results = sodium.crypto_kx_client_session_keys(keypair.publicKey, keypair.privateKey, peerKey);
                  conn.setSharedRx(results.sharedRx);
                  conn.setSharedTx(results.sharedTx);
                } else {
                  this._ctx.logger.error("_establish_crypto(): cannot establish crypto while connection is in %d state: ", conn.getState());
                }

              case 11:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function _establish_crypto(_x11, _x12) {
        return _establish_crypto2.apply(this, arguments);
      }

      return _establish_crypto;
    }()
    /**
     * Receives response from Edge 'Data' message where we sent the Crypto header.
     * 
     */

  }, {
    key: "_recvCryptoResponse",
    value: function () {
      var _recvCryptoResponse2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(msg) {
        var connId, conn, buffer, headersLengthView, headersLength, bodyView, state_in;
        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                _context11.next = 2;
                return this._messageGetConnId(msg);

              case 2:
                connId = _context11.sent;

                this._ctx.logger.debug("_recvCryptoResponse(): entered for conn[%d]", connId);

                conn = this._connections._getConnection(connId);
                throwIf(isUndefined(conn), formatMessage('Conn not found. Seeking connId { actual }', {
                  actual: connId
                })); //

                _context11.next = 8;
                return msg.buffer;

              case 8:
                buffer = _context11.sent;
                headersLengthView = new Int32Array(buffer, 12, 1);
                headersLength = headersLengthView[0];
                bodyView = new Uint8Array(buffer, 20 + headersLength);
                state_in = sodium.crypto_secretstream_xchacha20poly1305_init_pull(bodyView, conn.getSharedRx());
                conn.setCrypt_i(state_in); // Indicate that subsequent sends on this connection should be encrypted

                conn.setEncrypted(true); // Unblock writes to the connection now that we have sent the crypto header

                conn.setCryptoEstablishComplete(true);

                this._ctx.logger.debug("_recvCryptoResponse(): setCryptoEstablishComplete for conn[%d]", connId);

              case 17:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11, this);
      }));

      function _recvCryptoResponse(_x13) {
        return _recvCryptoResponse2.apply(this, arguments);
      }

      return _recvCryptoResponse;
    }()
    /**
     * Remain in lazy-sleepy loop until specified connection's crypto handshake is complete.
     * 
     * @param {*} conn 
     */

  }, {
    key: "awaitConnectionCryptoEstablishComplete",
    value: function awaitConnectionCryptoEstablishComplete(conn) {
      return new Promise(function (resolve) {
        (function waitForCryptoEstablishComplete() {
          if (conn.getCryptoEstablishComplete()) {
            conn._ctx.logger.debug('Connection [%d] now Crypto-enabled with Edge Router', conn.getId());

            return resolve();
          }

          conn._ctx.logger.debug('awaitConnectionCryptoEstablishComplete() conn[%d] still not yet CryptoEstablishComplete', conn.getId());

          setTimeout(waitForCryptoEstablishComplete, 100);
        })();
      });
    }
    /**
     * 
     */

  }, {
    key: "_send_crypto_header",
    value: function () {
      var _send_crypto_header2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13(conn) {
        var self;
        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                self = this;
                return _context13.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12(resolve, reject) {
                    var results, sequence, headers, msg;
                    return regeneratorRuntime.wrap(function _callee12$(_context12) {
                      while (1) {
                        switch (_context12.prev = _context12.next) {
                          case 0:
                            results = sodium.crypto_secretstream_xchacha20poly1305_init_push(conn.getSharedTx());
                            conn.setCrypt_o(results);
                            sequence = conn.getAndIncrementSequence();
                            headers = [new Header(edge_protocol.header_id.ConnId, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: conn.getId()
                            }), new Header(edge_protocol.header_id.SeqHeader, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: sequence
                            })];

                            self._ctx.logger.debug('_send_crypto_header(): conn[%d] sending Data [%o]', conn.getId(), conn.getCrypt_o().header); // self.sendMessageNoWait( edge_protocol.content_type.Data, headers, conn.getCrypt_o().header, { conn: conn, sequence: sequence });


                            _context12.next = 7;
                            return self.sendMessage(edge_protocol.content_type.Data, headers, conn.getCrypt_o().header, {
                              conn: conn,
                              sequence: sequence
                            });

                          case 7:
                            msg = _context12.sent;

                            self._ctx.logger.debug('_send_crypto_header() calling _recvCryptoResponse() for conn[%d]', conn.getId());

                            _context12.next = 11;
                            return self._recvCryptoResponse(msg.data, conn);

                          case 11:
                            resolve();

                          case 12:
                          case "end":
                            return _context12.stop();
                        }
                      }
                    }, _callee12);
                  }));

                  return function (_x15, _x16) {
                    return _ref5.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function _send_crypto_header(_x14) {
        return _send_crypto_header2.apply(this, arguments);
      }

      return _send_crypto_header;
    }()
    /**
     * Write data over specified Edge Router connection.
     *
     * @returns {Promise}
     */

  }, {
    key: "write",
    value: function () {
      var _write = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14(conn, data) {
        var sequence, headers;
        return regeneratorRuntime.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                if (!isEqual(conn.getState(), edge_protocol.conn_state.Closed)) {
                  sequence = conn.getAndIncrementSequence();
                  headers = [new Header(edge_protocol.header_id.ConnId, {
                    headerType: edge_protocol.header_type.IntType,
                    headerData: conn.getId()
                  }), new Header(edge_protocol.header_id.SeqHeader, {
                    headerType: edge_protocol.header_type.IntType,
                    headerData: sequence
                  })];
                  this.sendMessageNoWait(edge_protocol.content_type.Data, headers, data, {
                    conn: conn,
                    sequence: sequence
                  });
                }

              case 1:
              case "end":
                return _context14.stop();
            }
          }
        }, _callee14, this);
      }));

      function write(_x17, _x18) {
        return _write.apply(this, arguments);
      }

      return write;
    }()
    /**
     * Sends message and waits for response.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @returns {Promise}
     */

  }, {
    key: "sendMessage",
    value: function sendMessage(contentType, headers, body) {
      var _this4 = this;

      var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var timeout = options.timeout !== undefined ? options.timeout : this._timeout;
      var messageId;

      if (!isUndefined(options.sequence)) {
        messageId = options.sequence;
      } else if (!isUndefined(this._sequence)) {
        messageId = this._sequence;
      }

      throwIf(isUndefined(messageId), formatMessage('messageId is undefined', {}));
      var messagesQueue = this._messages;
      var conn;

      if (!isUndefined(options.conn)) {
        conn = options.conn;
        messagesQueue = options.conn.getMessages();
      }

      this._ctx.logger.debug("send -> conn[%o] seq[%o] contentType[%o] body[%s]", conn ? conn.getId() : 'n/a', messageId, contentType, body ? body.toString() : 'n/a');

      return messagesQueue.create(messageId, function () {
        _this4._sendMarshaled(contentType, headers, body, options, messageId);
      }, timeout);
    }
    /**
     * Sends message and does not wait for response.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @returns {Promise}
     */

  }, {
    key: "sendMessageNoWait",
    value: function sendMessageNoWait(contentType, headers, body) {
      var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var timeout = options.timeout !== undefined ? options.timeout : this._timeout;
      var messageId = options.sequence || this._sequence; // this._ctx.logger.debug("send (no wait) -> conn[%o] seq[%o] contentType[%o]", 
      //   (options.conn ? options.conn.getId() : 'n/a'), 
      //   messageId, 
      //   contentType, 
      //   (body ? body.toString() : 'n/a'));

      this._ctx.logger.debug("send (no wait) -> conn[%o] seq[%o] contentType[%o] bodyLen[%o] ", options.conn ? options.conn.getId() : 'n/a', messageId, contentType, body ? body.length : 'n/a', body ? body.toString() : 'n/a');

      this._sendMarshaled(contentType, headers, body, options, messageId);
    }
    /**
     * Marshals message into binary wire format and sends to the Edge Router.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @param {int} messageId
     */

  }, {
    key: "_sendMarshaled",
    value: function _sendMarshaled(contentType, headers, body, options, messageId) {
      var dataToMarshal = body;

      if (contentType != edge_protocol.content_type.HelloType) {
        var connId;
        forEach(headers, function (header) {
          if (header.getId() == edge_protocol.header_id.ConnId) {
            connId = header.getData();
          }
        });
        throwIf(isUndefined(connId), formatMessage('Cannot find ConnId heder', {}));

        var conn = this._connections._getConnection(connId);

        throwIf(isUndefined(conn), formatMessage('Conn not found. Seeking connId { actual }', {
          actual: connId
        }));

        if (conn.getEncrypted() && conn.getCryptoEstablishComplete()) {
          // if connected to a service that has 'encryptionRequired'
          var _ref6 = [conn.getCrypt_o().state, conn.getCrypt_o().header],
              state_out = _ref6[0],
              header = _ref6[1];
          var encryptedData = sodium.crypto_secretstream_xchacha20poly1305_push(state_out, body, null, sodium.crypto_secretstream_xchacha20poly1305_TAG_MESSAGE);
          dataToMarshal = encryptedData;
        }
      }

      var wireData = this._marshalMessage(contentType, headers, dataToMarshal, options, messageId);

      this._ctx.logger.trace("_sendMarshaled -> wireDataLen[%o] ", wireData.byteLength);

      this._dumpHeaders(' -> ', wireData); // Inject the listener if specified


      if (options.listener !== undefined) {
        this._zws.onMessage.addOnceListener(options.listener, this);
      } // If connected to a WS edge router


      if (isEqual(this._callerId, "ws:")) {
        this._tlsConn.prepare(wireData);
      } else {
        this._zws.send(wireData);
      }
    }
    /**
     * Marshals message into binary wire format.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @returns {byte[]}   
     */

  }, {
    key: "_marshalMessage",
    value: function _marshalMessage(contentType, headers, body, options, messageId) {
      // wire-protocol: message-section
      var buffer_message_section = new ArrayBuffer(4 // Version
      + 4 // ContentType  (offset 4)
      + 4 // Sequence     (offset 8)
      + 4 // hdrs-len     (offset 12)
      + 4 // body-len     (offset 16)
      ); // wire-protocol: Version

      var view_message_section = new Uint8Array(buffer_message_section);
      view_message_section.set(edge_protocol.VERSION, 0 // Offset 0
      );
      var bytes = new Buffer(4); // wire-protocol: ContentType

      bytes.writeUInt32LE(contentType, 0);
      view_message_section.set(bytes, 4 // Offset 4
      );
      bytes = new Buffer(4); // wire-protocol: Sequence

      bytes.writeInt32LE(messageId, 0);
      view_message_section.set(bytes, 8 // Offset 8
      );
      bytes = new Buffer(4);
      var hdrsLen = utils.sumBy(headers, function (header) {
        return header.getLength();
      }); // wire-protocol: hdrs-len

      bytes.writeInt32LE(hdrsLen, 0);
      view_message_section.set(bytes, 12 // Offset 12
      );
      bytes = new Buffer(4); // wire-protocol: body-len

      var bodyLength = 0;

      if (!isNull(body)) {
        bodyLength = body.length;
      }

      bytes.writeUInt32LE(bodyLength, 0);
      view_message_section.set(bytes, 16 // Offset 16
      ); // wire-protocol: headers

      var buffer_headers_section = new ArrayBuffer(hdrsLen);
      var view_headers_section = new Uint8Array(buffer_headers_section);
      var view_headers_section_offset = 0;
      forEach(headers, function (header) {
        view_headers_section.set(header.getBytesForWire(), view_headers_section_offset);
        view_headers_section_offset += header.getLength();
      }); // wire-protocol: body

      var buffer_body_section = new ArrayBuffer(bodyLength);

      if (bodyLength > 0) {
        var view_body_section = new Uint8Array(buffer_body_section);
        var body_bytes;

        if (typeof body === 'string') {
          body_bytes = utils.toUTF8Array(body);
        } else {
          body_bytes = body;
        }

        var bytesBuffer = Buffer.from(body_bytes);
        view_body_section.set(bytesBuffer, 0);
      } // Put it all together


      this._ctx.logger.trace("_marshalMessage -> buffer_message_section Len[%o] ", buffer_message_section.byteLength);

      this._ctx.logger.trace("_marshalMessage -> buffer_headers_section Len[%o] ", buffer_headers_section.byteLength);

      this._ctx.logger.trace("_marshalMessage -> buffer_body_section Len[%o] ", buffer_body_section.byteLength);

      var buffer_combined = utils.appendBuffer(buffer_message_section, buffer_headers_section);
      buffer_combined = utils.appendBuffer(buffer_combined, buffer_body_section);
      var view_combined = new Uint8Array(buffer_combined);
      return view_combined.buffer;
    }
    /**
     * Receives a send event from the Websocket.
     * 
     * @param {*} data 
     */

  }, {
    key: "_recvSend",
    value: function () {
      var _recvSend2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee15(data) {
        return regeneratorRuntime.wrap(function _callee15$(_context15) {
          while (1) {
            switch (_context15.prev = _context15.next) {
              case 0:
                if (!isUndefined(this._zws)) {
                  if (!isNull(this._zws._ws)) {
                    this._ctx.logger.debug('_recvSend -> sentLen[%o] bufferedLen[%o]', data.byteLength, this._zws._ws.bufferedAmount);
                  }
                }

              case 1:
              case "end":
                return _context15.stop();
            }
          }
        }, _callee15, this);
      }));

      function _recvSend(_x19) {
        return _recvSend2.apply(this, arguments);
      }

      return _recvSend;
    }()
    /**
     * Receives a close event from the Websocket.
     * 
     * @param {*} data 
     */

  }, {
    key: "_recvClose",
    value: function () {
      var _recvClose2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee16(data) {
        return regeneratorRuntime.wrap(function _callee16$(_context16) {
          while (1) {
            switch (_context16.prev = _context16.next) {
              case 0:
                this._ctx.closeChannelByEdgeRouter(this._edgeRouterHost);

              case 1:
              case "end":
                return _context16.stop();
            }
          }
        }, _callee16, this);
      }));

      function _recvClose(_x20) {
        return _recvClose2.apply(this, arguments);
      }

      return _recvClose;
    }()
    /**
     * Receives a message from the Edge Router.
     * 
     * @param {*} data 
     */

  }, {
    key: "_recvFromWire",
    value: function () {
      var _recvFromWire2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee17(data) {
        var buffer, tlsBinaryString;
        return regeneratorRuntime.wrap(function _callee17$(_context17) {
          while (1) {
            switch (_context17.prev = _context17.next) {
              case 0:
                _context17.next = 2;
                return data.arrayBuffer();

              case 2:
                buffer = _context17.sent;

                this._ctx.logger.debug("_recvFromWire <- data len[%o]", buffer.byteLength);

                tlsBinaryString = Buffer.from(buffer).toString('binary');

                this._tlsConn.process(tlsBinaryString);

              case 6:
              case "end":
                return _context17.stop();
            }
          }
        }, _callee17, this);
      }));

      function _recvFromWire(_x21) {
        return _recvFromWire2.apply(this, arguments);
      }

      return _recvFromWire;
    }()
    /**
     * Receives un-encrypted binary data from the Edge Router (which is either an entire edge protocol message, or a fragment thereof)
     * 
     * @param ArrayBuffer data 
     */

  }, {
    key: "_recvFromWireAfterDecrypt",
    value: function () {
      var _recvFromWireAfterDecrypt2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee18(ch, data) {
        var buffer, dataView, versionView, headersLengthView, headersLength, bodyLengthView, bodyLength, msgLength;
        return regeneratorRuntime.wrap(function _callee18$(_context18) {
          while (1) {
            switch (_context18.prev = _context18.next) {
              case 0:
                buffer = data;

                if (!isUndefined(ch._partialMessage)) {
                  // if we are awaiting rest of a partial msg to arrive, append this chunk onto the end, then proceed
                  dataView = new Uint8Array(data);
                  ch._partialMessage = utils.concatTypedArrays(ch._partialMessage, dataView);
                  buffer = ch._partialMessage.buffer.slice(0);
                }

                versionView = new Uint8Array(buffer, 0, 4);
                throwIf(!isEqual(versionView[0], ch._view_version[0]), formatMessage('Unexpected message version. Got { actual }, expected { expected }', {
                  actual: versionView[0],
                  expected: ch._view_version[0]
                }));
                headersLengthView = new Int32Array(buffer, 12, 1);
                headersLength = headersLengthView[0];
                bodyLengthView = new Int32Array(buffer, 16, 1);
                bodyLength = bodyLengthView[0];
                msgLength = 20 + headersLength + bodyLength;

                if (isEqual(msgLength, buffer.byteLength)) {
                  // if entire edge message is present, proceed with unmarshaling effort
                  ch._partialMessage = undefined;

                  ch._tryUnmarshal(buffer);
                } else {
                  ch._partialMessage = new Uint8Array(buffer);

                  if (buffer.byteLength > 20) {
                    // if we have a portion of the data section of the edge message 
                    ch._ctx.logger.debug("Only [%o] of msgLength [%o] received; will await rest of fragments before unmarshaling", buffer.byteLength, msgLength);
                  }
                }

              case 10:
              case "end":
                return _context18.stop();
            }
          }
        }, _callee18);
      }));

      function _recvFromWireAfterDecrypt(_x22, _x23) {
        return _recvFromWireAfterDecrypt2.apply(this, arguments);
      }

      return _recvFromWireAfterDecrypt;
    }()
    /**
     * Unmarshals binary from the wire into a message
     * 
     * @param {*} data 
     */

  }, {
    key: "_tryUnmarshal",
    value: function () {
      var _tryUnmarshal2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee19(data) {
        var buffer, versionView, contentTypeView, contentType, sequenceView, responseSequence, headersLengthView, headersLength, bodyLengthView, bodyLength, bodyView, connId, conn, replyForView, haveResponseSequence, release, result, _result, _result2, _connId, unencrypted_data, _ref7, m1, tag1, len, dbgStr, dataCallback;

        return regeneratorRuntime.wrap(function _callee19$(_context19) {
          while (1) {
            switch (_context19.prev = _context19.next) {
              case 0:
                buffer = data;
                versionView = new Uint8Array(buffer, 0, 4);
                throwIf(!isEqual(versionView[0], this._view_version[0]), formatMessage('Unexpected message version. Got { actual }, expected { expected }', {
                  actual: versionView[0],
                  expected: this._view_version[0]
                }));
                contentTypeView = new Int32Array(buffer, 4, 1);
                contentType = contentTypeView[0];
                sequenceView = new Int32Array(buffer, 8, 1);

                this._ctx.logger.trace("recv <- contentType[%o] seq[%o]", contentType, sequenceView[0]);

                responseSequence = sequenceView[0];
                headersLengthView = new Int32Array(buffer, 12, 1);
                headersLength = headersLengthView[0];

                this._ctx.logger.trace("recv <- headersLength[%o]", headersLength);

                bodyLengthView = new Int32Array(buffer, 16, 1);
                bodyLength = bodyLengthView[0];

                this._ctx.logger.trace("recv <- bodyLength[%o]", bodyLength);

                this._dumpHeaders(' <- ', buffer);

                bodyView = new Uint8Array(buffer, 20 + headersLength);
                haveResponseSequence = false;
                _context19.next = 19;
                return this._mutex.acquire();

              case 19:
                release = _context19.sent;

                if (!(contentType == edge_protocol.content_type.Data)) {
                  _context19.next = 32;
                  break;
                }

                _context19.next = 23;
                return this._messageGetConnId(data);

              case 23:
                connId = _context19.sent;

                if (isUndefined(connId)) {
                  _context19.next = 32;
                  break;
                }

                conn = this._connections._getConnection(connId);

                if (isUndefined(conn)) {
                  _context19.next = 32;
                  break;
                }

                if (!isEqual(conn.getState(), edge_protocol.conn_state.Connecting)) {
                  _context19.next = 32;
                  break;
                }

                _context19.next = 30;
                return this._messageGetBytesHeader(data, edge_protocol.header_id.SeqHeader);

              case 30:
                result = _context19.sent;

                if (!isUndefined(result)) {
                  replyForView = new Int32Array(result.data, 0, 1);
                  responseSequence = replyForView[0];
                  haveResponseSequence = true;

                  this._ctx.logger.debug("recv <- ReplyFor[%o] (should be for the crypto_header response)", responseSequence);
                }

              case 32:
                if (!(!haveResponseSequence && contentType >= edge_protocol.content_type.StateConnected)) {
                  _context19.next = 54;
                  break;
                }

                _context19.next = 35;
                return this._messageGetBytesHeader(data, edge_protocol.header_id.ReplyFor);

              case 35:
                _result = _context19.sent;

                if (isUndefined(_result)) {
                  _context19.next = 42;
                  break;
                }

                replyForView = new DataView(_result.data.buffer.slice(_result.data.byteOffset, _result.data.byteLength + _result.data.byteOffset));
                responseSequence = replyForView.getInt32(0, true); // second parameter truethy == want little endian;

                this._ctx.logger.trace("recv <- ReplyFor[%o]", responseSequence);

                _context19.next = 54;
                break;

              case 42:
                if (!(isEqual(contentType, edge_protocol.content_type.Data) && isEqual(bodyLength, 0))) {
                  _context19.next = 51;
                  break;
                }

                _context19.next = 45;
                return this._messageGetBytesHeader(data, edge_protocol.header_id.SeqHeader);

              case 45:
                _result2 = _context19.sent;
                replyForView = new Int32Array(_result2.data, 0, 1);
                responseSequence = replyForView[0];

                this._ctx.logger.trace("recv <- Close Response For [%o]", responseSequence);

                _context19.next = 54;
                break;

              case 51:
                this._ctx.logger.trace("recv <- ReplyFor[%o]", 'n/a');

                responseSequence--;

                this._ctx.logger.trace("reducing seq by 1 to [%o]", responseSequence);

              case 54:
                if (!(contentType >= edge_protocol.content_type.Connect && isUndefined(conn))) {
                  _context19.next = 61;
                  break;
                }

                _context19.next = 57;
                return this._messageGetConnId(data);

              case 57:
                _connId = _context19.sent;
                throwIf(isUndefined(_connId), formatMessage('Cannot find ConnId header', {}));
                conn = this._connections._getConnection(_connId);
                throwIf(isUndefined(conn), formatMessage('Conn not found. Seeking connId { actual }', {
                  actual: _connId
                }));

              case 61:
                /**
                 *  Data msgs might need to be decrypted before passing along
                 */
                if (contentType == edge_protocol.content_type.Data) {
                  if (bodyLength > 0) {
                    if (conn.getEncrypted() && conn.getCryptoEstablishComplete()) {
                      // if connected to a service that has 'encryptionRequired'
                      unencrypted_data = sodium.crypto_secretstream_xchacha20poly1305_pull(conn.getCrypt_i(), bodyView);

                      if (!unencrypted_data) {
                        this._ctx.logger.error("crypto_secretstream_xchacha20poly1305_pull failed. bodyLength[%d]", bodyLength);
                      }

                      try {
                        _ref7 = [sodium.to_string(unencrypted_data.message), unencrypted_data.tag], m1 = _ref7[0], tag1 = _ref7[1];
                        len = m1.length;

                        if (len > 2000) {
                          len = 2000;
                        }

                        this._ctx.logger.trace("recv <- unencrypted_data (first 2000): %s", m1.substring(0, len)); //


                        dbgStr = m1.substring(0, len);

                        this._ctx.logger.trace("recv <- data (first 2000): %s", dbgStr);
                      } catch (e) {}

                      bodyView = unencrypted_data.message;
                    } else {
                      /* debug...
                      let len = bodyView.length;
                      if (len > 2000) {
                        len = 2000;
                      }
                      let dbgStr = String.fromCharCode.apply(null, bodyView).substring(0, len);
                      this._ctx.logger.debug("recv <- data (first 2000): %s", dbgStr);
                      */
                      //temp debugging
                      // if (dbgStr.includes("var openMe = (window.parent")) {
                      //   let str = String.fromCharCode.apply(null, bodyView).substring(0, bodyView.length);
                      //   // str = str.replace('var openMe = (window.parent', 'debugger; var openMe = (window.parent');
                      //   if (str.indexOf( '/api/extplugins/config' ) !== -1) {
                      //     debugger
                      //   }
                      //   this._ctx.logger.debug("============== DEBUG INJECT: %s", str);
                      //   bodyView = new TextEncoder("utf-8").encode(str);
                      // }
                    }
                  } // 


                  dataCallback = conn.getDataCallback();

                  if (!isUndefined(dataCallback)) {
                    this._ctx.logger.trace("recv <- contentType[%o] seq[%o] passing body to dataCallback", contentType, sequenceView[0]);

                    dataCallback(conn, bodyView);
                  }
                }

                this._ctx.logger.trace("recv <- response body: ", bodyView);

                this._tryHandleResponse(conn, responseSequence, {
                  channel: this,
                  data: bodyView
                });

                release();

              case 65:
              case "end":
                return _context19.stop();
            }
          }
        }, _callee19, this);
      }));

      function _tryUnmarshal(_x24) {
        return _tryUnmarshal2.apply(this, arguments);
      }

      return _tryUnmarshal;
    }()
    /**
     * 
     */

  }, {
    key: "_tryHandleResponse",
    value: function _tryHandleResponse(conn, responseSequence, data) {
      if (!isUndefined(conn)) {
        var socket = conn.getSocket();

        if (!isUndefined(socket)) {
          if (socket.isWebSocket) {
            return;
          }
        }
      }

      var messagesQueue = this._messages;

      if (!isUndefined(conn)) {
        messagesQueue = conn.getMessages();
      }

      this._ctx.logger.trace("_tryHandleResponse():  conn[%d] seq[%d]", conn ? conn.getId() : 'n/a', responseSequence);

      if (!isNull(responseSequence)) {
        messagesQueue.resolve(responseSequence, data);
      } else {
        debugger;
      }
    }
    /**
     * 
     */

  }, {
    key: "_dumpHeaders",
    value: function () {
      var _dumpHeaders2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee20(pfx, buffer) {
        var headersView, headersLength, headersOffset, ndx, view, _headerId, _headerDataLength, _headerData, connId, _buffer;

        return regeneratorRuntime.wrap(function _callee20$(_context20) {
          while (1) {
            switch (_context20.prev = _context20.next) {
              case 0:
                headersView = new Int32Array(buffer, 12, 1);
                headersLength = headersView[0];
                headersOffset = 16 + 4;
                ndx = 0;
                view = new DataView(buffer);

                this._ctx.logger.trace("_dumpHeaders: " + pfx + "vv----------------------------------");

                for (; ndx < headersLength;) {
                  _headerId = view.getInt32(headersOffset + ndx, true);
                  ndx += 4;
                  _headerDataLength = view.getInt32(headersOffset + ndx, true);
                  ndx += 4;
                  _headerData = new Uint8Array(buffer, headersOffset + ndx, _headerDataLength);
                  ndx += _headerDataLength;
                  connId = 'n/a';

                  if (isEqual(_headerId, edge_protocol.header_id.ConnId)) {
                    _buffer = Buffer.from(_headerData);
                    connId = _buffer.readUIntLE(0, _headerDataLength);
                  }

                  this._ctx.logger.trace("headerId[%d] conn[%d] dataLength[%d] data[%o]", _headerId, connId, _headerDataLength, _headerData);
                }

                this._ctx.logger.trace("_dumpHeaders: " + pfx + "^^----------------------------------");

              case 8:
              case "end":
                return _context20.stop();
            }
          }
        }, _callee20, this);
      }));

      function _dumpHeaders(_x25, _x26) {
        return _dumpHeaders2.apply(this, arguments);
      }

      return _dumpHeaders;
    }()
    /**
     * 
     */

  }, {
    key: "_findHeader",
    value: function () {
      var _findHeader2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee21(msg, headerToFind) {
        var buffer, headersView, headersLength, headersOffset, ndx, view, _headerId, _headerDataLength, _headerData, result;

        return regeneratorRuntime.wrap(function _callee21$(_context21) {
          while (1) {
            switch (_context21.prev = _context21.next) {
              case 0:
                if (isUndefined(msg.arrayBuffer)) {
                  _context21.next = 6;
                  break;
                }

                _context21.next = 3;
                return msg.arrayBuffer();

              case 3:
                buffer = _context21.sent;
                _context21.next = 13;
                break;

              case 6:
                if (isUndefined(msg.buffer)) {
                  _context21.next = 12;
                  break;
                }

                _context21.next = 9;
                return msg.buffer;

              case 9:
                buffer = _context21.sent;
                _context21.next = 13;
                break;

              case 12:
                buffer = msg;

              case 13:
                headersView = new Int32Array(buffer, 12, 1);
                headersLength = headersView[0];
                headersOffset = 16 + 4;
                ndx = 0;
                view = new DataView(buffer);

              case 18:
                if (!(ndx < headersLength)) {
                  _context21.next = 30;
                  break;
                }

                _headerId = view.getInt32(headersOffset + ndx, true);
                ndx += 4;
                _headerDataLength = view.getInt32(headersOffset + ndx, true);
                ndx += 4;
                _headerData = new Uint8Array(buffer, headersOffset + ndx, _headerDataLength);
                ndx += _headerDataLength;

                if (!(_headerId == headerToFind)) {
                  _context21.next = 28;
                  break;
                }

                result = {
                  dataLength: _headerDataLength,
                  data: _headerData
                };
                return _context21.abrupt("return", result);

              case 28:
                _context21.next = 18;
                break;

              case 30:
                return _context21.abrupt("return", undefined);

              case 31:
              case "end":
                return _context21.stop();
            }
          }
        }, _callee21);
      }));

      function _findHeader(_x27, _x28) {
        return _findHeader2.apply(this, arguments);
      }

      return _findHeader;
    }()
    /**
     * 
     */

  }, {
    key: "_messageGetBytesHeader",
    value: function () {
      var _messageGetBytesHeader2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee22(msg, headerToFind) {
        return regeneratorRuntime.wrap(function _callee22$(_context22) {
          while (1) {
            switch (_context22.prev = _context22.next) {
              case 0:
                _context22.next = 2;
                return this._findHeader(msg, headerToFind);

              case 2:
                return _context22.abrupt("return", _context22.sent);

              case 3:
              case "end":
                return _context22.stop();
            }
          }
        }, _callee22, this);
      }));

      function _messageGetBytesHeader(_x29, _x30) {
        return _messageGetBytesHeader2.apply(this, arguments);
      }

      return _messageGetBytesHeader;
    }()
    /**
     * 
     */

  }, {
    key: "_messageGetConnId",
    value: function () {
      var _messageGetConnId2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee23(msg) {
        var results, length, buffer, connId;
        return regeneratorRuntime.wrap(function _callee23$(_context23) {
          while (1) {
            switch (_context23.prev = _context23.next) {
              case 0:
                _context23.next = 2;
                return this._findHeader(msg, edge_protocol.header_id.ConnId);

              case 2:
                results = _context23.sent;
                throwIf(results == undefined, formatMessage('No ConnId header found'));
                length = results.data.length;
                buffer = Buffer.from(results.data);
                connId = buffer.readUIntLE(0, length);
                return _context23.abrupt("return", connId);

              case 8:
              case "end":
                return _context23.stop();
            }
          }
        }, _callee23, this);
      }));

      function _messageGetConnId(_x31) {
        return _messageGetConnId2.apply(this, arguments);
      }

      return _messageGetConnId;
    }()
  }, {
    key: "getConnection",
    value: function getConnection(id) {
      return this._connections._getConnection(id);
    }
  }, {
    key: "getSocket",
    value: function getSocket() {
      return this._socket;
    }
  }, {
    key: "setSocket",
    value: function setSocket(socket) {
      this._socket = socket;
    }
  }, {
    key: "getDataCallback",
    value: function getDataCallback() {
      return this._dataCallback;
    }
  }, {
    key: "setDataCallback",
    value: function setDataCallback(fn) {
      this._dataCallback = fn;
    }
  }, {
    key: "getEncrypted",
    value: function getEncrypted() {
      return this._encrypted;
    }
  }, {
    key: "setEncrypted",
    value: function setEncrypted(encrypted) {
      this._encrypted = encrypted;
    }
  }, {
    key: "getCryptoEstablishComplete",
    value: function getCryptoEstablishComplete() {
      return this._cryptoEstablishComplete;
    }
  }, {
    key: "setCryptoEstablishComplete",
    value: function setCryptoEstablishComplete(complete) {
      this._cryptoEstablishComplete = complete;
    }
  }, {
    key: "getKeypair",
    value: function getKeypair() {
      return this._keypair;
    }
  }, {
    key: "setKeypair",
    value: function setKeypair(keypair) {
      this._keypair = keypair;
    }
  }, {
    key: "getSharedRx",
    value: function getSharedRx() {
      return this._sharedRx;
    }
  }, {
    key: "setSharedRx",
    value: function setSharedRx(sharedRx) {
      this._sharedRx = sharedRx;
    }
  }, {
    key: "getSharedTx",
    value: function getSharedTx() {
      return this._sharedTx;
    }
  }, {
    key: "setSharedTx",
    value: function setSharedTx(sharedTx) {
      this._sharedTx = sharedTx;
    }
  }, {
    key: "getCrypt_o",
    value: function getCrypt_o() {
      return this._crypt_o;
    }
  }, {
    key: "setCrypt_o",
    value: function setCrypt_o(crypt_o) {
      this._crypt_o = crypt_o;
    }
  }, {
    key: "getCrypt_i",
    value: function getCrypt_i() {
      return this._crypt_i;
    }
  }, {
    key: "setCrypt_i",
    value: function setCrypt_i(crypt_i) {
      this._crypt_i = crypt_i;
    }
  }]);

  return ZitiChannel;
}();