"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array-buffer.slice.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.typed-array.uint8-array.js");

require("core-js/modules/es.typed-array.copy-within.js");

require("core-js/modules/es.typed-array.every.js");

require("core-js/modules/es.typed-array.fill.js");

require("core-js/modules/es.typed-array.filter.js");

require("core-js/modules/es.typed-array.find.js");

require("core-js/modules/es.typed-array.find-index.js");

require("core-js/modules/es.typed-array.for-each.js");

require("core-js/modules/es.typed-array.includes.js");

require("core-js/modules/es.typed-array.index-of.js");

require("core-js/modules/es.typed-array.iterator.js");

require("core-js/modules/es.typed-array.join.js");

require("core-js/modules/es.typed-array.last-index-of.js");

require("core-js/modules/es.typed-array.map.js");

require("core-js/modules/es.typed-array.reduce.js");

require("core-js/modules/es.typed-array.reduce-right.js");

require("core-js/modules/es.typed-array.reverse.js");

require("core-js/modules/es.typed-array.set.js");

require("core-js/modules/es.typed-array.slice.js");

require("core-js/modules/es.typed-array.some.js");

require("core-js/modules/es.typed-array.sort.js");

require("core-js/modules/es.typed-array.subarray.js");

require("core-js/modules/es.typed-array.to-locale-string.js");

require("core-js/modules/es.typed-array.to-string.js");

require("core-js/modules/es.array-buffer.constructor.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/web.timers.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.typed-array.int32-array.js");

require("core-js/modules/es.data-view.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var flatOptions = require('flat-options');

var PromiseController = require('promise-controller');

var isNull = require('lodash.isnull');

var forEach = require('lodash.foreach');

var isEqual = require('lodash.isequal');

var isUndefined = require('lodash.isundefined');

var formatMessage = require('format-message');

var sodium = require('libsodium-wrappers');

var defaultOptions = require('./controller-channel-options');

var edge_protocol = require('./protocol');

var zitiConstants = require('../constants');

var ZitiConnections = require('./connections');

var ZitiWebSocket = require('../websocket/websocket');

var Header = require('./header');

var Messages = require('./messages');

var _require = require('../utils/throwif'),
    throwIf = _require.throwIf;

var utils = require('../utils/utils');

var ZitiTLSConnection = require('./tls-connection');
/**
 * @typicalname controllerChannel
 */


module.exports = /*#__PURE__*/function () {
  /**
   *
   * @param {Options} [options]
   */
  function ZitiControllerChannel(options) {
    _classCallCheck(this, ZitiControllerChannel);

    this._options = flatOptions(options, defaultOptions);
    this._ctx = this._options.ctx;
    this._id = this._ctx.getNextChannelId();
    this._data = this._options.data;
    this._timeout = this._options.timeout;
    this._helloTimeout = this._options.helloTimeout;
    this._controllerHost = this._options.controllerHost;
    this._controllerPort = this._options.controllerPort;
    this._connections = new ZitiConnections();
    this._zws = new ZitiWebSocket('ws://' + this._controllerHost + ':' + this._controllerPort + '/ws', {
      ctx: this._ctx
    });

    this._zws.onMessage.addListener(this._recvFromWire, this);

    this._messages = new Messages({
      ctx: this._ctx,
      channel: this
    });
    this._view_version = new Uint8Array(new ArrayBuffer(4));

    this._view_version.set(edge_protocol.VERSION, 0);

    this._msgSeq = 1;
  }

  _createClass(ZitiControllerChannel, [{
    key: "getCtx",
    value: function getCtx() {
      return this._ctx;
    }
  }, {
    key: "getId",
    value: function getId() {
      return this._id;
    }
  }, {
    key: "getData",
    value: function getData() {
      return this._data;
    }
  }, {
    key: "getAndIncrementSequence",
    value: function getAndIncrementSequence() {
      return this._msgSeq++;
    }
  }, {
    key: "getEdgeRouterHost",
    value: function getEdgeRouterHost() {
      return this._edgeRouterHost;
    }
    /**
     * 
     */

  }, {
    key: "helloCompletedTimestamp",
    get: function get() {
      return this._helloCompletedTimestamp;
    }
    /**
     * Has this Channel already completed the 'hello' sequence?
     *
     * @returns {Boolean}
     */

  }, {
    key: "isHelloCompleted",
    get: function get() {
      return Boolean(this._helloCompleted);
    }
  }, {
    key: "_createHelloController",
    value: function _createHelloController() {
      var helloTimeout = this._helloTimeout || this._timeout;
      this._helloing = new PromiseController({
        timeout: helloTimeout,
        timeoutReason: "Can't complete 'Hello' within allowed timeout: ".concat(helloTimeout, " ms.")
      });
    }
    /**
     * Remain in lazy-sleepy loop until tlsConn has completed its TLS handshake.
     * 
     */

  }, {
    key: "awaitTLSHandshakeComplete",
    value: function () {
      var _awaitTLSHandshakeComplete = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
        var self;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                self = this;
                return _context.abrupt("return", new Promise(function (resolve) {
                  (function waitForTLSHandshakeComplete() {
                    if (!self._tlsConn.isTLSHandshakeComplete()) {
                      self._ctx.logger.trace('awaitTLSHandshakeComplete() tlsConn for ch[%d] TLS handshake still not complete', self.getId());

                      setTimeout(waitForTLSHandshakeComplete, 100);
                    } else {
                      self._ctx.logger.trace('tlsConn for ch[%d] TLS handshake is now complete', self.getId());

                      return resolve();
                    }
                  })();
                }));

              case 2:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function awaitTLSHandshakeComplete() {
        return _awaitTLSHandshakeComplete.apply(this, arguments);
      }

      return awaitTLSHandshakeComplete;
    }()
    /**
     * Open websocket to Controller and do TLS handshake.
     * 
     */

  }, {
    key: "connect",
    value: function () {
      var _connect = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
        var _this = this;

        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return this._zws.open();

              case 2:
                // if (this.isHelloCompleted) {
                //   this._ctx.logger.debug('Connect was previously completed');
                //   return new Promise( async (resolve) => {
                //     resolve( {channel: this, data: null});
                //   });
                // }
                this._tlsConn = new ZitiTLSConnection({
                  type: 'controller',
                  ctx: this._ctx,
                  ws: this._zws,
                  ch: this,
                  datacb: this._recvFromWireAfterDecrypt
                });

                this._tlsConn.create();

                this._ctx.logger.debug('initiating TLS handshake');

                this._tlsConn.handshake();

                _context3.next = 8;
                return this.awaitTLSHandshakeComplete();

              case 8:
                this._ctx.logger.debug('TLS handshake completed');

                return _context3.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(resolve) {
                    return regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            resolve({
                              channel: _this,
                              data: null
                            });

                          case 1:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  return function (_x) {
                    return _ref.apply(this, arguments);
                  };
                }()));

              case 10:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function connect() {
        return _connect.apply(this, arguments);
      }

      return connect;
    }()
    /**
     * 
     * 
     */

  }, {
    key: "request",
    value: function () {
      var _request = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(options) {
        var _this2 = this;

        var self;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                self = this;
                return _context5.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(resolve, reject) {
                    var sequence, headers, msg;
                    return regeneratorRuntime.wrap(function _callee4$(_context4) {
                      while (1) {
                        switch (_context4.prev = _context4.next) {
                          case 0:
                            self._ctx.logger.debug('initiating request to Controller: [%o]', options);

                            sequence = _this2.getAndIncrementSequence();
                            headers = [new Header(edge_protocol.header_id.SeqHeader, {
                              headerType: edge_protocol.header_type.IntType,
                              headerData: sequence
                            })];
                            _context4.next = 5;
                            return self.sendMessage(edge_protocol.content_type.Data, headers, options, {
                              sequence: sequence
                            });

                          case 5:
                            msg = _context4.sent;
                            resolve(msg.data);

                          case 7:
                          case "end":
                            return _context4.stop();
                        }
                      }
                    }, _callee4);
                  }));

                  return function (_x3, _x4) {
                    return _ref2.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));

      function request(_x2) {
        return _request.apply(this, arguments);
      }

      return request;
    }()
    /**
     * Do echo test with Controller.
     * 
     */

  }, {
    key: "echo",
    value: function () {
      var _echo = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(wireData) {
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                this._tlsConn.prepare(wireData);

              case 1:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function echo(_x5) {
        return _echo.apply(this, arguments);
      }

      return echo;
    }() // /**
    //  * Connect specified Connection to associated Edge Router.
    //  * 
    //  */
    // async connect(conn) {
    //   const self = this;
    //   return new Promise( async (resolve, reject) => {
    //     self._ctx.logger.debug('initiating Connect to Edge Router [%s] for conn[%d]', this._edgeRouterHost, conn.getId());
    //     await sodium.ready;
    //     let keypair = sodium.crypto_kx_keypair();
    //     conn.setKeypair(keypair);
    //     let sequence = this.getAndIncrementSequence();
    //     let headers = [
    //       new Header( edge_protocol.header_id.ConnId, {
    //         headerType: edge_protocol.header_type.IntType,
    //         headerData: conn.getId()
    //       }),
    //       new Header( edge_protocol.header_id.SeqHeader, { 
    //         headerType: edge_protocol.header_type.IntType, 
    //         headerData: 0 
    //       }),
    //       new Header( edge_protocol.header_id.PublicKey, { 
    //         headerType: edge_protocol.header_type.Uint8ArrayType, 
    //         headerData: keypair.publicKey
    //       })
    //     ];
    //     conn.setState(edge_protocol.conn_state.Connecting);
    //     self._ctx.logger.debug('about to send Connect to Edge Router [%s] for conn[%d]', conn.getChannel().getEdgeRouterHost(), conn.getId());
    //     let msg = await self.sendMessage( edge_protocol.content_type.Connect, headers, self._options.network_session_token, { 
    //         conn: conn,
    //         sequence: sequence,
    //       } 
    //     );
    //     self._ctx.logger.debug('connect() calling _recvConnectResponse() for conn[%d]', conn.getId());
    //     await self._recvConnectResponse(msg.data, conn);
    //     resolve();
    //   });
    // }

    /**
     * Sends message and waits for response.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @returns {Promise}
     */

  }, {
    key: "sendMessage",
    value: function sendMessage(contentType, headers, body) {
      var _this3 = this;

      var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var timeout = options.timeout !== undefined ? options.timeout : this._timeout;
      var messageId;

      if (!isUndefined(options.sequence)) {
        messageId = options.sequence;
      } else if (!isUndefined(this._sequence)) {
        messageId = this._sequence;
      }

      throwIf(isUndefined(messageId), formatMessage('messageId is undefined', {})); // let messagesQueue = this._messages;
      // let conn;
      // if (!isUndefined(options.conn)) {
      //   conn = options.conn;
      //   messagesQueue = options.conn.getMessages();
      // }

      this._ctx.logger.debug("send -> seq[%o] contentType[%o] body[%s]", messageId, contentType, body ? body.toString() : 'n/a');

      return this._messages.create(messageId, function () {
        _this3._sendMarshaled(contentType, headers, body, options, messageId);
      }, timeout);
    }
    /**
     * Marshals message into binary wire format and sends to the Controller.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @param {int} messageId
     */

  }, {
    key: "_sendMarshaled",
    value: function _sendMarshaled(contentType, headers, body, options, messageId) {
      var dataToMarshal = body;

      var wireData = this._marshalMessage(contentType, headers, dataToMarshal, options, messageId);

      this._dumpHeaders(' -> ', wireData); // Inject the listener if specified


      if (options.listener !== undefined) {
        this._zws.onMessage.addOnceListener(options.listener, this);
      }

      this._tlsConn.prepare(wireData);
    }
    /**
     * Marshals message into binary wire format.
     *
     * @param {String|Number} contentType
     * @param {[Header]} headers
     * @param {*} body
     * @param {Object} [options]
     * @returns {byte[]}   
     */

  }, {
    key: "_marshalMessage",
    value: function _marshalMessage(contentType, headers, body, options, messageId) {
      // wire-protocol: message-section
      var buffer_message_section = new ArrayBuffer(4 // Version
      + 4 // ContentType  (offset 4)
      + 4 // Sequence     (offset 8)
      + 4 // hdrs-len     (offset 12)
      + 4 // body-len     (offset 16)
      ); // wire-protocol: Version

      var view_message_section = new Uint8Array(buffer_message_section);
      view_message_section.set(edge_protocol.VERSION, 0 // Offset 0
      );
      var bytes = new Buffer(4); // wire-protocol: ContentType

      bytes.writeUInt32LE(contentType, 0);
      view_message_section.set(bytes, 4 // Offset 4
      );
      bytes = new Buffer(4); // wire-protocol: Sequence

      bytes.writeInt32LE(messageId, 0);
      view_message_section.set(bytes, 8 // Offset 8
      );
      bytes = new Buffer(4);
      var hdrsLen = utils.sumBy(headers, function (header) {
        return header.getLength();
      }); // wire-protocol: hdrs-len

      bytes.writeInt32LE(hdrsLen, 0);
      view_message_section.set(bytes, 12 // Offset 12
      );
      bytes = new Buffer(4); // wire-protocol: body-len

      var bodyLength = 0;

      if (!isNull(body)) {
        bodyLength = body.length;
      }

      bytes.writeUInt32LE(bodyLength, 0);
      view_message_section.set(bytes, 16 // Offset 16
      ); // wire-protocol: headers

      var buffer_headers_section = new ArrayBuffer(hdrsLen);
      var view_headers_section = new Uint8Array(buffer_headers_section);
      var view_headers_section_offset = 0;
      forEach(headers, function (header) {
        view_headers_section.set(header.getBytesForWire(), view_headers_section_offset);
        view_headers_section_offset += header.getLength();
      }); // wire-protocol: body

      var buffer_body_section = new ArrayBuffer(bodyLength);

      if (bodyLength > 0) {
        var view_body_section = new Uint8Array(buffer_body_section);
        var body_bytes;

        if (typeof body === 'string') {
          body_bytes = utils.toUTF8Array(body);
        } else {
          body_bytes = body;
        }

        var bytesBuffer = Buffer.from(body_bytes);
        view_body_section.set(bytesBuffer, 0);
      } // Put it all together


      var buffer_combined = utils.appendBuffer(buffer_message_section, buffer_headers_section);
      buffer_combined = utils.appendBuffer(buffer_combined, buffer_body_section);
      var view_combined = new Uint8Array(buffer_combined);
      return view_combined.buffer;
    }
    /**
     * Receives encrypted message from the Controller.
     * 
     * @param {*} data 
     */

  }, {
    key: "_recvFromWire",
    value: function () {
      var _recvFromWire2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(data) {
        var buffer, tlsBinaryString;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                _context7.next = 2;
                return data.arrayBuffer();

              case 2:
                buffer = _context7.sent;
                tlsBinaryString = Buffer.from(buffer).toString('binary');

                this._tlsConn.process(tlsBinaryString);

              case 5:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7, this);
      }));

      function _recvFromWire(_x6) {
        return _recvFromWire2.apply(this, arguments);
      }

      return _recvFromWire;
    }()
    /**
     * Receives un-encrypted message from the Controller.
     * 
     * @param {*} data 
     */

  }, {
    key: "_recvFromWireAfterDecrypt",
    value: function () {
      var _recvFromWireAfterDecrypt2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(ch, data) {
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                ch._tryUnmarshal(data);

              case 1:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8);
      }));

      function _recvFromWireAfterDecrypt(_x7, _x8) {
        return _recvFromWireAfterDecrypt2.apply(this, arguments);
      }

      return _recvFromWireAfterDecrypt;
    }()
    /**
     * Unmarshals binary from the wire into a message
     * 
     * @param {*} data 
     */

  }, {
    key: "_tryUnmarshal",
    value: function () {
      var _tryUnmarshal2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(data) {
        var buffer, versionView, contentTypeView, contentType, sequenceView, responseSequence, headersLengthView, headersLength, headersView, bodyView;
        return regeneratorRuntime.wrap(function _callee9$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                buffer = data;
                versionView = new Uint8Array(buffer, 0, 4);
                throwIf(!isEqual(versionView[0], this._view_version[0]), formatMessage('Unexpected message version. Got { actual }, expected { expected }', {
                  actual: versionView[0],
                  expected: this._view_version[0]
                })); // let acceptableContentTypes = [edge_protocol.content_type.ResultType, edge_protocol.content_type.StateConnected]

                contentTypeView = new Int32Array(buffer, 4, 1);
                contentType = contentTypeView[0]; // throwIf(!acceptableContentTypes.includes(contentType), formatMessage('Unexpected message content-type. Got { actual }, expected { expected }', { actual: contentType, expected:  edge_protocol.content_type.ResultType}) );

                sequenceView = new Int32Array(buffer, 8, 1); // throwIf(!isEqual(sequenceView[0], -1), formatMessage('Unexpected message sequence. Got { actual }, expected { expected }', { actual: sequenceView[0], expected:  -1 }));

                this._ctx.logger.debug("recv <- contentType[%o] seq[%o]", contentType, sequenceView[0]);

                responseSequence = sequenceView[0];
                headersLengthView = new Int32Array(buffer, 12, 1);
                headersLength = headersLengthView[0];
                headersView = new Uint8Array(buffer, 20); // this._dumpHeaders(' <- ', data);

                bodyView = new Uint8Array(buffer, 20 + headersLength); // let connId;
                // let conn;
                // let replyForView;
                // let haveResponseSequence = false;

                this._ctx.logger.debug("recv <- response body: ", bodyView);

                this._tryHandleResponse(responseSequence, {
                  channel: this,
                  data: bodyView
                });

              case 14:
              case "end":
                return _context9.stop();
            }
          }
        }, _callee9, this);
      }));

      function _tryUnmarshal(_x9) {
        return _tryUnmarshal2.apply(this, arguments);
      }

      return _tryUnmarshal;
    }()
    /**
     * 
     */

  }, {
    key: "_tryHandleResponse",
    value: function _tryHandleResponse(responseSequence, data) {
      // let messagesQueue = this._messages;
      // if (!isUndefined(conn)) {
      // messagesQueue = conn.getMessages()
      // }
      // this._ctx.logger.debug("_tryHandleResponse():  conn[%d] seq[%d]", (conn ? conn.getId() : 'n/a'), responseSequence);
      this._ctx.logger.debug("_tryHandleResponse(): seq[%d]", responseSequence);

      if (!isNull(responseSequence)) {
        this._messages.resolve(responseSequence, data);
      } else {
        debugger;
      }
    }
    /**
     * 
     */

  }, {
    key: "_dumpHeaders",
    value: function () {
      var _dumpHeaders2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(pfx, buffer) {
        var headersView, headersLength, headersOffset, ndx, view, _headerId, _headerDataLength, _headerData, connId, _buffer;

        return regeneratorRuntime.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                headersView = new Int32Array(buffer, 12, 1);
                headersLength = headersView[0];
                headersOffset = 16 + 4;
                ndx = 0;
                view = new DataView(buffer);

                this._ctx.logger.trace("_dumpHeaders: " + pfx + "vv----------------------------------");

                for (; ndx < headersLength;) {
                  _headerId = view.getInt32(headersOffset + ndx, true);
                  ndx += 4;
                  _headerDataLength = view.getInt32(headersOffset + ndx, true);
                  ndx += 4;
                  _headerData = new Uint8Array(buffer, headersOffset + ndx, _headerDataLength);
                  ndx += _headerDataLength;
                  connId = 'n/a';

                  if (isEqual(_headerId, edge_protocol.header_id.ConnId)) {
                    _buffer = Buffer.from(_headerData);
                    connId = _buffer.readUIntLE(0, _headerDataLength);
                  }

                  this._ctx.logger.trace("headerId[%d] conn[%d] dataLength[%d] data[%o]", _headerId, connId, _headerDataLength, _headerData);
                }

                this._ctx.logger.trace("_dumpHeaders: " + pfx + "^^----------------------------------");

              case 8:
              case "end":
                return _context10.stop();
            }
          }
        }, _callee10, this);
      }));

      function _dumpHeaders(_x10, _x11) {
        return _dumpHeaders2.apply(this, arguments);
      }

      return _dumpHeaders;
    }()
    /**
     * 
     */

  }, {
    key: "_findHeader",
    value: function () {
      var _findHeader2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(msg, headerToFind) {
        var buffer, headersView, headersLength, headersOffset, ndx, view, _headerId, _headerDataLength, _headerData, result;

        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                if (isUndefined(msg.arrayBuffer)) {
                  _context11.next = 6;
                  break;
                }

                _context11.next = 3;
                return msg.arrayBuffer();

              case 3:
                buffer = _context11.sent;
                _context11.next = 9;
                break;

              case 6:
                _context11.next = 8;
                return msg.buffer;

              case 8:
                buffer = _context11.sent;

              case 9:
                headersView = new Int32Array(buffer, 12, 1);
                headersLength = headersView[0];
                headersOffset = 16 + 4;
                ndx = 0;
                view = new DataView(buffer);

              case 14:
                if (!(ndx < headersLength)) {
                  _context11.next = 26;
                  break;
                }

                _headerId = view.getInt32(headersOffset + ndx, true);
                ndx += 4;
                _headerDataLength = view.getInt32(headersOffset + ndx, true);
                ndx += 4;
                _headerData = new Uint8Array(buffer, headersOffset + ndx, _headerDataLength);
                ndx += _headerDataLength;

                if (!(_headerId == headerToFind)) {
                  _context11.next = 24;
                  break;
                }

                result = {
                  dataLength: _headerDataLength,
                  data: _headerData
                };
                return _context11.abrupt("return", result);

              case 24:
                _context11.next = 14;
                break;

              case 26:
                return _context11.abrupt("return", undefined);

              case 27:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11);
      }));

      function _findHeader(_x12, _x13) {
        return _findHeader2.apply(this, arguments);
      }

      return _findHeader;
    }()
    /**
     * 
     */

  }, {
    key: "_messageGetBytesHeader",
    value: function () {
      var _messageGetBytesHeader2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12(msg, headerToFind) {
        return regeneratorRuntime.wrap(function _callee12$(_context12) {
          while (1) {
            switch (_context12.prev = _context12.next) {
              case 0:
                _context12.next = 2;
                return this._findHeader(msg, headerToFind);

              case 2:
                return _context12.abrupt("return", _context12.sent);

              case 3:
              case "end":
                return _context12.stop();
            }
          }
        }, _callee12, this);
      }));

      function _messageGetBytesHeader(_x14, _x15) {
        return _messageGetBytesHeader2.apply(this, arguments);
      }

      return _messageGetBytesHeader;
    }()
    /**
     * 
     */

  }, {
    key: "_messageGetConnId",
    value: function () {
      var _messageGetConnId2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13(msg) {
        var results, length, buffer, connId;
        return regeneratorRuntime.wrap(function _callee13$(_context13) {
          while (1) {
            switch (_context13.prev = _context13.next) {
              case 0:
                _context13.next = 2;
                return this._findHeader(msg, edge_protocol.header_id.ConnId);

              case 2:
                results = _context13.sent;
                throwIf(results == undefined, formatMessage('No ConnId header found'));
                length = results.data.length;
                buffer = Buffer.from(results.data);
                connId = buffer.readUIntLE(0, length);
                return _context13.abrupt("return", connId);

              case 8:
              case "end":
                return _context13.stop();
            }
          }
        }, _callee13, this);
      }));

      function _messageGetConnId(_x16) {
        return _messageGetConnId2.apply(this, arguments);
      }

      return _messageGetConnId;
    }()
  }, {
    key: "getConnection",
    value: function getConnection(id) {
      return this._connections._getConnection(id);
    }
  }, {
    key: "getSocket",
    value: function getSocket() {
      return this._socket;
    }
  }, {
    key: "setSocket",
    value: function setSocket(socket) {
      this._socket = socket;
    }
  }, {
    key: "getDataCallback",
    value: function getDataCallback() {
      return this._dataCallback;
    }
  }, {
    key: "setDataCallback",
    value: function setDataCallback(fn) {
      this._dataCallback = fn;
    }
  }, {
    key: "getEncrypted",
    value: function getEncrypted() {
      return this._encrypted;
    }
  }, {
    key: "setEncrypted",
    value: function setEncrypted(encrypted) {
      this._encrypted = encrypted;
    }
  }, {
    key: "getCryptoEstablishComplete",
    value: function getCryptoEstablishComplete() {
      return this._cryptoEstablishComplete;
    }
  }, {
    key: "setCryptoEstablishComplete",
    value: function setCryptoEstablishComplete(complete) {
      this._cryptoEstablishComplete = complete;
    }
  }, {
    key: "getKeypair",
    value: function getKeypair() {
      return this._keypair;
    }
  }, {
    key: "setKeypair",
    value: function setKeypair(keypair) {
      this._keypair = keypair;
    }
  }, {
    key: "getSharedRx",
    value: function getSharedRx() {
      return this._sharedRx;
    }
  }, {
    key: "setSharedRx",
    value: function setSharedRx(sharedRx) {
      this._sharedRx = sharedRx;
    }
  }, {
    key: "getSharedTx",
    value: function getSharedTx() {
      return this._sharedTx;
    }
  }, {
    key: "setSharedTx",
    value: function setSharedTx(sharedTx) {
      this._sharedTx = sharedTx;
    }
  }, {
    key: "getCrypt_o",
    value: function getCrypt_o() {
      return this._crypt_o;
    }
  }, {
    key: "setCrypt_o",
    value: function setCrypt_o(crypt_o) {
      this._crypt_o = crypt_o;
    }
  }, {
    key: "getCrypt_i",
    value: function getCrypt_i() {
      return this._crypt_i;
    }
  }, {
    key: "setCrypt_i",
    value: function setCrypt_i(crypt_i) {
      this._crypt_i = crypt_i;
    }
  }]);

  return ZitiControllerChannel;
}();