"use strict";

require("core-js/modules/es.array.concat.js");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var formatMessage = require('format-message');

var flatOptions = require('flat-options');

var isNull = require('lodash.isnull');

var isEqual = require('lodash.isequal');

var utils = require('../utils/utils');

var defaultOptions = require('./header-options');

var _require = require('../utils/throwif'),
    throwIf = _require.throwIf;

var edge_protocol = require('./protocol');

formatMessage.setup({
  //   locale: 'es-ES', // what locale strings should be displayed
  //   missingReplacement: '!!NOT TRANSLATED!!', // use this when a translation is missing instead of the default message
  missingTranslation: 'ignore' // don't console.warn or throw an error when a translation is missing

});
/**
 * @typicalname header
 */

module.exports = /*#__PURE__*/function () {
  /**
   *
   * @param {int} headerId ZitiEnums.header_id
   * @param {Options} [options]
   */
  function Header(headerId, options) {
    _classCallCheck(this, Header);

    this._headerId = headerId;
    this._options = flatOptions(options, defaultOptions);
    throwIf(isNull(this._options.headerType), formatMessage('headerType not specified'));
    this._headerType = this._options.headerType;
    throwIf(isNull(this._options.headerData), formatMessage('headerData not specified'));
    this._headerData = this._options.headerData;
    this._bytesForWire = this._createBytesForWire();
    this._length = this._bytesForWire.length;
  }

  _createClass(Header, [{
    key: "getId",
    value: function getId() {
      return this._headerId;
    }
  }, {
    key: "getData",
    value: function getData() {
      return this._headerData;
    }
  }, {
    key: "getLength",
    value: function getLength() {
      return this._length;
    }
  }, {
    key: "getBytesForWire",
    value: function getBytesForWire() {
      return this._bytesForWire;
    }
  }, {
    key: "_createBytesForWire",
    value: function _createBytesForWire() {
      if (isEqual(this._headerType, edge_protocol.header_type.StringType)) {
        var headerDataLength = Buffer.byteLength(this._headerData, 'utf8');
        var bytes_header_id_and_length = new Buffer(4 + 4);
        bytes_header_id_and_length.writeUInt32LE(this._headerId, 0);
        bytes_header_id_and_length.writeUInt32LE(headerDataLength, 4);
        var bytes_header_data = utils.toUTF8Array(this._headerData);
        var buffer_header_data = Buffer.from(bytes_header_data);
        var bytes_complete_header = Buffer.concat([bytes_header_id_and_length, buffer_header_data], 4 + 4 + headerDataLength);
        return bytes_complete_header;
      } else if (isEqual(this._headerType, edge_protocol.header_type.IntType)) {
        var _headerDataLength = 4;

        var _bytes_complete_header = new Buffer(4 + 4 + 4);

        _bytes_complete_header.writeUInt32LE(this._headerId, 0);

        _bytes_complete_header.writeUInt32LE(_headerDataLength, 4);

        _bytes_complete_header.writeInt32LE(this._headerData, 8);

        return _bytes_complete_header;
      } else if (isEqual(this._headerType, edge_protocol.header_type.Uint8ArrayType)) {
        var _headerDataLength2 = Buffer.byteLength(this._headerData, 'utf8');

        var _bytes_header_id_and_length = new Buffer(4 + 4);

        _bytes_header_id_and_length.writeUInt32LE(this._headerId, 0);

        _bytes_header_id_and_length.writeUInt32LE(_headerDataLength2, 4);

        var _buffer_header_data = Buffer.from(this._headerData);

        var _bytes_complete_header2 = Buffer.concat([_bytes_header_id_and_length, _buffer_header_data], 4 + 4 + _headerDataLength2);

        return _bytes_complete_header2;
      } else {
        throw new Error('unknown headerType');
      }
    }
  }, {
    key: "_createFromBytesFromWire",
    value: function _createFromBytesFromWire(bytes) {}
  }]);

  return Header;
}();