"use strict";

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.map.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.array.concat.js");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Class for manage pending messages.
 * @private
 */
var PromiseController = require('promise-controller');

var promiseFinally = require('promise.prototype.finally');

var isNull = require('lodash.isnull');

module.exports = /*#__PURE__*/function () {
  function Messages(options) {
    _classCallCheck(this, Messages);

    this._items = new Map();
    this._ctx = options.ctx;
    this._conn = options.conn;
    this._channel = options.channel;
  }
  /**
   * Creates new message and stores it in the list.
   *
   * @param {String|Number} messageId
   * @param {Function} fn
   * @param {Number} timeout
   * @returns {Promise}
   */


  _createClass(Messages, [{
    key: "create",
    value: function create(messageId, fn, timeout) {
      this._ctx.logger.trace("messages.create(): conn[%d] messageId[%o]", this._conn ? this._conn.getId() : 'n/a', messageId);

      this._rejectExistingMessage(messageId);

      return this._createNewMessage(messageId, fn, timeout);
    }
  }, {
    key: "resolve",
    value: function resolve(messageId, data) {
      this._ctx.logger.trace("messages.resolve(): conn[%d] messageId[%o] data[%o]", this._conn ? this._conn.getId() : 'n/a', messageId, data);

      if (!isNull(messageId) && this._items.has(messageId)) {
        this._ctx.logger.trace("messages.resolve(): FOUND messageId: [%o]", messageId);

        this._items.get(messageId).resolve(data);
      }
    }
  }, {
    key: "rejectAll",
    value: function rejectAll(error) {
      this._items.forEach(function (message) {
        return message.isPending ? message.reject(error) : null;
      });
    }
  }, {
    key: "_rejectExistingMessage",
    value: function _rejectExistingMessage(messageId) {
      var existingMessage = this._items.get(messageId);

      if (existingMessage && existingMessage.isPending) {
        existingMessage.reject(new Error("message is replaced, messageId: ".concat(messageId)));
      }
    }
  }, {
    key: "_createNewMessage",
    value: function _createNewMessage(messageId, fn, timeout) {
      var _this = this;

      var message = new PromiseController({
        timeout: timeout,
        timeoutReason: "message was rejected by timeout (".concat(timeout, " ms). messageId: ").concat(messageId)
      });

      this._items.set(messageId, message);

      return promiseFinally(message.call(fn), function () {
        return _this._deleteMessage(messageId, message);
      });
    }
  }, {
    key: "_deleteMessage",
    value: function _deleteMessage(messageId, message) {
      // this check is important when message was replaced
      if (this._items.get(messageId) === message) {
        this._items.delete(messageId);
      }
    }
  }]);

  return Messages;
}();