"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/es.array.slice.js");

require("core-js/modules/es.regexp.to-string.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var Buffer = require('buffer/').Buffer; // note: the trailing slash is important!


var flatOptions = require('flat-options');

var ls = require('../utils/localstorage');

var defaultOptions = require('./tls-connection-options');

var utils = require('../utils/utils');

var zitiConstants = require('../constants');

var forge = require('node-forge');

var ab2str = require('arraybuffer-to-string');

var isUndefined = require('lodash.isundefined');

var isNull = require('lodash.isnull');

var _require = require('uuid'),
    uuidv4 = _require.v4;

forge.options.usePureJavaScript = true;
/**
 * @typicalname connection
 */

module.exports = /*#__PURE__*/function () {
  /**
   *
   * @param {Options} [options]
   */
  function ZitiTLSConnection(options) {
    _classCallCheck(this, ZitiTLSConnection);

    this._options = flatOptions(options, defaultOptions);
    this._type = this._options.type; // debugging

    this._ctx = this._options.ctx;
    this._ws = this._options.ws;
    this._ch = this._options.ch;
    this._datacb = this._options.datacb;
    this._connected = false;
    this._uuid = uuidv4();
  }
  /**
   * Populate this TLS Connection object with the keypair from local storage
   */


  _createClass(ZitiTLSConnection, [{
    key: "pullKeyPair",
    value: function () {
      var _pullKeyPair = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
        var _this = this;

        var self;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                self = this;
                return _context2.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(resolve, reject) {
                    return regeneratorRuntime.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            _context.next = 2;
                            return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                          case 2:
                            _this._clientCertPEM = _context.sent;
                            _context.next = 5;
                            return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY);

                          case 5:
                            _this._clientPrivateKeyPEM = _context.sent;

                            if (!(isUndefined(_this._clientCertPEM) || isUndefined(_this._clientPrivateKeyPEM) || isNull(_this._clientCertPEM) || isNull(_this._clientPrivateKeyPEM))) {
                              _context.next = 8;
                              break;
                            }

                            return _context.abrupt("return", reject(new Error('keypair not present in local storage')));

                          case 8:
                            return _context.abrupt("return", resolve());

                          case 9:
                          case "end":
                            return _context.stop();
                        }
                      }
                    }, _callee);
                  }));

                  return function (_x, _x2) {
                    return _ref.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function pullKeyPair() {
        return _pullKeyPair.apply(this, arguments);
      }

      return pullKeyPair;
    }()
  }, {
    key: "getUUID",
    value: function getUUID() {
      return this._uuid;
    }
  }, {
    key: "create",
    value: function create() {
      var self = this;
      this._tlsClient = forge.tls.createConnection({
        // We're always the client
        server: false,
        // caStore: self._caStore, /* Array of PEM-formatted certs or a CA store object */
        caStore: forge.pki.createCaStore([]),
        //
        sessionCache: {},
        // These are the cipher suites we support (in order of preference)
        cipherSuites: [forge.tls.CipherSuites.TLS_RSA_WITH_AES_128_CBC_SHA256, // forge.tls.CipherSuites.TLS_RSA_WITH_AES_256_CBC_SHA256,
        forge.tls.CipherSuites.TLS_RSA_WITH_AES_128_CBC_SHA, forge.tls.CipherSuites.TLS_RSA_WITH_AES_256_CBC_SHA],
        // virtualHost: 'curt-edge-wss-router:3023',
        verify: function verify(connection, verified, depth, certs) {
          // skip verification for testing
          return true;
          /*
          if(depth === 0) {
            var cn = certs[0].subject.getField('CN').value;
            if(cn !== 'curt-edge-wss-router') {
              verified = {
                alert: forge.tls.Alert.Description.bad_certificate,
                message: 'Certificate common name does not match hostname.'
              };
            }
          }
          return verified;
          */
        },
        connected: function connected(connection) {
          self._ctx.logger.debug('TLS handshake completed successfully');

          self._connected = true; // send message to server
          // connection.prepare(forge.util.encodeUtf8('Hi server!'));

          /* NOTE: experimental, start heartbeat retransmission timer
          myHeartbeatTimer = setInterval(function() {
            connection.prepareHeartbeatRequest(forge.util.createBuffer('1234'));
          }, 5*60*1000);*/
        },
        // client-side cert
        getCertificate: function getCertificate(connection, hint) {
          self._ctx.logger.trace('getCertificate(): for: %o, [%o]', self._uuid, self._clientCertPEM);

          return self._clientCertPEM;
        },
        // client-side private key
        getPrivateKey: function getPrivateKey(connection, cert) {
          self._ctx.logger.trace('getPrivateKey(): for: %o, [%o]', self._uuid, self._clientPrivateKeyPEM);

          return self._clientPrivateKeyPEM;
        },
        // encrypted data is ready to be sent to the server  --->
        tlsDataReady: function tlsDataReady(connection) {
          var chunk = new Buffer(connection.tlsData.getBytes(), "binary");

          if (chunk.length > 0) {
            self._ctx.logger.trace('tlsDataReady: encrypted data is ready to be sent to the server  ---> [%o]', chunk);

            self._ws.send(chunk);
          }
        },
        // clear data from the server is ready               <---
        dataReady: function dataReady(connection) {
          var chunk = new Buffer(connection.data.getBytes(), "binary");
          var ab = chunk.buffer.slice(0, chunk.byteLength);

          self._ctx.logger.trace('dataReady: clear data from the server is ready  <--- ');

          self._datacb(self._ch, ab);
        },

        /* NOTE: experimental
        heartbeatReceived: function(connection, payload) {
          // restart retransmission timer, look at payload
          clearInterval(myHeartbeatTimer);
          myHeartbeatTimer = setInterval(function() {
            connection.prepareHeartbeatRequest(forge.util.createBuffer('1234'));
          }, 5*60*1000);
          payload.getBytes();
        },*/
        closed: function closed(connection) {
          self._ctx.logger.debug('disconnected');
        },
        error: function error(connection, _error) {
          self._ctx.logger.error('uh oh', _error);

          throw _error;
        }
      });
    }
    /**
     * 
     */

  }, {
    key: "handshake",
    value: function handshake() {
      this._tlsClient.handshake();
    }
    /**
     * 
     * @param {*} data 
     */

  }, {
    key: "isTLSHandshakeComplete",
    value: function isTLSHandshakeComplete() {
      return this._connected;
    }
    /**
     * 
     * @param {*} data 
     */

  }, {
    key: "process",
    value: function process(data) {
      this._ctx.logger.trace('process: encrypted data from the server arrived  <--- [%o]', data);

      var results = this._tlsClient.process(data);
    }
    /**
     * 
     * @param {*} data 
     */

  }, {
    key: "prepare",
    value: function prepare(wireData) {
      this._ctx.logger.trace('prepare: unencrypted data is ready to be sent to the server  ---> [%o]', wireData);

      var tlsBinaryString = Buffer.from(wireData).toString('binary');

      this._tlsClient.prepare(tlsBinaryString);
    }
  }]);

  return ZitiTLSConnection;
}();