"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.split.js");

require("core-js/modules/es.string.trim.js");

require("core-js/modules/es.string.starts-with.js");

require("core-js/modules/es.array.concat.js");

require("core-js/modules/es.regexp.constructor.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.string.match.js");

require("core-js/modules/es.string.replace.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/web.timers.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.url.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var isUndefined = require('lodash.isundefined');

var isEqual = require('lodash.isequal');

var forEach = require('lodash.foreach');

var isNull = require('lodash.isnull');

var formatMessage = require('format-message');

var _require = require('readable-stream'),
    PassThrough = _require.PassThrough;

var Mutex = require('async-mutex');

var withTimeout = require('async-mutex').withTimeout;

var Cookies = require('js-cookie');

var CookieInterceptor = require('cookie-interceptor');

var select = require('html-select');

var tokenize = require('html-tokenize');

var through = require('through2');

var ZitiContext = require('./context/context');

var contextTypes = require('./context/contexttypes');

var ZitiConnection = require('./channel/connection');

var HttpRequest = require('./http/request');

var HttpResponse = require('./http/response');

var ZitiFormData = require('./http/form-data');

var BrowserStdout = require('./http/browser-stdout');

var http = require('./http/http');

var ZitiXMLHttpRequest = require('./http/ziti-xhr');

var ZitiWebSocketWrapper = require('./http/ziti-websocket-wrapper');

var LogLevel = require('./logLevels');

var pjson = require('../package.json');

var _require2 = require('./utils/throwif'),
    throwIf = _require2.throwIf;

var ZitiPKI = require('./pki/pki');

var ZitiUPDB = require('./updb/updb');

var ls = require('./utils/localstorage');

var zitiConstants = require('./constants');

var error = require('./updb/error');

formatMessage.setup({
  // locale: 'en', // what locale strings should be displayed
  // missingReplacement: '!!NOT TRANSLATED!!', // use this when a translation is missing instead of the default message
  missingTranslation: 'ignore' // don't console.warn or throw an error when a translation is missing

});

if (!zitiConfig.serviceWorker.active) {
  window.realFetch = window.fetch;
  window.realXMLHttpRequest = window.XMLHttpRequest;
  window.realWebSocket = window.WebSocket;
  window.realInsertBefore = Element.prototype.insertBefore;
  window.realAppendChild = Element.prototype.appendChild;
  window.realSetAttribute = Element.prototype.setAttribute;
}
/**
 * @typicalname client
 */


var ZitiClient = /*#__PURE__*/function () {
  function ZitiClient() {
    _classCallCheck(this, ZitiClient);

    if (!zitiConfig.serviceWorker.active) {
      CookieInterceptor.init(); // Hijack the `document.cookie` object

      CookieInterceptor.write.use(function (cookie) {
        var name = cookie.substring(0, cookie.indexOf("="));
        var value = cookie.substring(cookie.indexOf("=") + 1);
        var cookie_value = value.substring(0, value.indexOf(";"));
        sendMessageToServiceworker({
          command: 'setCookie',
          name: name,
          value: cookie_value
        });

        _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
          var release, zitiCookies, parts, cookiePath, expires, j, part;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  // we use an IIFE because we need to run some await calls, and we cannot make
                  // our write.use() an async func because it will then return a Promise,
                  // which would cause Cookie storage in the browser to get corrupted.
                  console.log('=====> CookieInterceptor sees write of Cookie: ', cookie);
                  _context.next = 3;
                  return ziti._cookiemutex.acquire();

                case 3:
                  release = _context.sent;
                  _context.next = 6;
                  return ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);

                case 6:
                  zitiCookies = _context.sent;

                  if (isNull(zitiCookies)) {
                    zitiCookies = {};
                  } // console.log('=====> CookieInterceptor ZITI_COOKIES (before): ', zitiCookies);


                  if (!(cookie_value !== '')) {
                    _context.next = 14;
                    break;
                  }

                  parts = value.split(";");

                  for (j = 0; j < parts.length; j++) {
                    part = parts[j].trim();
                    part = part.toLowerCase();

                    if (part.startsWith("path")) {
                      cookiePath = part.substring(part.indexOf("=") + 1);
                    } else if (part.startsWith("expires")) {
                      expires = new Date(part.substring(part.indexOf("=") + 1));
                    } else if (part.startsWith("httponly")) {
                      httpOnly = true;
                    }
                  }

                  zitiCookies[name] = cookie_value;
                  _context.next = 14;
                  return ls.setWithExpiry(zitiConstants.get().ZITI_COOKIES, zitiCookies, new Date(8640000000000000));

                case 14:
                  release();

                case 15:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }))();

        return cookie;
      });
    }
  }
  /**
   * Initialize.
   *
   * @param {Options} [options]
   * @return {ZitiContext}
   * @api public
   */


  _createClass(ZitiClient, [{
    key: "init",
    value: function () {
      var _init = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(options) {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                return _context3.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(resolve, reject) {
                    var ctx;
                    return regeneratorRuntime.wrap(function _callee2$(_context2) {
                      while (1) {
                        switch (_context2.prev = _context2.next) {
                          case 0:
                            if (isUndefined(window.realFetch)) {
                              window.realFetch = window.fetch;
                              window.realXMLHttpRequest = window.XMLHttpRequest;
                              window.fetch = fetch;
                              window.XMLHttpRequest = ZitiXMLHttpRequest;
                            }

                            ctx = new ZitiContext(ZitiContext.prototype);
                            _context2.next = 4;
                            return ctx.init(options);

                          case 4:
                            ctx.logger.success('JS SDK version %s init completed', pjson.version);
                            ziti._ctx = ctx;
                            resolve(ctx);

                          case 7:
                          case "end":
                            return _context2.stop();
                        }
                      }
                    }, _callee2);
                  }));

                  return function (_x2, _x3) {
                    return _ref2.apply(this, arguments);
                  };
                }()));

              case 1:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      function init(_x) {
        return _init.apply(this, arguments);
      }

      return init;
    }()
  }, {
    key: "initFromServiceWorker",
    value:
    /**
     * Initialize from Service Worker.
     *
     * @param {Options} [options]
     * @return {ZitiContext}
     * @api public
     */
    function () {
      var _initFromServiceWorker = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(options) {
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                return _context5.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(resolve, reject) {
                    var ctx;
                    return regeneratorRuntime.wrap(function _callee4$(_context4) {
                      while (1) {
                        switch (_context4.prev = _context4.next) {
                          case 0:
                            ctx = new ZitiContext(ZitiContext.prototype);
                            _context4.next = 3;
                            return ctx.init(options);

                          case 3:
                            ctx.logger.success('JS SDK version %s initFromServiceWorker completed', pjson.version);
                            ziti._ctx = ctx;
                            resolve(ctx);

                          case 6:
                          case "end":
                            return _context4.stop();
                        }
                      }
                    }, _callee4);
                  }));

                  return function (_x5, _x6) {
                    return _ref3.apply(this, arguments);
                  };
                }()));

              case 1:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5);
      }));

      function initFromServiceWorker(_x4) {
        return _initFromServiceWorker.apply(this, arguments);
      }

      return initFromServiceWorker;
    }()
  }, {
    key: "newConnection",
    value:
    /**
     * Allocate a new Connection.
     *
     * @param {ZitiContext} ctx
     * @param {*} data
     * @return {ZitiConection}
     * @api public
     */
    function newConnection(ctx, data) {
      throwIf(isUndefined(ctx), formatMessage('Specified context is undefined.', {}));
      throwIf(isEqual(ctx, null), formatMessage('Specified context is null.', {}));
      var conn = new ZitiConnection({
        ctx: ctx,
        data: data
      });
      ctx.logger.debug('newConnection: conn[%d]', conn.getId());
      return conn;
    }
  }, {
    key: "dial",
    value:
    /**
     * Dial the `service`.
     *
     * @param {ZitiConnection} conn
     * @param {String} service
     * @param {Object} [options]
     * @return {Conn}
     * @api public
     */
    function () {
      var _dial = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(conn, service) {
        var options,
            ctx,
            service_id,
            network_session,
            _args6 = arguments;
        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                options = _args6.length > 2 && _args6[2] !== undefined ? _args6[2] : {};
                ctx = conn.getCtx();
                throwIf(isUndefined(ctx), formatMessage('Connection has no context.', {}));
                ctx.logger.debug('dial: conn[%d] service[%s]', conn.getId(), service);

                if (!isEqual(ctx.getServices().size, 0)) {
                  _context6.next = 7;
                  break;
                }

                _context6.next = 7;
                return ctx.fetchServices();

              case 7:
                service_id = ctx.getServiceIdByName(service);
                conn.setEncrypted(ctx.getServiceEncryptionRequiredByName(service));
                _context6.next = 11;
                return ctx.getNetworkSessionByServiceId(service_id);

              case 11:
                network_session = _context6.sent;
                _context6.next = 14;
                return ctx.connect(conn, network_session);

              case 14:
                ctx.logger.debug('dial: conn[%d] service[%s] encryptionRequired[%o] is now complete', conn.getId(), service, conn.getEncrypted());

              case 15:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6);
      }));

      function dial(_x7, _x8) {
        return _dial.apply(this, arguments);
      }

      return dial;
    }()
  }, {
    key: "close",
    value:
    /**
     * Close the `connection`.
     *
     * @param {ZitiConnection} conn
     * @api public
     */
    function () {
      var _close = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(conn) {
        var self;
        return regeneratorRuntime.wrap(function _callee8$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                self = this;
                return _context8.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(resolve, reject) {
                    var ctx;
                    return regeneratorRuntime.wrap(function _callee7$(_context7) {
                      while (1) {
                        switch (_context7.prev = _context7.next) {
                          case 0:
                            ctx = conn.getCtx();
                            throwIf(isUndefined(ctx), formatMessage('Connection has no context.', {}));
                            ctx.logger.debug('close: conn[%d]', conn.getId());
                            _context7.next = 5;
                            return ctx.close(conn);

                          case 5:
                            ctx.logger.debug('close: conn[%d] is now complete', conn.getId());
                            resolve();

                          case 7:
                          case "end":
                            return _context7.stop();
                        }
                      }
                    }, _callee7);
                  }));

                  return function (_x10, _x11) {
                    return _ref4.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context8.stop();
            }
          }
        }, _callee8, this);
      }));

      function close(_x9) {
        return _close.apply(this, arguments);
      }

      return close;
    }()
  }, {
    key: "fetch",
    value:
    /**
     * Do a 'fetch' request over the specified Ziti connection.
     *
     * @param {ZitiConnection} conn
     * @param {String} url
     * @param {Object} opts
     * @return {Promise}
     * @api public
     */
    function () {
      var _fetch = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(conn, url, opts) {
        var ctx;
        return regeneratorRuntime.wrap(function _callee11$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                ctx = conn.getCtx();
                ctx.logger.logger.debug('ziti.fetch() entered');
                return _context11.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(resolve, reject) {
                    var request, options, req;
                    return regeneratorRuntime.wrap(function _callee10$(_context10) {
                      while (1) {
                        switch (_context10.prev = _context10.next) {
                          case 0:
                            // build HTTP request object
                            request = new HttpRequest(conn, url, opts);
                            _context10.next = 3;
                            return request.getRequestOptions();

                          case 3:
                            options = _context10.sent;

                            if (options.method === 'GET') {
                              req = http.get(options);
                            } else {
                              req = http.request(options);
                              req.end();
                            }

                            req.on('error', function (err) {
                              ctx.logger.logger.error('error EVENT: err: %o', err);
                              reject(new Error("request to ".concat(request.url, " failed, reason: ").concat(err.message)));
                              finalize();
                            });
                            req.on('response', /*#__PURE__*/function () {
                              var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(res) {
                                var body, response_options, response;
                                return regeneratorRuntime.wrap(function _callee9$(_context9) {
                                  while (1) {
                                    switch (_context9.prev = _context9.next) {
                                      case 0:
                                        body = res.pipe(new PassThrough());
                                        response_options = {
                                          url: request.url,
                                          status: res.statusCode,
                                          statusText: res.statusMessage,
                                          headers: res.headers,
                                          size: request.size,
                                          timeout: request.timeout,
                                          counter: request.counter
                                        };
                                        response = new HttpResponse(body, response_options);
                                        resolve(response);

                                      case 4:
                                      case "end":
                                        return _context9.stop();
                                    }
                                  }
                                }, _callee9);
                              }));

                              return function (_x17) {
                                return _ref6.apply(this, arguments);
                              };
                            }());

                          case 7:
                          case "end":
                            return _context10.stop();
                        }
                      }
                    }, _callee10);
                  }));

                  return function (_x15, _x16) {
                    return _ref5.apply(this, arguments);
                  };
                }()));

              case 3:
              case "end":
                return _context11.stop();
            }
          }
        }, _callee11);
      }));

      function fetch(_x12, _x13, _x14) {
        return _fetch.apply(this, arguments);
      }

      return fetch;
    }()
    /**
     * Do a 'fetch' request over the specified Ziti connection, on behalf of Service Worker
     *
     * @param {String} url
     * @param {Object} opts
     * @return {Promise}
     * @api public
     */

  }, {
    key: "fetchFromServiceWorker",
    value: function () {
      var _fetchFromServiceWorker = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee18(url, opts) {
        var self;
        return regeneratorRuntime.wrap(function _callee18$(_context18) {
          while (1) {
            switch (_context18.prev = _context18.next) {
              case 0:
                self = this;
                return _context18.abrupt("return", new Promise( /*#__PURE__*/function () {
                  var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee17(resolve, reject) {
                    var redp, domProxyHit, serviceName, request, options, req, push, bodyStream, bodyStreamReader;
                    return regeneratorRuntime.wrap(function _callee17$(_context17) {
                      while (1) {
                        switch (_context17.prev = _context17.next) {
                          case 0:
                            _context17.next = 2;
                            return ziti._serviceWorkerMutexWithTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
                              var ctx;
                              return regeneratorRuntime.wrap(function _callee12$(_context12) {
                                while (1) {
                                  switch (_context12.prev = _context12.next) {
                                    case 0:
                                      console.log('js-sdk fetchFromServiceWorker() acquired ziti._mutex');

                                      if (!isUndefined(ziti._ctx)) {
                                        _context12.next = 7;
                                        break;
                                      }

                                      // If we have no context, create it now
                                      ctx = new ZitiContext(ZitiContext.prototype);
                                      _context12.next = 5;
                                      return ctx.initFromServiceWorker({
                                        contextType: contextTypes.ServiceWorkerType,
                                        logLevel: LogLevel[zitiConfig.httpAgent.zitiSDKjs.logLevel]
                                      });

                                    case 5:
                                      ctx.logger.success('JS SDK version %s init (fetchFromServiceWorker) completed', pjson.version);
                                      ziti._ctx = ctx;

                                    case 7:
                                    case "end":
                                      return _context12.stop();
                                  }
                                }
                              }, _callee12);
                            }))).catch(function (err) {
                              ziti._ctx.logger.error(err);

                              return reject(err);
                            });

                          case 2:
                            redp = new RegExp(zitiConfig.httpAgent.target.host + "/ziti-dom-proxy/", "gi");
                            domProxyHit = (url.match(redp) || []).length;

                            if (domProxyHit > 0) {
                              url = url.replace(redp, '');

                              ziti._ctx.logger.debug('fetchFromServiceWorker: transformed dom-proxy url: ', url);
                            }

                            _context17.next = 7;
                            return ziti._ctx.shouldRouteOverZiti(url).catch( /*#__PURE__*/function () {
                              var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13(error) {
                                return regeneratorRuntime.wrap(function _callee13$(_context13) {
                                  while (1) {
                                    switch (_context13.prev = _context13.next) {
                                      case 0:
                                        ziti._ctx.logger.debug('fetchFromServiceWorker: purging cert and API token due to err: ', error);

                                        _context13.next = 3;
                                        return ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

                                      case 3:
                                        _context13.next = 5;
                                        return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                                      case 5:
                                        return _context13.abrupt("return", reject(error));

                                      case 6:
                                      case "end":
                                        return _context13.stop();
                                    }
                                  }
                                }, _callee13);
                              }));

                              return function (_x22) {
                                return _ref9.apply(this, arguments);
                              };
                            }());

                          case 7:
                            serviceName = _context17.sent;

                            if (!isUndefined(serviceName)) {
                              _context17.next = 10;
                              break;
                            }

                            return _context17.abrupt("return", reject(new Error('no serviceConfig associated with the url: ' + url)));

                          case 10:
                            /**
                             * ------------ Now Routing over Ziti -----------------
                             */
                            ziti._ctx.logger.debug('fetchFromServiceWorker(): serviceConfig match; intercepting [%s]', url); // build HTTP request object


                            // build HTTP request object
                            request = new HttpRequest(serviceName, url, opts);
                            _context17.next = 14;
                            return request.getRequestOptions();

                          case 14:
                            options = _context17.sent;
                            options.domProxyHit = domProxyHit;

                            if (!(options.method === 'GET')) {
                              _context17.next = 20;
                              break;
                            }

                            req = http.get(options);
                            _context17.next = 28;
                            break;

                          case 20:
                            req = http.request(options);

                            if (!options.body) {
                              _context17.next = 27;
                              break;
                            }

                            push = /*#__PURE__*/function () {
                              var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee15() {
                                return regeneratorRuntime.wrap(function _callee15$(_context15) {
                                  while (1) {
                                    switch (_context15.prev = _context15.next) {
                                      case 0:
                                        return _context15.abrupt("return", new Promise( /*#__PURE__*/function () {
                                          var _ref11 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14(resolve, reject) {
                                            var chunk;
                                            return regeneratorRuntime.wrap(function _callee14$(_context14) {
                                              while (1) {
                                                switch (_context14.prev = _context14.next) {
                                                  case 0:
                                                    _context14.next = 2;
                                                    return bodyStreamReader.read();

                                                  case 2:
                                                    chunk = _context14.sent;

                                                    if (!chunk) {
                                                      _context14.next = 8;
                                                      break;
                                                    }

                                                    if (!(!chunk.done && chunk.value)) {
                                                      _context14.next = 8;
                                                      break;
                                                    }

                                                    req.write(chunk.value);
                                                    _context14.next = 8;
                                                    return push();

                                                  case 8:
                                                    resolve();

                                                  case 9:
                                                  case "end":
                                                    return _context14.stop();
                                                }
                                              }
                                            }, _callee14);
                                          }));

                                          return function (_x23, _x24) {
                                            return _ref11.apply(this, arguments);
                                          };
                                        }()));

                                      case 1:
                                      case "end":
                                        return _context15.stop();
                                    }
                                  }
                                }, _callee15);
                              }));

                              return function push() {
                                return _ref10.apply(this, arguments);
                              };
                            }();

                            bodyStream = options.body.stream();
                            bodyStreamReader = bodyStream.getReader();
                            _context17.next = 27;
                            return push();

                          case 27:
                            req.end();

                          case 28:
                            ziti._ctx.logger.debug('fetchFromServiceWorker(): req launched for [%s]', url);

                            req.on('error', function (err) {
                              ziti._ctx.logger.error('error EVENT: err: %o', err);

                              reject(new Error("request to ".concat(request.url, " failed, reason: ").concat(err.message)));
                            });
                            req.on('response', /*#__PURE__*/function () {
                              var _ref12 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee16(res) {
                                var body, reS, reH, reS2, s, l, h, location, targetHost, response_options, response, hdr, cookieArray, cookiePath, expires, _httpOnly, zitiCookies, i, cookie, name, value, cookie_value, parts, j, part;

                                return regeneratorRuntime.wrap(function _callee16$(_context16) {
                                  while (1) {
                                    switch (_context16.prev = _context16.next) {
                                      case 0:
                                        ziti._ctx.logger.debug('fetchFromServiceWorker(): on.response entered for [%s]', url);

                                        if (req.domProxyHit > 0) {
                                          reS = new RegExp("src=\"/", "gi");
                                          reH = new RegExp("href=\"/", "gi");
                                          reS2 = new RegExp("\'\/resources", "gi");
                                          s = select('script', function (e) {
                                            var tr = through.obj(function (row, buf, next) {
                                              var val = String(row[1]);

                                              if (row[0] === 'open') {
                                                var replace = 'src="https://' + req.host + '/';
                                                var newVal = val.replace(reS, replace);
                                                this.push([row[0], newVal]);
                                              } else if (row[0] === 'text') {
                                                var _replace = "'https://" + req.host + "/resources";

                                                var _newVal = val.replace(reS2, _replace);

                                                ziti._ctx.logger.debug('newVal [%s]', _newVal);

                                                this.push([row[0], _newVal]);
                                              } else {
                                                this.push([row[0], val]);
                                              }

                                              next();
                                            });
                                            tr.pipe(e.createStream()).pipe(tr);
                                          });
                                          l = select('link', function (e) {
                                            var tr = through.obj(function (row, buf, next) {
                                              var val = String(row[1]);

                                              if (row[0] === 'open') {
                                                var replace = 'href="https://' + req.host + '/';
                                                var newVal = val.replace(reH, replace);
                                                this.push([row[0], newVal]);
                                              } else {
                                                this.push([row[0], val]);
                                              }

                                              next();
                                            });
                                            tr.pipe(e.createStream()).pipe(tr);
                                          });
                                          h = select('html', function (e) {
                                            var tr = through.obj(function (row, buf, next) {
                                              var val = String(row[1]);

                                              if (row[0] === 'open') {
                                                ziti._ctx.logger.debug('fetchFromServiceWorker(): in <html>, see open of [%s]', val);

                                                var newVal = val;

                                                if (val == '<html>') {
                                                  newVal = val + "<base href=\"https://".concat(req.host, "/\">\n<script type=\"text/javascript\">\nvar zitiConfig = {\n  controller: {\n    api: \"https://curt-controller:1280\"\n  },\n  httpAgent: {\n    self: {\n        host: \"browzer.duckdns.org\",\n        port: \"8443\"\n    },\n    target: {\n      scheme: \"http\",\n      host: \"spendai-dev2.electrifai.net\",\n      port: \"443\"     \n    },\n    additionalTarget: {\n      scheme: \"http\",\n      host: \"sisense-dev2.electrifai.net\",\n      port: \"443\"     \n    },\n    corsProxy: {\n      hosts: \"electrifai-products.oktapreview.com:443\",\n    },\n    domProxy: {\n      hosts: \"sisense-dev2.electrifai.net\",\n    },\n    zitiSDKjs: {\n      location: \"https://ziti-npm.s3.amazonaws.com/ziti_sdk_js/ziti.js\",\n      logLevel: \"Debug\"\n    },\n    zitiSDKjsInjectionURL: {\n      location: \"\",\n    }\n  },\n  serviceWorker: {\n    location: \"ziti-sw.js\",\n    active: false\n  }\n}\n</script>\n<script type=\"text/javascript\" src=\"https://ziti-npm.s3.amazonaws.com/ziti_sdk_js/ziti.js\"></script>");
                                                }

                                                this.push([row[0], newVal]);
                                              } else if (row[0] === 'text') {
                                                ziti._ctx.logger.debug('fetchFromServiceWorker(): in <html>, see text of <head> [%s]', val);

                                                this.push([row[0], val]);
                                              } else {
                                                this.push([row[0], val]);
                                              }

                                              next();
                                            });
                                            tr.pipe(e.createStream()).pipe(tr);
                                          });
                                          body = res.pipe(tokenize()).pipe(s).pipe(l).pipe(h).pipe(through.obj(function (row, buf, next) {
                                            this.push(row[1]);
                                            next();
                                          })).pipe(new PassThrough());
                                        } else {
                                          body = res.pipe(new PassThrough());
                                        } // Make sure browser won't kill response with a CORS error


                                        // Make sure browser won't kill response with a CORS error
                                        res.headers['access-control-allow-origin'] = 'https://' + zitiConfig.httpAgent.self.host;
                                        location = res.headers.location;

                                        if (!isUndefined(location)) {
                                          location = location.replace("redirect_uri=".concat(zitiConfig.httpAgent.target.scheme, "%"), "redirect_uri=https%");
                                          targetHost = "".concat(zitiConfig.httpAgent.target.host);
                                          targetHost = targetHost.toLowerCase();
                                          location = location.replace("".concat(targetHost), "".concat(zitiConfig.httpAgent.self.host));
                                          res.headers.location = location;
                                        }

                                        response_options = {
                                          url: request.url,
                                          status: res.statusCode,
                                          statusText: res.statusMessage,
                                          headers: res.headers,
                                          size: request.size,
                                          timeout: request.timeout,
                                          counter: request.counter
                                        };
                                        response = new HttpResponse(body, response_options);
                                        _context16.t0 = regeneratorRuntime.keys(response_options.headers);

                                      case 8:
                                        if ((_context16.t1 = _context16.t0()).done) {
                                          _context16.next = 38;
                                          break;
                                        }

                                        hdr = _context16.t1.value;

                                        if (!response_options.headers.hasOwnProperty(hdr)) {
                                          _context16.next = 36;
                                          break;
                                        }

                                        if (!(hdr === 'set-cookie')) {
                                          _context16.next = 36;
                                          break;
                                        }

                                        cookieArray = response_options.headers[hdr];
                                        cookiePath = void 0;
                                        expires = void 0;
                                        _httpOnly = false;
                                        _context16.next = 18;
                                        return ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);

                                      case 18:
                                        zitiCookies = _context16.sent;

                                        if (isNull(zitiCookies)) {
                                          zitiCookies = {};
                                        }

                                        i = 0;

                                      case 21:
                                        if (!(i < cookieArray.length)) {
                                          _context16.next = 36;
                                          break;
                                        }

                                        cookie = cookieArray[i];
                                        name = cookie.substring(0, cookie.indexOf("="));
                                        value = cookie.substring(cookie.indexOf("=") + 1);
                                        cookie_value = value.substring(0, value.indexOf(";"));

                                        if (!(cookie_value !== '')) {
                                          _context16.next = 33;
                                          break;
                                        }

                                        parts = value.split(";");

                                        for (j = 0; j < parts.length; j++) {
                                          part = parts[j].trim();

                                          if (part.startsWith("Path")) {
                                            cookiePath = part.substring(part.indexOf("=") + 1);
                                          } else if (part.startsWith("Expires")) {
                                            expires = new Date(part.substring(part.indexOf("=") + 1));
                                          } else if (part.startsWith("HttpOnly")) {
                                            _httpOnly = true;
                                          }
                                        }

                                        zitiCookies[name] = cookie_value;
                                        _context16.next = 32;
                                        return ls.setWithExpiry(zitiConstants.get().ZITI_COOKIES, zitiCookies, new Date(8640000000000000));

                                      case 32:
                                        Cookies.set(name, cookie_value, {
                                          expires: expires,
                                          path: cookiePath
                                        });

                                      case 33:
                                        i++;
                                        _context16.next = 21;
                                        break;

                                      case 36:
                                        _context16.next = 8;
                                        break;

                                      case 38:
                                        resolve(response);

                                      case 39:
                                      case "end":
                                        return _context16.stop();
                                    }
                                  }
                                }, _callee16);
                              }));

                              return function (_x25) {
                                return _ref12.apply(this, arguments);
                              };
                            }());

                          case 31:
                          case "end":
                            return _context17.stop();
                        }
                      }
                    }, _callee17);
                  }));

                  return function (_x20, _x21) {
                    return _ref7.apply(this, arguments);
                  };
                }()));

              case 2:
              case "end":
                return _context18.stop();
            }
          }
        }, _callee18, this);
      }));

      function fetchFromServiceWorker(_x18, _x19) {
        return _fetchFromServiceWorker.apply(this, arguments);
      }

      return fetchFromServiceWorker;
    }()
  }]);

  return ZitiClient;
}();

var ziti = new ZitiClient();
ziti.LogLevel = LogLevel;
ziti._clientMutexNoTimeout = new Mutex.Mutex();
ziti._clientMutexWithTimeout = withTimeout(new Mutex.Mutex(), 30000);
ziti._serviceWorkerMutexNoTimeout = new Mutex.Mutex();
ziti._serviceWorkerMutexWithTimeout = withTimeout(new Mutex.Mutex(), 30000);
ziti._cookiemutex = new Mutex.Mutex();
ziti.VERSION = pjson.version;
module.exports = ziti;
/**
 * Intercept all 'fetch' requests and route them over Ziti if the target host:port matches an active Ziti Service Config
 *
 * @param {String} url
 * @param {Object} opts
 * @return {Promise}
 * @api public
 */

zitiFetch = /*#__PURE__*/function () {
  var _ref13 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee28(url, opts) {
    var serviceName, identyPresent, regex, regexSlash, regexDotSlash, newUrl, _newUrl, baseURIUrl, baseURI, _newUrl2, _baseURIUrl, slashUrl, _baseURI, routeOverCORSProxy, _newUrl3, corsTargetHostname, corsTargetPort, corsTargetPathname;

    return regeneratorRuntime.wrap(function _callee28$(_context28) {
      while (1) {
        switch (_context28.prev = _context28.next) {
          case 0:
            _context28.next = 2;
            return ziti._clientMutexWithTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee19() {
              var ctx;
              return regeneratorRuntime.wrap(function _callee19$(_context19) {
                while (1) {
                  switch (_context19.prev = _context19.next) {
                    case 0:
                      if (!isUndefined(ziti._ctx)) {
                        _context19.next = 6;
                        break;
                      }

                      // If we have no context, create it now
                      ctx = new ZitiContext(ZitiContext.prototype); // await ctx.initFromServiceWorker({ contextType: contextTypes.ClientType, logLevel: LogLevel[zitiConfig.httpAgent.zitiSDKjs.logLevel] } );

                      _context19.next = 4;
                      return ctx.init({
                        contextType: contextTypes.ClientType,
                        logLevel: LogLevel[zitiConfig.httpAgent.zitiSDKjs.logLevel]
                      });

                    case 4:
                      ctx.logger.success('JS SDK version %s init (zitiFetch) completed', pjson.version);
                      ziti._ctx = ctx;

                    case 6:
                    case "end":
                      return _context19.stop();
                  }
                }
              }, _callee19);
            }))).catch(function (err) {
              ziti._ctx.logger.error(err);

              return new Promise( /*#__PURE__*/function () {
                var _ref15 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee20(resolve, reject) {
                  return regeneratorRuntime.wrap(function _callee20$(_context20) {
                    while (1) {
                      switch (_context20.prev = _context20.next) {
                        case 0:
                          reject(err);

                        case 1:
                        case "end":
                          return _context20.stop();
                      }
                    }
                  }, _callee20);
                }));

                return function (_x28, _x29) {
                  return _ref15.apply(this, arguments);
                };
              }());
            });

          case 2:
            _internal_generateKeyPair();

            _context28.next = 5;
            return _internal_isIdentityPresent();

          case 5:
            identyPresent = _context28.sent;
            // We only want to intercept fetch requests that target the Ziti HTTP Agent
            regex = new RegExp(zitiConfig.httpAgent.self.host, 'g');
            regexSlash = new RegExp(/^\//, 'g');
            regexDotSlash = new RegExp(/^\.\//, 'g');

            if (!url.match(regex)) {
              _context28.next = 25;
              break;
            }

            _context28.next = 12;
            return ziti._clientMutexNoTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee22() {
              var isExpired, updb;
              return regeneratorRuntime.wrap(function _callee22$(_context22) {
                while (1) {
                  switch (_context22.prev = _context22.next) {
                    case 0:
                      _context22.next = 2;
                      return ziti._ctx.isIdentityCertExpired();

                    case 2:
                      isExpired = _context22.sent;

                      if (!isExpired) {
                        _context22.next = 14;
                        break;
                      }

                      updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context22.next = 7;
                      return updb.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 7:
                      _context22.next = 9;
                      return updb.awaitCredentialsAndAPISession();

                    case 9:
                      _context22.next = 11;
                      return ziti._ctx._awaitIdentityLoadComplete().catch(function (err) {
                        ziti._ctx.logger.error(err);

                        return new Promise( /*#__PURE__*/function () {
                          var _ref17 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee21(resolve, reject) {
                            return regeneratorRuntime.wrap(function _callee21$(_context21) {
                              while (1) {
                                switch (_context21.prev = _context21.next) {
                                  case 0:
                                    reject(err);

                                  case 1:
                                  case "end":
                                    return _context21.stop();
                                }
                              }
                            }, _callee21);
                          }));

                          return function (_x30, _x31) {
                            return _ref17.apply(this, arguments);
                          };
                        }());
                      });

                    case 11:
                      // Let sw know it should reset identity awareness
                      sendMessageToServiceworker({
                        command: 'identityLoaded',
                        identityLoaded: 0
                      }); // Trigger a page reload now that we have a fresh identity

                      // Trigger a page reload now that we have a fresh identity
                      updb.relodingPage();
                      setTimeout(function () {
                        window.location.reload();
                      }, 500);

                    case 14:
                    case "end":
                      return _context22.stop();
                  }
                }
              }, _callee22);
            }))).catch(function (err) {
              ziti._ctx.logger.error(err);

              return new Promise( /*#__PURE__*/function () {
                var _ref18 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee23(resolve, reject) {
                  return regeneratorRuntime.wrap(function _callee23$(_context23) {
                    while (1) {
                      switch (_context23.prev = _context23.next) {
                        case 0:
                          reject(err);

                        case 1:
                        case "end":
                          return _context23.stop();
                      }
                    }
                  }, _callee23);
                }));

                return function (_x32, _x33) {
                  return _ref18.apply(this, arguments);
                };
              }());
            });

          case 12:
            newUrl = new URL(url);
            newUrl.hostname = zitiConfig.httpAgent.target.host;
            newUrl.port = zitiConfig.httpAgent.target.port;

            ziti._ctx.logger.trace('zitiFetch: transformed URL: ', newUrl.toString());

            _context28.next = 18;
            return ziti._ctx.shouldRouteOverZiti(newUrl);

          case 18:
            serviceName = _context28.sent;

            if (!isUndefined(serviceName)) {
              _context28.next = 22;
              break;
            }

            // If we have no serviceConfig associated with the hostname:port, do not intercept
            ziti._ctx.logger.warn('zitiFetch(): no associated serviceConfig, bypassing intercept of [%s]', url);

            return _context28.abrupt("return", window.realFetch(url, opts));

          case 22:
            url = newUrl.toString();
            _context28.next = 78;
            break;

          case 25:
            if (!url.match(regexSlash)) {
              _context28.next = 40;
              break;
            }

            _context28.next = 28;
            return ziti._clientMutexNoTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee24() {
              var isExpired, updb;
              return regeneratorRuntime.wrap(function _callee24$(_context24) {
                while (1) {
                  switch (_context24.prev = _context24.next) {
                    case 0:
                      _context24.next = 2;
                      return ziti._ctx.isIdentityCertExpired();

                    case 2:
                      isExpired = _context24.sent;

                      if (!isExpired) {
                        _context24.next = 9;
                        break;
                      }

                      updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context24.next = 7;
                      return updb.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 7:
                      _context24.next = 9;
                      return updb.awaitCredentialsAndAPISession();

                    case 9:
                    case "end":
                      return _context24.stop();
                  }
                }
              }, _callee24);
            }))).catch(function (err) {
              ziti._ctx.logger.error(err);

              throw err;
            });

          case 28:
            baseURIUrl = new URL(document.baseURI);

            if (baseURIUrl.hostname === zitiConfig.httpAgent.self.host) {
              _newUrl = new URL('https://' + zitiConfig.httpAgent.target.host + ':' + zitiConfig.httpAgent.target.port + url);
            } else {
              baseURI = document.baseURI.replace(/\.\/$/, '');
              _newUrl = new URL(baseURI + url);
            }

            ziti._ctx.logger.debug('zitiFetch: transformed URL: ', _newUrl.toString());

            _context28.next = 33;
            return ziti._ctx.shouldRouteOverZiti(_newUrl);

          case 33:
            serviceName = _context28.sent;

            if (!isUndefined(serviceName)) {
              _context28.next = 37;
              break;
            }

            // If we have no serviceConfig associated with the hostname:port, do not intercept
            ziti._ctx.logger.warn('zitiFetch(): no associated serviceConfig, bypassing intercept of [%s]', url);

            return _context28.abrupt("return", window.realFetch(url, opts));

          case 37:
            url = _newUrl.toString();
            _context28.next = 78;
            break;

          case 40:
            if (!url.match(regexDotSlash)) {
              _context28.next = 55;
              break;
            }

            _context28.next = 43;
            return ziti._clientMutexNoTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee25() {
              var isExpired, updb;
              return regeneratorRuntime.wrap(function _callee25$(_context25) {
                while (1) {
                  switch (_context25.prev = _context25.next) {
                    case 0:
                      _context25.next = 2;
                      return ziti._ctx.isIdentityCertExpired();

                    case 2:
                      isExpired = _context25.sent;

                      if (!isExpired) {
                        _context25.next = 9;
                        break;
                      }

                      updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context25.next = 7;
                      return updb.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 7:
                      _context25.next = 9;
                      return updb.awaitCredentialsAndAPISession();

                    case 9:
                    case "end":
                      return _context25.stop();
                  }
                }
              }, _callee25);
            }))).catch(function (err) {
              ziti._ctx.logger.error(err);

              throw err;
            });

          case 43:
            _baseURIUrl = new URL(document.baseURI);

            if (_baseURIUrl.hostname === zitiConfig.httpAgent.self.host) {
              slashUrl = url.replace('./', '/');
              _newUrl2 = new URL('https://' + zitiConfig.httpAgent.target.host + ':' + zitiConfig.httpAgent.target.port + slashUrl);
            } else {
              _baseURI = document.baseURI.replace(/\/$/, '');
              _newUrl2 = new URL(_baseURI + url);
            }

            ziti._ctx.logger.debug('zitiFetch: transformed URL: ', _newUrl2.toString());

            _context28.next = 48;
            return ziti._ctx.shouldRouteOverZiti(_newUrl2);

          case 48:
            serviceName = _context28.sent;

            if (!isUndefined(serviceName)) {
              _context28.next = 52;
              break;
            }

            // If we have no serviceConfig associated with the hostname:port, do not intercept
            ziti._ctx.logger.warn('zitiFetch(): no associated serviceConfig, bypassing intercept of [%s]', url);

            return _context28.abrupt("return", window.realFetch(url, opts));

          case 52:
            url = _newUrl2.toString();
            _context28.next = 78;
            break;

          case 55:
            _context28.next = 57;
            return ziti._ctx.shouldRouteOverZiti(url);

          case 57:
            serviceName = _context28.sent;

            if (!isUndefined(serviceName)) {
              _context28.next = 78;
              break;
            }

            _context28.next = 61;
            return ziti._ctx.shouldRouteOverCORSProxy(url);

          case 61:
            routeOverCORSProxy = _context28.sent;

            if (!routeOverCORSProxy) {
              _context28.next = 76;
              break;
            }

            // If we hostname:port is something we need to CORS Proxy
            ziti._ctx.logger.warn('zitiFetch(): doing CORS Proxying of [%s]', url);

            _newUrl3 = new URL(url);
            corsTargetHostname = _newUrl3.hostname;
            corsTargetPort = _newUrl3.port;

            if (corsTargetPort === '') {
              if (_newUrl3.protocol === 'https:') {
                corsTargetPort = '443';
              } else {
                corsTargetPort = '80';
              }
            }

            corsTargetPathname = _newUrl3.pathname;
            _newUrl3.hostname = zitiConfig.httpAgent.self.host;
            _newUrl3.port = 443;
            _newUrl3.pathname = '/ziti-cors-proxy/' + corsTargetHostname + ':' + corsTargetPort + corsTargetPathname; // newUrl.pathname = '/ziti-cors-proxy/' + corsTargetHostname  + corsTargetPathname;

            ziti._ctx.logger.warn('zitiFetch: transformed URL: ', _newUrl3.toString());

            return _context28.abrupt("return", window.realFetch(_newUrl3, opts));

          case 76:
            ziti._ctx.logger.warn('zitiFetch(): no associated serviceConfig, bypassing intercept of [%s]', url);

            return _context28.abrupt("return", window.realFetch(url, opts));

          case 78:
            /**
             * ------------ Now Routing over Ziti -----------------
             */
            ziti._ctx.logger.trace('zitiFetch(): serviceConfig match; intercepting [%s]', url);

            return _context28.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref21 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee27(resolve, reject) {
                var request, options, req, chunk, p;
                return regeneratorRuntime.wrap(function _callee27$(_context27) {
                  while (1) {
                    switch (_context27.prev = _context27.next) {
                      case 0:
                        // build HTTP request object
                        request = new HttpRequest(serviceName, url, opts);
                        _context27.next = 3;
                        return request.getRequestOptions();

                      case 3:
                        options = _context27.sent;

                        if (!(options.method === 'GET')) {
                          _context27.next = 8;
                          break;
                        }

                        req = http.get(options);
                        _context27.next = 25;
                        break;

                      case 8:
                        req = http.request(options);

                        if (!options.body) {
                          _context27.next = 24;
                          break;
                        }

                        if (!(options.body instanceof Promise)) {
                          _context27.next = 17;
                          break;
                        }

                        _context27.next = 13;
                        return options.body;

                      case 13:
                        chunk = _context27.sent;
                        req.write(chunk);
                        _context27.next = 24;
                        break;

                      case 17:
                        if (!(options.body instanceof ZitiFormData)) {
                          _context27.next = 23;
                          break;
                        }

                        p = new Promise(function (resolve, reject) {
                          var stream = options.body.getStream();
                          stream.on('error', function (err) {
                            reject(new Error("".concat(err.message)));
                          });
                          stream.on('end', function () {
                            try {
                              resolve();
                            } catch (err) {
                              reject(new Error("".concat(err.message)));
                            }
                          });
                          stream.pipe(BrowserStdout({
                            req: req
                          }));
                        });
                        _context27.next = 21;
                        return p;

                      case 21:
                        _context27.next = 24;
                        break;

                      case 23:
                        req.write(options.body);

                      case 24:
                        req.end();

                      case 25:
                        req.on('error', function (err) {
                          ziti._ctx.logger.error('error EVENT: err: %o', err);

                          reject(new Error("request to ".concat(request.url, " failed, reason: ").concat(err.message)));
                        });
                        req.on('response', /*#__PURE__*/function () {
                          var _ref22 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee26(res) {
                            var body, response_options, response, hdr, cookieArray, cookiePath, expires, _httpOnly2, zitiCookies, i, cookie, name, value, cookie_value, parts, j, part;

                            return regeneratorRuntime.wrap(function _callee26$(_context26) {
                              while (1) {
                                switch (_context26.prev = _context26.next) {
                                  case 0:
                                    body = res.pipe(new PassThrough());
                                    response_options = {
                                      url: url,
                                      status: res.statusCode,
                                      statusText: res.statusMessage,
                                      headers: res.headers,
                                      size: request.size,
                                      timeout: request.timeout,
                                      counter: request.counter
                                    };
                                    response = new HttpResponse(body, response_options);
                                    _context26.t0 = regeneratorRuntime.keys(response_options.headers);

                                  case 4:
                                    if ((_context26.t1 = _context26.t0()).done) {
                                      _context26.next = 34;
                                      break;
                                    }

                                    hdr = _context26.t1.value;

                                    if (!response_options.headers.hasOwnProperty(hdr)) {
                                      _context26.next = 32;
                                      break;
                                    }

                                    if (!(hdr === 'set-cookie')) {
                                      _context26.next = 32;
                                      break;
                                    }

                                    cookieArray = response_options.headers[hdr];
                                    cookiePath = void 0;
                                    expires = void 0;
                                    _httpOnly2 = false;
                                    _context26.next = 14;
                                    return ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);

                                  case 14:
                                    zitiCookies = _context26.sent;

                                    if (isNull(zitiCookies)) {
                                      zitiCookies = {};
                                    }

                                    i = 0;

                                  case 17:
                                    if (!(i < cookieArray.length)) {
                                      _context26.next = 32;
                                      break;
                                    }

                                    cookie = cookieArray[i];
                                    name = cookie.substring(0, cookie.indexOf("="));
                                    value = cookie.substring(cookie.indexOf("=") + 1);
                                    cookie_value = value.substring(0, value.indexOf(";"));

                                    if (!(cookie_value !== '')) {
                                      _context26.next = 29;
                                      break;
                                    }

                                    parts = value.split(";");

                                    for (j = 0; j < parts.length; j++) {
                                      part = parts[j].trim();

                                      if (part.startsWith("Path")) {
                                        cookiePath = part.substring(part.indexOf("=") + 1);
                                      } else if (part.startsWith("Expires")) {
                                        expires = new Date(part.substring(part.indexOf("=") + 1));
                                      } else if (part.startsWith("HttpOnly")) {
                                        _httpOnly2 = true;
                                      }
                                    }

                                    zitiCookies[name] = cookie_value;
                                    _context26.next = 28;
                                    return ls.setWithExpiry(zitiConstants.get().ZITI_COOKIES, zitiCookies, new Date(8640000000000000));

                                  case 28:
                                    Cookies.set(name, cookie_value, {
                                      expires: expires,
                                      path: cookiePath
                                    });

                                  case 29:
                                    i++;
                                    _context26.next = 17;
                                    break;

                                  case 32:
                                    _context26.next = 4;
                                    break;

                                  case 34:
                                    resolve(response);

                                  case 35:
                                  case "end":
                                    return _context26.stop();
                                }
                              }
                            }, _callee26);
                          }));

                          return function (_x36) {
                            return _ref22.apply(this, arguments);
                          };
                        }());

                      case 27:
                      case "end":
                        return _context27.stop();
                    }
                  }
                }, _callee27);
              }));

              return function (_x34, _x35) {
                return _ref21.apply(this, arguments);
              };
            }()));

          case 80:
          case "end":
            return _context28.stop();
        }
      }
    }, _callee28);
  }));

  return function zitiFetch(_x26, _x27) {
    return _ref13.apply(this, arguments);
  };
}();
/**
 * 
 */


zitiDocumentInsertBefore = function zitiDocumentInsertBefore(elem, args) {// console.log('zitiDocumentInsertBefore(): ', elem);
};
/**
 * 
 */


zitiDocumentAppendChild = function zitiDocumentAppendChild(elem, args) {
  // console.log('zitiDocumentAppendChild() elem: ', elem);
  var transformed = false;

  if (elem[0].outerHTML) {
    var domHostsArray = zitiConfig.httpAgent.domProxy.hosts.split(',');
    forEach(domHostsArray, function (domHost) {
      var re = new RegExp(domHost, "gi");
      var redp = new RegExp("ziti-dom-proxy", "gi");
      var hit = (elem[0].outerHTML.match(re) || []).length;

      if (hit > 0) {
        // we see a hostname we need to transform
        hit = (elem[0].outerHTML.match(redp) || []).length;

        if (hit == 0) {
          // ...and we haven't been here before
          // Transform all occurances of the DOM Proxy hostname into a URL our sw can intercept
          var replace = zitiConfig.httpAgent.self.host + '/ziti-dom-proxy/' + domHost;

          try {
            if (!isUndefined(elem[0].src)) {
              var newSRC = elem[0].src.replace(re, replace);
              elem[0].src = newSRC;
              console.log('zitiDocumentAppendChild() TRANSFORMED: ', elem[0].outerHTML);
              transformed = true;
            }
          } catch (e) {
            console.error(e);
          }
        }
      }
    });

    if (!transformed) {
      // console.log('zitiDocumentAppendChild() NOT TRANSFORMED YET');
      var re = new RegExp("src\=\"\/", "gi");
      var hit = (elem[0].outerHTML.match(re) || []).length;

      if (hit > 0) {
        // we see a relative path
        console.log('zitiDocumentAppendChild() starts with SLASH: ', elem[0].outerHTML);
        var replace = 'src="https://' + zitiConfig.httpAgent.additionalTarget.host + '/';
        console.log('zitiDocumentAppendChild() additionalTarget: ', zitiConfig.httpAgent.additionalTarget.host);
        console.log('zitiDocumentAppendChild() replace: ', replace);
        var newSRC = elem[0].src.replace(re, replace);
        console.log('zitiDocumentAppendChild() newSRC: ', newSRC);
        elem[0].src = newSRC;
        console.log('zitiDocumentAppendChild() TRANSFORMED: ', elem[0].outerHTML);
      }
    }
  }
};
/**
 * 
 */


zitiDocumentSetAttribute = function zitiDocumentSetAttribute(elem, args) {// console.log('zitiDocumentSetAttribute(): ', elem, args);
};

if (!zitiConfig.serviceWorker.active) {
  window.fetch = zitiFetch;
  window.XMLHttpRequest = ZitiXMLHttpRequest;
  window.WebSocket = ZitiWebSocketWrapper;

  Element.prototype.insertBefore = function () {
    zitiDocumentInsertBefore.call(this, arguments);
    return window.realInsertBefore.apply(this, arguments);
  };

  Element.prototype.appendChild = function () {
    zitiDocumentAppendChild.call(this, arguments);
    return window.realAppendChild.apply(this, arguments);
  };

  Element.prototype.setAttribute = function () {
    zitiDocumentSetAttribute.call(this, arguments);
    return window.realSetAttribute.apply(this, arguments);
  };
}

if (typeof window !== 'undefined') {
  if (typeof window.fetch !== 'undefined') {
    window.fetch = zitiFetch;
    window.XMLHttpRequest = ZitiXMLHttpRequest;
    window.WebSocket = ZitiWebSocketWrapper;

    Element.prototype.insertBefore = function () {
      zitiDocumentInsertBefore.call(this, arguments);
      return window.realInsertBefore.apply(this, arguments);
    };

    Element.prototype.appendChild = function () {
      zitiDocumentAppendChild.call(this, arguments);
      return window.realAppendChild.apply(this, arguments);
    };

    Element.prototype.setAttribute = function () {
      zitiDocumentSetAttribute.call(this, arguments);
      return window.realSetAttribute.apply(this, arguments);
    };

    window.addEventListener('beforeunload', function (e) {
      if (!isUndefined(ziti._ctx)) {} //TEMP
      // purgeSensitiveValues();   // flush the IndexedDB
      // e.preventDefault(); // If you prevent default behavior in Mozilla Firefox prompt will always be shown
      // e.returnValue = '';       // Chrome requires returnValue to be set


      return undefined;
    });
  }
}
/**
 * 
 */


_sendResponse = function _sendResponse(event, responseObject) {
  var data = {
    command: event.data.command,
    // echo this back
    response: responseObject
  };
  event.ports[0].postMessage(data);
};
/**
 * 
 */


_onMessage_setControllerApi = /*#__PURE__*/function () {
  var _ref23 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee29(event) {
    return regeneratorRuntime.wrap(function _callee29$(_context29) {
      while (1) {
        switch (_context29.prev = _context29.next) {
          case 0:
            _context29.next = 2;
            return ls.setWithExpiry(zitiConstants.get().ZITI_CONTROLLER, event.data.controller, new Date(8640000000000000));

          case 2:
            _sendResponse(event, 'OK');

          case 3:
          case "end":
            return _context29.stop();
        }
      }
    }, _callee29);
  }));

  return function _onMessage_setControllerApi(_x37) {
    return _ref23.apply(this, arguments);
  };
}();
/**
 * 
 */


_internal_generateKeyPair = /*#__PURE__*/function () {
  var _ref24 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee30() {
    var pki;
    return regeneratorRuntime.wrap(function _callee30$(_context30) {
      while (1) {
        switch (_context30.prev = _context30.next) {
          case 0:
            pki = new ZitiPKI(ZitiPKI.prototype);
            _context30.next = 3;
            return pki.init({
              ctx: ziti._ctx,
              logger: ziti._ctx.logger
            });

          case 3:
            pki.generateKeyPair(); // initiate keypair calculation

          case 4:
          case "end":
            return _context30.stop();
        }
      }
    }, _callee30);
  }));

  return function _internal_generateKeyPair() {
    return _ref24.apply(this, arguments);
  };
}();

_onMessage_generateKeyPair = /*#__PURE__*/function () {
  var _ref25 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee31(event) {
    return regeneratorRuntime.wrap(function _callee31$(_context31) {
      while (1) {
        switch (_context31.prev = _context31.next) {
          case 0:
            // _internal_generateKeyPair();
            _sendResponse(event, 'OK');

          case 1:
          case "end":
            return _context31.stop();
        }
      }
    }, _callee31);
  }));

  return function _onMessage_generateKeyPair(_x38) {
    return _ref25.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_isKeyPairPresent = /*#__PURE__*/function () {
  var _ref26 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee32(event) {
    var pki, haveKeys;
    return regeneratorRuntime.wrap(function _callee32$(_context32) {
      while (1) {
        switch (_context32.prev = _context32.next) {
          case 0:
            pki = new ZitiPKI(ZitiPKI.prototype);
            _context32.next = 3;
            return pki._haveKeypair();

          case 3:
            haveKeys = _context32.sent;

            if (haveKeys) {
              _sendResponse(event, '1');
            } else {
              _sendResponse(event, '0');
            }

          case 5:
          case "end":
            return _context32.stop();
        }
      }
    }, _callee32);
  }));

  return function _onMessage_isKeyPairPresent(_x39) {
    return _ref26.apply(this, arguments);
  };
}();
/**
 * 
 */


_internal_isIdentityPresent = /*#__PURE__*/function () {
  var _ref27 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee34() {
    return regeneratorRuntime.wrap(function _callee34$(_context34) {
      while (1) {
        switch (_context34.prev = _context34.next) {
          case 0:
            return _context34.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref28 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee33(resolve, reject) {
                var identyPresent, apisess, cert;
                return regeneratorRuntime.wrap(function _callee33$(_context33) {
                  while (1) {
                    switch (_context33.prev = _context33.next) {
                      case 0:
                        identyPresent = false; // await ziti._serviceWorkerMutexNoTimeout.runExclusive(async () => {  // enter critical-section

                        _context33.next = 3;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                      case 3:
                        apisess = _context33.sent;

                        if (!(isNull(apisess) || isUndefined(apisess))) {
                          _context33.next = 11;
                          break;
                        }

                        _context33.next = 7;
                        return ls.removeItem(zitiConstants.get().ZITI_NETWORK_SESSIONS);

                      case 7:
                        _context33.next = 9;
                        return ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

                      case 9:
                        _context33.next = 23;
                        break;

                      case 11:
                        _context33.next = 13;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                      case 13:
                        cert = _context33.sent;

                        if (!(!isNull(cert) && !isUndefined(cert))) {
                          _context33.next = 18;
                          break;
                        }

                        identyPresent = true;
                        _context33.next = 23;
                        break;

                      case 18:
                        _context33.next = 20;
                        return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                      case 20:
                        _context33.next = 22;
                        return ls.removeItem(zitiConstants.get().ZITI_NETWORK_SESSIONS);

                      case 22:
                        // and also reset the channels
                        if (!isUndefined(ziti._ctx)) {
                          ziti._ctx.closeAllChannels();
                        }

                      case 23:
                        return _context33.abrupt("return", resolve(identyPresent));

                      case 24:
                      case "end":
                        return _context33.stop();
                    }
                  }
                }, _callee33);
              }));

              return function (_x40, _x41) {
                return _ref28.apply(this, arguments);
              };
            }()));

          case 1:
          case "end":
            return _context34.stop();
        }
      }
    }, _callee34);
  }));

  return function _internal_isIdentityPresent() {
    return _ref27.apply(this, arguments);
  };
}();

_onMessage_isIdentityPresent = /*#__PURE__*/function () {
  var _ref29 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee35(event) {
    var identyPresent;
    return regeneratorRuntime.wrap(function _callee35$(_context35) {
      while (1) {
        switch (_context35.prev = _context35.next) {
          case 0:
            _context35.next = 2;
            return _internal_isIdentityPresent();

          case 2:
            identyPresent = _context35.sent;

            if (identyPresent) {
              _sendResponse(event, '1');
            } else {
              _sendResponse(event, '0');
            }

          case 4:
          case "end":
            return _context35.stop();
        }
      }
    }, _callee35);
  }));

  return function _onMessage_isIdentityPresent(_x42) {
    return _ref29.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_promptForZitiCreds = /*#__PURE__*/function () {
  var _ref30 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee36(event) {
    var username, password, updb, pki;
    return regeneratorRuntime.wrap(function _callee36$(_context36) {
      while (1) {
        switch (_context36.prev = _context36.next) {
          case 0:
            _context36.next = 2;
            return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_USERNAME);

          case 2:
            username = _context36.sent;
            _context36.next = 5;
            return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_PASSWORD);

          case 5:
            password = _context36.sent;

            if (!(isNull(username) || isUndefined(username) || isNull(password) || isUndefined(password))) {
              _context36.next = 18;
              break;
            }

            updb = new ZitiUPDB(ZitiUPDB.prototype);
            _context36.next = 10;
            return updb.init({
              ctx: ziti._ctx,
              logger: ziti._ctx.logger
            });

          case 10:
            _context36.next = 12;
            return updb.awaitCredentialsAndAPISession();

          case 12:
            // Do not proceed until we have a keypair (this will render a dialog to the user informing them of status)
            pki = new ZitiPKI(ZitiPKI.prototype);
            _context36.next = 15;
            return pki.init({
              ctx: ziti._ctx,
              logger: ziti._ctx.logger
            });

          case 15:
            _context36.next = 17;
            return pki.awaitKeyPairGenerationComplete();

          case 17:
            // await completion of keypair calculation
            // Trigger a page reload now that we have creds and keypair
            setTimeout(function () {
              window.location.reload();
            }, 1000); // setTimeout(function(){ window.location.href = window.location.href }, 1000);  

          case 18:
            _sendResponse(event, 'OK');

          case 19:
          case "end":
            return _context36.stop();
        }
      }
    }, _callee36);
  }));

  return function _onMessage_promptForZitiCreds(_x43) {
    return _ref30.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_promptForZitiCredsNoWait = /*#__PURE__*/function () {
  var _ref31 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee38(event) {
    return regeneratorRuntime.wrap(function _callee38$(_context38) {
      while (1) {
        switch (_context38.prev = _context38.next) {
          case 0:
            _sendResponse(event, 'OK'); // release the sw immediately


            _context38.next = 3;
            return ziti._serviceWorkerMutexNoTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee37() {
              var ctx, apisess, cert, updb, haveKeypair, keypairPresent, _pki, haveCreds, validCreds, pki;

              return regeneratorRuntime.wrap(function _callee37$(_context37) {
                while (1) {
                  switch (_context37.prev = _context37.next) {
                    case 0:
                      if (!isUndefined(ziti._ctx)) {
                        _context37.next = 6;
                        break;
                      }

                      ctx = new ZitiContext(ZitiContext.prototype);
                      _context37.next = 4;
                      return ctx.initFromServiceWorker({
                        logLevel: LogLevel[event.data.options.logLevel]
                      });

                    case 4:
                      ctx.logger.success('JS SDK version %s (_onMessage_promptForZitiCredsNoWait) completed', pjson.version);
                      ziti._ctx = ctx;

                    case 6:
                      _context37.next = 8;
                      return ls.getWithExpiry(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                    case 8:
                      apisess = _context37.sent;
                      _context37.next = 11;
                      return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                    case 11:
                      cert = _context37.sent;

                      ziti._ctx.logger.debug('_onMessage_promptForZitiCredsNoWait: apisess is: %o, cert is: %o', apisess, cert);

                      if (!(isNull(apisess) || isUndefined(apisess) || isNull(cert) || isUndefined(cert))) {
                        _context37.next = 63;
                        break;
                      }

                      // Let sw know we do NOT have an identity
                      sendMessageToServiceworker({
                        command: 'identityLoaded',
                        identityLoaded: 0
                      });
                      updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context37.next = 18;
                      return updb.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 18:
                      _context37.next = 20;
                      return updb._haveKeypair();

                    case 20:
                      haveKeypair = _context37.sent;

                      if (haveKeypair) {
                        _context37.next = 42;
                        break;
                      }

                      _context37.next = 24;
                      return updb.awaitKeypair(zitiConstants.get().ZITI_IDENTITY_KEYPAIR_OBTAIN_FROM_FS);

                    case 24:
                      keypairPresent = _context37.sent;

                      ziti._ctx.logger.debug('_onMessage_promptForZitiCredsNoWait: awaitKeypair returned [%o]', keypairPresent);

                      if (!(keypairPresent == zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY_FILE_NOT_FOUND || keypairPresent == zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY_FILE_NOT_FOUND)) {
                        _context37.next = 42;
                        break;
                      }

                      // If keypair not in FS or IDB, then initiate keypair generation
                      _internal_generateKeyPair(); // Do not proceed until we have a keypair (this will render a dialog to the user informing them of status)


                      // Do not proceed until we have a keypair (this will render a dialog to the user informing them of status)
                      _pki = new ZitiPKI(ZitiPKI.prototype);
                      _context37.next = 31;
                      return _pki.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 31:
                      _context37.next = 33;
                      return _pki.awaitKeyPairGenerationComplete();

                    case 33:
                      _context37.next = 35;
                      return updb.awaitKeypair(zitiConstants.get().ZITI_IDENTITY_KEYPAIR_OBTAIN_FROM_IDB);

                    case 35:
                      keypairPresent = _context37.sent;

                      ziti._ctx.logger.debug('_onMessage_promptForZitiCredsNoWait: awaitKeypair returned [%o]', keypairPresent);

                      if (!(keypairPresent !== zitiConstants.get().ZITI_IDENTITY_KEYPAIR_FOUND)) {
                        _context37.next = 41;
                        break;
                      }

                      throw new Error('should not happen');

                    case 41:
                      ziti._ctx.logger.debug('_onMessage_promptForZitiCredsNoWait: now have the keypair we need, so proceeding to obtain creds');

                    case 42:
                      _context37.next = 44;
                      return updb._haveCreds();

                    case 44:
                      haveCreds = _context37.sent;

                      if (haveCreds) {
                        _context37.next = 50;
                        break;
                      }

                      _context37.next = 48;
                      return updb.awaitCredentialsAndAPISession();

                    case 48:
                      _context37.next = 54;
                      break;

                    case 50:
                      _context37.next = 52;
                      return ziti._ctx.getFreshAPISession();

                    case 52:
                      validCreds = _context37.sent;

                    case 53:
                      if (!validCreds) {
                        _context37.next = 50;
                        break;
                      }

                    case 54:
                      // Do not proceed until we have a keypair (this will render a dialog to the user informing them of status)
                      pki = new ZitiPKI(ZitiPKI.prototype);
                      _context37.next = 57;
                      return pki.init({
                        ctx: ziti._ctx,
                        logger: ziti._ctx.logger
                      });

                    case 57:
                      _context37.next = 59;
                      return pki.awaitKeyPairGenerationComplete();

                    case 59:
                      _context37.next = 61;
                      return ziti._ctx._awaitIdentityLoadComplete().catch(function (err) {
                        ziti._ctx.logger.error(err);
                      });

                    case 61:
                      // Let sw know we now have an identity
                      sendMessageToServiceworker({
                        command: 'identityLoaded',
                        identityLoaded: 1
                      }); // Trigger a page reload now that we have creds and keypair

                      // Trigger a page reload now that we have creds and keypair
                      setTimeout(function () {
                        ziti._ctx.logger.info('_onMessage_promptForZitiCredsNoWait: triggering page reload now');

                        window.location.reload();
                      }, 500);

                    case 63:
                    case "end":
                      return _context37.stop();
                  }
                }
              }, _callee37);
            })));

          case 3:
          case "end":
            return _context38.stop();
        }
      }
    }, _callee38);
  }));

  return function _onMessage_promptForZitiCredsNoWait(_x44) {
    return _ref31.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_initClient = /*#__PURE__*/function () {
  var _ref33 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee40(event) {
    return regeneratorRuntime.wrap(function _callee40$(_context40) {
      while (1) {
        switch (_context40.prev = _context40.next) {
          case 0:
            _context40.next = 2;
            return ziti._serviceWorkerMutexWithTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee39() {
              var ctx;
              return regeneratorRuntime.wrap(function _callee39$(_context39) {
                while (1) {
                  switch (_context39.prev = _context39.next) {
                    case 0:
                      if (!isUndefined(ziti._ctx)) {
                        _context39.next = 6;
                        break;
                      }

                      ctx = new ZitiContext(ZitiContext.prototype);
                      _context39.next = 4;
                      return ctx.initFromServiceWorker({
                        logLevel: LogLevel[event.data.options.logLevel]
                      });

                    case 4:
                      ctx.logger.success('JS SDK version %s initFromServiceWorker completed', pjson.version);
                      ziti._ctx = ctx;

                    case 6:
                    case "end":
                      return _context39.stop();
                  }
                }
              }, _callee39);
            }))).catch(function (err) {
              throw err;
            });

          case 2:
            _sendResponse(event, 'OK');

          case 3:
          case "end":
            return _context40.stop();
        }
      }
    }, _callee40);
  }));

  return function _onMessage_initClient(_x45) {
    return _ref33.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_purgeCert = /*#__PURE__*/function () {
  var _ref35 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee41(event) {
    return regeneratorRuntime.wrap(function _callee41$(_context41) {
      while (1) {
        switch (_context41.prev = _context41.next) {
          case 0:
            _context41.next = 2;
            return ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

          case 2:
            _context41.next = 4;
            return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

          case 4:
            _sendResponse(event, 'nop OK');

          case 5:
          case "end":
            return _context41.stop();
        }
      }
    }, _callee41);
  }));

  return function _onMessage_purgeCert(_x46) {
    return _ref35.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_getCookies = /*#__PURE__*/function () {
  var _ref36 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee42(event) {
    return regeneratorRuntime.wrap(function _callee42$(_context42) {
      while (1) {
        switch (_context42.prev = _context42.next) {
          case 0:
            console.log('JS SDK _onMessage_getCookies entered'); // if (ziti._cookiemutex.isLocked()) {
            // ziti._cookiemutex.waitForUnlock();
            // }
            // let zitiCookies = await ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);
            // console.log('JS SDK _onMessage_getCookies returning: ', zitiCookies);
            // _sendResponse( event, zitiCookies );

            _sendResponse(event, 'nop OK');

          case 2:
          case "end":
            return _context42.stop();
        }
      }
    }, _callee42);
  }));

  return function _onMessage_getCookies(_x47) {
    return _ref36.apply(this, arguments);
  };
}();
/**
 * 
 */


_onMessage_nop = /*#__PURE__*/function () {
  var _ref37 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee43(event) {
    return regeneratorRuntime.wrap(function _callee43$(_context43) {
      while (1) {
        switch (_context43.prev = _context43.next) {
          case 0:
            _sendResponse(event, 'nop OK');

          case 1:
          case "end":
            return _context43.stop();
        }
      }
    }, _callee43);
  }));

  return function _onMessage_nop(_x48) {
    return _ref37.apply(this, arguments);
  };
}();

if (!zitiConfig.serviceWorker.active) {
  if ('serviceWorker' in navigator) {
    if (isNull(navigator.serviceWorker.controller)) {
      /**
       *  Service Worker registration
       */
      navigator.serviceWorker.register('https://' + zitiConfig.httpAgent.self.host + '/ziti-sw.js', {
        scope: './'
      }).then(function (reg) {
        if (navigator.serviceWorker.controller) {
          // If .controller is set, then this page is being actively controlled by our service worker.
          console.log('The Ziti service worker is now registered.'); // (function checkForUpdatedServiceWorker() {
          //   console.log('checking for updated service worker.');
          //   reg.update();
          //   setTimeout( checkForUpdatedServiceWorker, 1000 * 60 * 30 );
          // })();
        } else {// If .controller isn't set, then prompt the user to reload the page so that the service worker can take
          // control. Until that happens, the service worker's fetch handler won't be used.
          // console.log('Please reload this page to allow the Ziti service worker to handle network operations.');
        }
      }).catch(function (error) {
        // Something went wrong during registration.
        console.error(error);
      });
    }
    /**
     *  Service Worker 'message' handler'
     */


    navigator.serviceWorker.addEventListener('message', function (event) {
      console.log('----- Client received msg from serviceWorker: ', event.data.command);

      if (event.data.command === 'initClient') {
        _onMessage_initClient(event);
      } else if (event.data.command === 'generateKeyPair') {
        _onMessage_generateKeyPair(event);
      } else if (event.data.command === 'setControllerApi') {
        _onMessage_setControllerApi(event);
      } else if (event.data.command === 'isKeyPairPresent') {
        _onMessage_isKeyPairPresent(event);
      } else if (event.data.command === 'isIdentityPresent') {
        _onMessage_isIdentityPresent(event);
      } else if (event.data.command === 'promptForZitiCreds') {
        _onMessage_promptForZitiCreds(event);
      } else if (event.data.command === 'promptForZitiCredsNoWait') {
        _onMessage_promptForZitiCredsNoWait(event);
      } else if (event.data.command === 'purgeCert') {
        _onMessage_purgeCert(event);
      } else if (event.data.command === 'getCookies') {
        _onMessage_getCookies(event);
      } else if (event.data.command === 'nop') {
        _onMessage_nop(event);
      } else {
        throw new Error('unknown message.command received [' + event.data.command + ']');
      }
    });
    /**
     * 
     */

    navigator.serviceWorker.startMessages(); // Kiosk mode?

    if (window.opener) {
      // wait for messages from opener
      window.addEventListener('message', function (e) {
        this.window.kioskZitiUserName = e.data.username;
        this.window.kioskZitiPassword = e.data.password;
      }); // tell the opener we are waiting

      window.opener.postMessage('inited', '*');
    }
  } else {
    console.error("The current browser doesn't support service workers");
  }
}
/**
 * 
 */


function sendMessageToServiceworker(_x49) {
  return _sendMessageToServiceworker.apply(this, arguments);
}
/**
 * 
 */


function _sendMessageToServiceworker() {
  _sendMessageToServiceworker = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee44(message) {
    return regeneratorRuntime.wrap(function _callee44$(_context44) {
      while (1) {
        switch (_context44.prev = _context44.next) {
          case 0:
            return _context44.abrupt("return", new Promise(function (resolve, reject) {
              var messageChannel = new MessageChannel();

              messageChannel.port1.onmessage = function (event) {
                if (event.data.error) {
                  reject(event.data.error);
                } else {
                  resolve(event.data);
                }
              };

              if (!isUndefined(navigator.serviceWorker)) {
                if (!isUndefined(navigator.serviceWorker.controller)) {
                  navigator.serviceWorker.controller.postMessage(message, [messageChannel.port2]);
                }
              }
            }));

          case 1:
          case "end":
            return _context44.stop();
        }
      }
    }, _callee44);
  }));
  return _sendMessageToServiceworker.apply(this, arguments);
}

function purgeSensitiveValues() {
  return _purgeSensitiveValues.apply(this, arguments);
}
/**
 * Propagate the cookies from the browser's cookie cache into the Ziti-owned storage.
 */


function _purgeSensitiveValues() {
  _purgeSensitiveValues = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee45() {
    return regeneratorRuntime.wrap(function _callee45$(_context45) {
      while (1) {
        switch (_context45.prev = _context45.next) {
          case 0:
            _context45.next = 2;
            return ls.removeItem(zitiConstants.get().ZITI_CONTROLLER);

          case 2:
            _context45.next = 4;
            return ls.removeItem(zitiConstants.get().ZITI_SERVICES);

          case 4:
            _context45.next = 6;
            return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

          case 6:
            _context45.next = 8;
            return ls.removeItem(zitiConstants.get().ZITI_NETWORK_SESSIONS);

          case 8:
            _context45.next = 10;
            return ls.removeItem(zitiConstants.get().ZITI_CLIENT_CERT_PEM);

          case 10:
            _context45.next = 12;
            return ls.removeItem(zitiConstants.get().ZITI_CLIENT_PRIVATE_KEY_PEM);

          case 12:
            _context45.next = 14;
            return ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

          case 14:
          case "end":
            return _context45.stop();
        }
      }
    }, _callee45);
  }));
  return _purgeSensitiveValues.apply(this, arguments);
}

function propagateBrowserCookieValues() {
  return _propagateBrowserCookieValues.apply(this, arguments);
}

function _propagateBrowserCookieValues() {
  _propagateBrowserCookieValues = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee46() {
    var zitiCookies, browserCookies, cookie;
    return regeneratorRuntime.wrap(function _callee46$(_context46) {
      while (1) {
        switch (_context46.prev = _context46.next) {
          case 0:
            _context46.next = 2;
            return ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);

          case 2:
            zitiCookies = _context46.sent;

            if (isNull(zitiCookies)) {
              zitiCookies = {};
            } // Obtain all Cookie KV pairs from the browser Cookie cache


            browserCookies = Cookies.get();

            for (cookie in browserCookies) {
              if (browserCookies.hasOwnProperty(cookie)) {
                zitiCookies[cookie] = browserCookies[cookie];
              }
            }

            _context46.next = 8;
            return ls.setWithExpiry(zitiConstants.get().ZITI_COOKIES, zitiCookies, new Date(8640000000000000));

          case 8:
          case "end":
            return _context46.stop();
        }
      }
    }, _callee46);
  }));
  return _propagateBrowserCookieValues.apply(this, arguments);
}

function purgeExpiredValues() {
  return _purgeExpiredValues.apply(this, arguments);
}

function _purgeExpiredValues() {
  _purgeExpiredValues = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee47() {
    return regeneratorRuntime.wrap(function _callee47$(_context47) {
      while (1) {
        switch (_context47.prev = _context47.next) {
          case 0:
            propagateBrowserCookieValues(); // await ls.getWithExpiry( zitiConstants.get().ZITI_CONTROLLER );               // The location of the Controller REST endpoint
            // await ls.getWithExpiry( zitiConstants.get().ZITI_SERVICES );                 // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_API_SESSION_TOKEN );        // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_NETWORK_SESSIONS );         // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_COOKIES );                  // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_CLIENT_CERT_PEM );          // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_CLIENT_PRIVATE_KEY_PEM );   // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_IDENTITY_CERT );            // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_IDENTITY_USERNAME );        // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_IDENTITY_PASSWORD );        // 
            // await ls.getWithExpiry( zitiConstants.get().ZITI_COOKIES );                  // 

            setTimeout(purgeExpiredValues, 1000 * 5); // pulse this function every few seconds

          case 2:
          case "end":
            return _context47.stop();
        }
      }
    }, _callee47);
  }));
  return _purgeExpiredValues.apply(this, arguments);
}

setTimeout(purgeExpiredValues, 1);