"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/web.timers.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.map.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/web.url.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.split.js");

require("core-js/modules/es.function.name.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var flatOptions = require('flat-options');

var consola = require('consola');

var find = require('lodash.find');

var filter = require('lodash.filter');

var isEqual = require('lodash.isequal');

var isNull = require('lodash.isnull');

var minby = require('lodash.minby');

var forEach = require('lodash.foreach');

var isUndefined = require('lodash.isundefined');

var result = require('lodash.result');

var has = require('lodash.has');

var Mutex = require('async-mutex');

var withTimeout = require('async-mutex').withTimeout;

var formatMessage = require('format-message');

var MicroModal;

if (!zitiConfig.serviceWorker.active) {
  MicroModal = require('micromodal');
}

var utils = require('../utils/utils');

var ls = require('../utils/localstorage');

var edge_protocol = require('../channel/protocol');

var defaultOptions = require('./options');

var contextTypes = require('./contexttypes');

var zitiConstants = require('../constants');

var ZitiReporter = require('../utils/ziti-reporter');

var ZitiControllerClient = require('../context/controller-client');

var ZitiUPDB = require('../updb/updb');

var ZitiChannel = require('../channel/channel');

var _require = require('../utils/throwif'),
    throwIf = _require.throwIf;

var ZitiEnroller;

if (!zitiConfig.serviceWorker.active) {
  ZitiEnroller = require('../enroll/enroller');
}

var pjson = require('../../package.json');

var _require2 = require('format-message'),
    selectordinal = _require2.selectordinal; // const EXPIRE_WINDOW = 28.0 // TEMP, for debugging


var EXPIRE_WINDOW = 2.0;
/**
 * Expose `ZitiContext`.
 */

module.exports = ZitiContext;
/**
 * Initialize a new `ZitiContext`.
 *
 * @api public
 */

function ZitiContext(obj) {
  var ctx = mixin(obj);
  return ctx;
}
/**
 * Mixin the prototype properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */


function mixin(obj) {
  for (var key in ZitiContext.prototype) {
    if (Object.prototype.hasOwnProperty.call(ZitiContext.prototype, key)) obj[key] = ZitiContext.prototype[key];
  }

  return obj;
}
/**
 * Load the Identity.
 *
 */


ZitiContext.prototype.loadAPISessionToken = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(self) {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            return _context2.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(resolve, reject) {
                var apisess;
                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        self.logger.debug('loadAPISessionToken() starting');
                        _context.next = 3;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                      case 3:
                        apisess = _context.sent;
                        self.logger.debug('loadAPISessionToken() ZITI_API_SESSION_TOKEN is: [%o]', apisess);

                        if (!isNull(apisess)) {
                          self._apiSession = apisess; // Set the token header on behalf of all subsequent Controller API calls

                          // Set the token header on behalf of all subsequent Controller API calls
                          self._controllerClient.setApiKey(self._apiSession.token, 'zt-session', false);
                        }

                        setTimeout(self.apiSessionHeartbeat, 1000 * 60 * 5, self);
                        resolve();

                      case 8:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x2, _x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 1:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();

ZitiContext.prototype.haveRequiredVariables = function (self) {
  if (isNull(self._IDENTITY_CERT) || isUndefined(self._IDENTITY_CERT) || isNull(self._IDENTITY_KEY) || isUndefined(self._IDENTITY_KEY)) {
    return false;
  } else {
    return true;
  }
};
/**
 * Load the Identity.
 *
 */


ZitiContext.prototype.loadIdentity = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(self) {
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            return _context4.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(resolve, reject) {
                var enroller;
                return regeneratorRuntime.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        self.logger.debug('loadIdentity() starting'); // Load Identity variables from storage

                        _context3.next = 3;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                      case 3:
                        self._IDENTITY_CERT = _context3.sent;
                        _context3.next = 6;
                        return ls.get(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY);

                      case 6:
                        self._IDENTITY_KEY = _context3.sent;

                        if (self.haveRequiredVariables(self)) {
                          _context3.next = 21;
                          break;
                        }

                        if (!(self.contextType == contextTypes.ServiceWorkerType)) {
                          _context3.next = 10;
                          break;
                        }

                        return _context3.abrupt("return", reject('service worker cannot perform identity load; must be done from client'));

                      case 10:
                        enroller = new ZitiEnroller(ZitiEnroller.prototype);
                        enroller.init({
                          ctx: self,
                          logger: self.logger
                        });
                        _context3.next = 14;
                        return enroller.enroll().catch(function (e) {
                          self.logger.error('Enrollment failed: [%o]', e);
                          ls.removeItem(zitiConstants.get().ZITI_JWT);
                          return reject('Enrollment failed');
                        });

                      case 14:
                        self.logger.debug('enroll() completed successfully'); // Now that enrollment completed successfully, reload Identity

                        _context3.next = 17;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                      case 17:
                        self._IDENTITY_CERT = _context3.sent;
                        _context3.next = 20;
                        return ls.get(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY);

                      case 20:
                        self._IDENTITY_KEY = _context3.sent;

                      case 21:
                        return _context3.abrupt("return", resolve());

                      case 22:
                      case "end":
                        return _context3.stop();
                    }
                  }
                }, _callee3);
              }));

              return function (_x5, _x6) {
                return _ref4.apply(this, arguments);
              };
            }()));

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function (_x4) {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * Return a Promise that will resolve as soon as we have the location of the Ziti Controller.
 *
 * @returns {Promise}   
 */


ZitiContext.prototype._awaitIdentityLoadComplete = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
  var _this = this;

  var self;
  return regeneratorRuntime.wrap(function _callee6$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          self = this;
          return _context6.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(resolve, reject) {
              var startTime;
              return regeneratorRuntime.wrap(function _callee5$(_context5) {
                while (1) {
                  switch (_context5.prev = _context5.next) {
                    case 0:
                      self.logger.debug('awaitIdentityLoadComplete() starting');
                      _context5.next = 3;
                      return _this.loadIdentity(self).catch(function (e) {
                        return reject(e);
                      });

                    case 3:
                      self.logger.debug('loadIdentity() returned');
                      startTime = new Date();

                      (function waitForIdentityLoadComplete() {
                        if (self.haveRequiredVariables(self)) {
                          if (self._modalIsOpen) {
                            MicroModal.close('ziti-updb-modal');
                            self._modalIsOpen = false;
                          }

                          return resolve();
                        }

                        self.logger.debug('awaitIdentityLoadComplete() identity still not loaded');
                        var now = new Date();
                        var elapsed = now.getTime() - startTime.getTime();

                        if (elapsed > 1000 * 60) {
                          return reject('Identity not specified');
                        } else {
                          setTimeout(waitForIdentityLoadComplete, 500);
                        }
                      })();

                    case 6:
                    case "end":
                      return _context5.stop();
                  }
                }
              }, _callee5);
            }));

            return function (_x7, _x8) {
              return _ref6.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee6, this);
}));
ZitiContext.prototype._awaitIdentityLoadCompleteFromServiceWorker = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
  var self;
  return regeneratorRuntime.wrap(function _callee7$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          self = this;
          console.log('_awaitIdentityLoadCompleteFromServiceWorker() entered');
          self.logger.debug('_awaitIdentityLoadCompleteFromServiceWorker() entered');
          return _context7.abrupt("return", new Promise(function (resolve, reject) {
            var startTime = new Date();

            (function waitForIdentityLoadComplete() {
              if (self.haveRequiredVariables(self)) {
                console.log('_awaitIdentityLoadCompleteFromServiceWorker() resolving with: ', self._ztAPI);
                self.logger.debug('_awaitIdentityLoadCompleteFromServiceWorker() resolving with: [%s]', self._ztAPI);
                resolve(self._ztAPI);
                return;
              }

              self.logger.debug('_awaitIdentityLoadCompleteFromServiceWorker() identity still not loaded');
              var now = new Date();
              var elapsed = now.getTime() - startTime.getTime();

              if (elapsed > 1000 * 60) {
                self.logger.error('_awaitIdentityLoadCompleteFromServiceWorker() identity never acquired');
                return reject();
              } else {
                setTimeout(waitForIdentityLoadComplete, 500);
              }
            })();
          }));

        case 4:
        case "end":
          return _context7.stop();
      }
    }
  }, _callee7, this);
}));
/**
 * 
 *
 */

ZitiContext.prototype.getFreshAPISession = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9() {
  var self;
  return regeneratorRuntime.wrap(function _callee9$(_context9) {
    while (1) {
      switch (_context9.prev = _context9.next) {
        case 0:
          self = this;
          return _context9.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(resolve, reject) {
              var res;
              return regeneratorRuntime.wrap(function _callee8$(_context8) {
                while (1) {
                  switch (_context8.prev = _context8.next) {
                    case 0:
                      self.logger.debug('ctx.getFreshAPISession() entered'); // Get an API session with Controller

                      _context8.next = 3;
                      return self._controllerClient.authenticate({
                        method: 'password',
                        body: {
                          username: self._loginFormValues.username,
                          password: self._loginFormValues.password,
                          configTypes: ['ziti-tunneler-client.v1', 'intercept.v1'],
                          // envInfo: {
                          // arch: window.navigator.platform,    // e.g. 'MacIntel'
                          // os: window.navigator.appVersion,    // e.g. '5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36'
                          // },
                          sdkInfo: {
                            // branch: "string",
                            // revision: "string",
                            type: 'ziti-sdk-js',
                            version: pjson.version
                          }
                        }
                      });

                    case 3:
                      res = _context8.sent;

                      if (isUndefined(res.error)) {
                        _context8.next = 7;
                        break;
                      }

                      self.logger.error(res.error.message);
                      return _context8.abrupt("return", resolve(false));

                    case 7:
                      self._apiSession = res.data;
                      self.logger.debug('ctx.getFreshAPISession(): _apiSession[%o]', self._apiSession); // Set the token header on behalf of all subsequent Controller API calls

                      // Set the token header on behalf of all subsequent Controller API calls
                      self._controllerClient.setApiKey(self._apiSession.token, 'zt-session', false);

                      _context8.next = 12;
                      return ls.setWithExpiry(zitiConstants.get().ZITI_API_SESSION_TOKEN, self._apiSession, new Date(Date.parse(self._apiSession.expiresAt)));

                    case 12:
                      return _context8.abrupt("return", resolve(true));

                    case 13:
                    case "end":
                      return _context8.stop();
                  }
                }
              }, _callee8);
            }));

            return function (_x9, _x10) {
              return _ref9.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context9.stop();
      }
    }
  }, _callee9, this);
}));

ZitiContext.prototype.apiSessionHeartbeat = /*#__PURE__*/function () {
  var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(self) {
    var res;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return self._controllerClient.getCurrentAPISession({});

          case 2:
            res = _context10.sent;

            if (isUndefined(res.data)) {
              _context10.next = 9;
              break;
            }

            self._apiSession = res.data;
            self.logger.debug('ctx.apiSessionHeartbeat(): _apiSession[%o]', self._apiSession);

            if (!isUndefined(self._controllerClient)) {
              self._controllerClient.setApiKey(self._apiSession.token, 'zt-session', false);
            }

            _context10.next = 9;
            return ls.setWithExpiry(zitiConstants.get().ZITI_API_SESSION_TOKEN, self._apiSession, new Date(Date.parse(self._apiSession.expiresAt)));

          case 9:
            setTimeout(self.apiSessionHeartbeat, 1000 * 60 * 5, self);

          case 10:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));

  return function (_x11) {
    return _ref10.apply(this, arguments);
  };
}();
/**
 * 
 *
 */


ZitiContext.prototype.ensureAPISession = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
  var self;
  return regeneratorRuntime.wrap(function _callee12$(_context12) {
    while (1) {
      switch (_context12.prev = _context12.next) {
        case 0:
          self = this;
          self.logger.debug('ctx.ensureAPISession: entered: self._apiSession[%o]', self._apiSession);
          return _context12.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref12 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(resolve, reject) {
              var validCreds, updb, now, expiresAt, diffTime, diffMins, _validCreds, _updb;

              return regeneratorRuntime.wrap(function _callee11$(_context11) {
                while (1) {
                  switch (_context11.prev = _context11.next) {
                    case 0:
                      if (!isUndefined(self._apiSession)) {
                        _context11.next = 13;
                        break;
                      }

                      self.logger.debug('ctx.ensureAPISession: calling getFreshAPISession()');
                      _context11.next = 4;
                      return self.getFreshAPISession();

                    case 4:
                      validCreds = _context11.sent;

                      if (validCreds) {
                        _context11.next = 11;
                        break;
                      }

                      self.logger.debug('ctx.ensureAPISession: getFreshAPISession() failed');
                      updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context11.next = 10;
                      return updb.init({
                        ctx: self,
                        logger: self.logger
                      });

                    case 10:
                      updb.awaitCredentialsAndAPISession();

                    case 11:
                      _context11.next = 29;
                      break;

                    case 13:
                      now = Date.now();
                      expiresAt = Date.parse(self._apiSession.expiresAt);
                      diffTime = expiresAt - now;
                      diffMins = diffTime / (1000 * 60);
                      self.logger.debug('ctx.ensureAPISession: mins before apiSession expiration [%o]', diffMins);

                      if (!(diffMins < 5.0)) {
                        _context11.next = 29;
                        break;
                      }

                      // if expired, or about to expire
                      self.logger.debug('ctx.ensureAPISession: calling getFreshAPISession()');
                      _context11.next = 22;
                      return self.getFreshAPISession();

                    case 22:
                      _validCreds = _context11.sent;

                      if (_validCreds) {
                        _context11.next = 29;
                        break;
                      }

                      self.logger.debug('ctx.ensureAPISession: getFreshAPISession() failed');
                      _updb = new ZitiUPDB(ZitiUPDB.prototype);
                      _context11.next = 28;
                      return _updb.init({
                        ctx: self,
                        logger: self.logger
                      });

                    case 28:
                      _updb.awaitCredentialsAndAPISession();

                    case 29:
                      return _context11.abrupt("return", resolve(true));

                    case 30:
                    case "end":
                      return _context11.stop();
                  }
                }
              }, _callee11);
            }));

            return function (_x12, _x13) {
              return _ref12.apply(this, arguments);
            };
          }()));

        case 3:
        case "end":
          return _context12.stop();
      }
    }
  }, _callee12, this);
}));
/**
 * 
 *
 */

ZitiContext.prototype.flushExpiredAPISessionData = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13() {
  return regeneratorRuntime.wrap(function _callee13$(_context13) {
    while (1) {
      switch (_context13.prev = _context13.next) {
        case 0:
          _context13.next = 2;
          return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

        case 2:
          _context13.next = 4;
          return ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

        case 4:
          this._apiSession = undefined;

        case 5:
        case "end":
          return _context13.stop();
      }
    }
  }, _callee13, this);
}));
/**
 * 
 *
 */

ZitiContext.prototype.isAPISessionExpired = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee15() {
  var self;
  return regeneratorRuntime.wrap(function _callee15$(_context15) {
    while (1) {
      switch (_context15.prev = _context15.next) {
        case 0:
          self = this;
          return _context15.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref15 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14(resolve, reject) {
              var now, expiresAt, diffTime, diffMins;
              return regeneratorRuntime.wrap(function _callee14$(_context14) {
                while (1) {
                  switch (_context14.prev = _context14.next) {
                    case 0:
                      if (!isUndefined(self._apiSession)) {
                        _context14.next = 5;
                        break;
                      }

                      self.flushExpiredAPISessionData();
                      return _context14.abrupt("return", resolve(true));

                    case 5:
                      now = Date.now();
                      expiresAt = Date.parse(self._apiSession.expiresAt);
                      diffTime = expiresAt - now;
                      diffMins = diffTime / (1000 * 60);
                      self.logger.debug('ctx.isAPISessionExpired: mins before apiSession expiration [%o]', diffMins);

                      if (!(diffMins < EXPIRE_WINDOW)) {
                        _context14.next = 15;
                        break;
                      }

                      // if expired, or about to expire
                      self.flushExpiredAPISessionData();
                      return _context14.abrupt("return", resolve(true));

                    case 15:
                      return _context14.abrupt("return", resolve(false));

                    case 16:
                    case "end":
                      return _context14.stop();
                  }
                }
              }, _callee14);
            }));

            return function (_x14, _x15) {
              return _ref15.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context15.stop();
      }
    }
  }, _callee15, this);
}));
/**
 * 
 *
 */

ZitiContext.prototype.isIdentityCertExpired = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee17() {
  var self;
  return regeneratorRuntime.wrap(function _callee17$(_context17) {
    while (1) {
      switch (_context17.prev = _context17.next) {
        case 0:
          self = this;
          return _context17.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref17 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee16(resolve, reject) {
              var certExpiry, now, diffTime, diffMins;
              return regeneratorRuntime.wrap(function _callee16$(_context16) {
                while (1) {
                  switch (_context16.prev = _context16.next) {
                    case 0:
                      _context16.next = 2;
                      return ls.getExpiry(zitiConstants.get().ZITI_IDENTITY_CERT);

                    case 2:
                      certExpiry = _context16.sent;

                      if (!isNull(certExpiry)) {
                        _context16.next = 8;
                        break;
                      }

                      self.flushExpiredAPISessionData();
                      return _context16.abrupt("return", resolve(true));

                    case 8:
                      now = Date.now();
                      diffTime = certExpiry - now;
                      diffMins = diffTime / (1000 * 60);
                      self.logger.debug('ctx.isIdentityCertExpired: mins before cert expiration [%o]', diffMins);

                      if (!(diffMins < EXPIRE_WINDOW)) {
                        _context16.next = 17;
                        break;
                      }

                      // if expired, or about to expire
                      self.flushExpiredAPISessionData();
                      return _context16.abrupt("return", resolve(true));

                    case 17:
                      return _context16.abrupt("return", resolve(false));

                    case 18:
                    case "end":
                      return _context16.stop();
                  }
                }
              }, _callee16);
            }));

            return function (_x16, _x17) {
              return _ref17.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context17.stop();
      }
    }
  }, _callee17, this);
}));
/**
 * Initialize the Ziti Context.
 * 
 * Tasks:
 * - validate options
 * - create logger
 * - load the Ziti Identity
 * - create Controller client
 * - get Controller version
 * - establish an API session with Controller
 * - fetch list of active services from Controller
 *
 * @param {Object} [options]
 * @returns {nothing}   
 */

ZitiContext.prototype.init = /*#__PURE__*/function () {
  var _ref18 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee19(options) {
    var self;
    return regeneratorRuntime.wrap(function _callee19$(_context19) {
      while (1) {
        switch (_context19.prev = _context19.next) {
          case 0:
            self = this;
            return _context19.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref19 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee18(resolve, reject) {
                var _options, domain, network_sessions, services;

                return regeneratorRuntime.wrap(function _callee18$(_context18) {
                  while (1) {
                    switch (_context18.prev = _context18.next) {
                      case 0:
                        _options = flatOptions(options, defaultOptions);
                        self.contextType = _options.contextType;
                        self.logger = consola.create({
                          level: _options.logLevel,
                          reporters: [new ZitiReporter()],
                          defaults: {
                            additionalColor: 'white'
                          }
                        });
                        self.logger.wrapConsole();
                        domain = window.zitiConfig.controller.api;
                        self._controllerClient = new ZitiControllerClient({
                          domain: domain,
                          logger: self.logger
                        }); // Get Controller version info
                        // let res = await self._controllerClient.listVersion();
                        // self._controllerVersion = res.data;
                        // self.logger.info('init Controller Version: [%o]', self._controllerVersion);

                        // Get Controller version info
                        // let res = await self._controllerClient.listVersion();
                        // self._controllerVersion = res.data;
                        // self.logger.info('init Controller Version: [%o]', self._controllerVersion);
                        self._timeout = zitiConstants.get().ZITI_DEFAULT_TIMEOUT;
                        self._network_sessions = new Map();
                        _context18.next = 10;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_NETWORK_SESSIONS);

                      case 10:
                        network_sessions = _context18.sent;

                        if (!isNull(network_sessions)) {
                          self._network_sessions = network_sessions;
                        }

                        self._services = new Map();
                        self._channels = new Map();
                        self._channelSeq = 0;
                        self._connSeq = 0;
                        self._loginFormValues = {};
                        _context18.next = 19;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_USERNAME);

                      case 19:
                        self._loginFormValues.username = _context18.sent;
                        self.logger.info('ZITI_IDENTITY_USERNAME: [%o]', self._loginFormValues.username);
                        _context18.next = 23;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_PASSWORD);

                      case 23:
                        self._loginFormValues.password = _context18.sent;
                        self.logger.info('ZITI_IDENTITY_PASSWORD: [%o]', self._loginFormValues.password);
                        self._mutex = new Mutex.Mutex();
                        self._connectMutexWithTimeout = withTimeout(new Mutex.Mutex(), 30000);
                        _context18.next = 29;
                        return self.loadAPISessionToken(self);

                      case 29:
                        _context18.next = 31;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_SERVICES);

                      case 31:
                        services = _context18.sent;

                        if (!isNull(services)) {
                          self._services = services;
                        }

                        resolve();

                      case 34:
                      case "end":
                        return _context18.stop();
                    }
                  }
                }, _callee18);
              }));

              return function (_x19, _x20) {
                return _ref19.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context19.stop();
        }
      }
    }, _callee19, this);
  }));

  return function (_x18) {
    return _ref18.apply(this, arguments);
  };
}();
/**
 * Initialize the Ziti Context.
 * 
 * Tasks:
 * - validate options
 * - create logger
 * - load the Ziti Identity
 * - create Controller client
 * - get Controller version
 * - establish an API session with Controller
 * - fetch list of active services from Controller
 *
 * @param {Object} [options]
 * @returns {nothing}   
 */


ZitiContext.prototype.initFromServiceWorker = /*#__PURE__*/function () {
  var _ref20 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee21(options) {
    var self;
    return regeneratorRuntime.wrap(function _callee21$(_context21) {
      while (1) {
        switch (_context21.prev = _context21.next) {
          case 0:
            self = this;
            return _context21.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref21 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee20(resolve, reject) {
                var _options, domain, network_sessions, services;

                return regeneratorRuntime.wrap(function _callee20$(_context20) {
                  while (1) {
                    switch (_context20.prev = _context20.next) {
                      case 0:
                        _options = flatOptions(options, defaultOptions);
                        self.contextType = _options.contextType;
                        self.logger = consola.create({
                          level: _options.logLevel,
                          reporters: [new ZitiReporter()],
                          defaults: {
                            additionalColor: 'white'
                          }
                        });
                        domain = zitiConfig.controller.api;
                        self._controllerClient = new ZitiControllerClient({
                          domain: domain,
                          logger: self.logger
                        }); // Get Controller version info
                        // let res = await self._controllerClient.listVersion();
                        // self._controllerVersion = res.data;
                        // self.logger.info('initFromServiceWorker Controller Version: [%o]', self._controllerVersion);

                        // Get Controller version info
                        // let res = await self._controllerClient.listVersion();
                        // self._controllerVersion = res.data;
                        // self.logger.info('initFromServiceWorker Controller Version: [%o]', self._controllerVersion);
                        self._timeout = zitiConstants.get().ZITI_DEFAULT_TIMEOUT;
                        self._network_sessions = new Map();
                        _context20.next = 9;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_NETWORK_SESSIONS);

                      case 9:
                        network_sessions = _context20.sent;

                        if (!isNull(network_sessions)) {
                          self._network_sessions = network_sessions;
                        }

                        self._services = new Map();
                        self._channels = new Map();
                        self._channelSeq = 0;
                        self._connSeq = 0;
                        self._loginFormValues = {};
                        _context20.next = 18;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_USERNAME);

                      case 18:
                        self._loginFormValues.username = _context20.sent;
                        self.logger.info('ZITI_IDENTITY_USERNAME: [%o]', self._loginFormValues.username);
                        _context20.next = 22;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_IDENTITY_PASSWORD);

                      case 22:
                        self._loginFormValues.password = _context20.sent;
                        self.logger.info('ZITI_IDENTITY_PASSWORD: [%o]', self._loginFormValues.password);
                        self._mutex = new Mutex.Mutex();
                        self._connectMutexWithTimeout = withTimeout(new Mutex.Mutex(), 30000);
                        _context20.next = 28;
                        return self.loadAPISessionToken(self);

                      case 28:
                        _context20.next = 30;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_SERVICES);

                      case 30:
                        services = _context20.sent;

                        if (!isNull(services)) {
                          self._services = services;
                        }

                        resolve();

                      case 33:
                      case "end":
                        return _context20.stop();
                    }
                  }
                }, _callee20);
              }));

              return function (_x22, _x23) {
                return _ref21.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context21.stop();
        }
      }
    }, _callee21, this);
  }));

  return function (_x21) {
    return _ref20.apply(this, arguments);
  };
}();

ZitiContext.prototype.setLoginFormValues = /*#__PURE__*/function () {
  var _ref22 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee22(loginFormValues) {
    return regeneratorRuntime.wrap(function _callee22$(_context22) {
      while (1) {
        switch (_context22.prev = _context22.next) {
          case 0:
            this._loginFormValues = loginFormValues;

          case 1:
          case "end":
            return _context22.stop();
        }
      }
    }, _callee22, this);
  }));

  return function (_x24) {
    return _ref22.apply(this, arguments);
  };
}();

ZitiContext.prototype.getLoginFormValues = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee23() {
  return regeneratorRuntime.wrap(function _callee23$(_context23) {
    while (1) {
      switch (_context23.prev = _context23.next) {
        case 0:
          return _context23.abrupt("return", this._loginFormValues);

        case 1:
        case "end":
          return _context23.stop();
      }
    }
  }, _callee23, this);
}));
ZitiContext.prototype.ensure = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee24() {
  return regeneratorRuntime.wrap(function _callee24$(_context24) {
    while (1) {
      switch (_context24.prev = _context24.next) {
        case 0:
          return _context24.abrupt("return", this._loginFormValues);

        case 1:
        case "end":
          return _context24.stop();
      }
    }
  }, _callee24, this);
}));
ZitiContext.prototype.fetchServices = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee26() {
  var self;
  return regeneratorRuntime.wrap(function _callee26$(_context26) {
    while (1) {
      switch (_context26.prev = _context26.next) {
        case 0:
          self = this;
          return _context26.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref26 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee25(resolve, reject) {
              return regeneratorRuntime.wrap(function _callee25$(_context25) {
                while (1) {
                  switch (_context25.prev = _context25.next) {
                    case 0:
                      _context25.next = 2;
                      return self.ensureAPISession();

                    case 2:
                      _context25.next = 4;
                      return self._controllerClient.listServices({
                        limit: '100'
                      });

                    case 4:
                      res = _context25.sent;

                      if (isUndefined(res.error)) {
                        _context25.next = 8;
                        break;
                      }

                      self.logger.error(res.error);
                      return _context25.abrupt("return", reject(res.error));

                    case 8:
                      self._services = res.data;
                      self.logger.debug('List of available Services acquired: [%o]', self._services);
                      _context25.t0 = ls;
                      _context25.t1 = zitiConstants.get().ZITI_SERVICES;
                      _context25.t2 = self._services;
                      _context25.next = 15;
                      return ls.getWithExpiry(zitiConstants.get().ZITI_EXPIRY_TIME);

                    case 15:
                      _context25.t3 = _context25.sent;
                      _context25.next = 18;
                      return _context25.t0.setWithExpiry.call(_context25.t0, _context25.t1, _context25.t2, _context25.t3);

                    case 18:
                      resolve();

                    case 19:
                    case "end":
                      return _context25.stop();
                  }
                }
              }, _callee25);
            }));

            return function (_x25, _x26) {
              return _ref26.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context26.stop();
      }
    }
  }, _callee26, this);
}));
/**
"config": {
  "ziti-tunneler-client.v1": {
      "ziti-tunneler-client.v1": {
        "hostname": "example.com",
        "port": 443
      }
  }
}
*/

ZitiContext.prototype._getMatchConfigTunnelerClientV1 = function (config, hostname, port) {
  if (isUndefined(config)) {
    return false;
  }

  if (config.hostname !== hostname) {
    return false;
  }

  if (config.port !== port) {
    return false;
  }

  return true;
};
/**
"config": {
  "intercept.v1": {
    "addresses": ["example.com"],
    "portRanges": [{
      "high": 443,
      "low": 443
    }],
    "protocols": ["tcp"]
  }
}
*/


ZitiContext.prototype._getMatchConfigInterceptV1 = function (config, hostname, port) {
  if (isUndefined(config)) {
    return false;
  }

  var foundAddress = result(find(config.addresses, function (address) {
    if (address !== hostname) {
      return false;
    }

    return true;
  }), 'name', 'default');

  if (!foundAddress) {
    return false;
  }

  var foundPort = result(find(config.portRanges, function (portRange) {
    if (port >= portRange.low && port <= portRange.high) {
      return true;
    }

    return false;
  }), 'name', 'default');
  return foundPort;
};

ZitiContext.prototype.getServiceNameByHostNameAndPort = /*#__PURE__*/function () {
  var _ref27 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee28(hostname, port) {
    var self;
    return regeneratorRuntime.wrap(function _callee28$(_context28) {
      while (1) {
        switch (_context28.prev = _context28.next) {
          case 0:
            self = this;
            return _context28.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref28 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee27(resolve, reject) {
                var release, serviceName;
                return regeneratorRuntime.wrap(function _callee27$(_context27) {
                  while (1) {
                    switch (_context27.prev = _context27.next) {
                      case 0:
                        if (typeof port === 'string') {
                          port = parseInt(port, 10);
                        }

                        _context27.next = 3;
                        return self._mutex.acquire();

                      case 3:
                        release = _context27.sent;

                        if (!isEqual(self.getServices().size, 0)) {
                          _context27.next = 7;
                          break;
                        }

                        _context27.next = 7;
                        return self.fetchServices().catch(function (error) {
                          release();
                          return reject(error);
                        });

                      case 7:
                        release();
                        serviceName = result(find(self._services, function (obj) {
                          if (self._getMatchConfigTunnelerClientV1(obj.config['ziti-tunneler-client.v1'], hostname, port)) {
                            return true;
                          }

                          return self._getMatchConfigInterceptV1(obj.config['intercept.v1'], hostname, port);
                        }), 'name');
                        resolve(serviceName);

                      case 10:
                      case "end":
                        return _context27.stop();
                    }
                  }
                }, _callee27);
              }));

              return function (_x29, _x30) {
                return _ref28.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context28.stop();
        }
      }
    }, _callee28, this);
  }));

  return function (_x27, _x28) {
    return _ref27.apply(this, arguments);
  };
}();
/**
 * Connect specified ZitiConnection to the nearest Edge Router.
 * 
 * @param {Array} edgeRouters
 * @returns {Promise}
 */


ZitiContext.prototype._getPendingChannelConnects = /*#__PURE__*/function () {
  var _ref29 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee31(conn, edgeRouters) {
    var _this2 = this;

    return regeneratorRuntime.wrap(function _callee31$(_context31) {
      while (1) {
        switch (_context31.prev = _context31.next) {
          case 0:
            return _context31.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref30 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee30(resolve) {
                var pendingChannelConnects, self;
                return regeneratorRuntime.wrap(function _callee30$(_context30) {
                  while (1) {
                    switch (_context30.prev = _context30.next) {
                      case 0:
                        pendingChannelConnects = new Array();
                        self = _this2; // Get a channel connection to each of the Edge Routers that have a WS binding, initiating a connection if channel is not yet connected

                        // Get a channel connection to each of the Edge Routers that have a WS binding, initiating a connection if channel is not yet connected
                        edgeRouters.forEach( /*#__PURE__*/function () {
                          var _ref31 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee29(edgeRouter, idx, array) {
                            var ch;
                            return regeneratorRuntime.wrap(function _callee29$(_context29) {
                              while (1) {
                                switch (_context29.prev = _context29.next) {
                                  case 0:
                                    _context29.next = 2;
                                    return self.getChannelByEdgeRouter(conn, edgeRouter);

                                  case 2:
                                    ch = _context29.sent;
                                    self.logger.debug('initiating Hello to [%s] for session[%s]', edgeRouter.urls.ws, conn.token);
                                    pendingChannelConnects.push(ch.hello()); // }      

                                    // }      
                                    if (idx === array.length - 1) {
                                      resolve(pendingChannelConnects); // Return to caller only after we have processed all edge routers
                                    }

                                  case 6:
                                  case "end":
                                    return _context29.stop();
                                }
                              }
                            }, _callee29);
                          }));

                          return function (_x34, _x35, _x36) {
                            return _ref31.apply(this, arguments);
                          };
                        }());

                      case 3:
                      case "end":
                        return _context30.stop();
                    }
                  }
                }, _callee30);
              }));

              return function (_x33) {
                return _ref30.apply(this, arguments);
              };
            }()));

          case 1:
          case "end":
            return _context31.stop();
        }
      }
    }, _callee31);
  }));

  return function (_x31, _x32) {
    return _ref29.apply(this, arguments);
  };
}();
/**
 * Connect specified ZitiConnection to the nearest Edge Router.
 * 
 * @param {ZitiConnection} conn
 * @param {ZitiConnection} networkSession
 * @returns {bool}
 */


ZitiContext.prototype.connect = /*#__PURE__*/function () {
  var _ref32 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee34(conn, networkSession) {
    var self;
    return regeneratorRuntime.wrap(function _callee34$(_context34) {
      while (1) {
        switch (_context34.prev = _context34.next) {
          case 0:
            self = this;
            return _context34.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref33 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee33(resolve, reject) {
                var edgeRouters;
                return regeneratorRuntime.wrap(function _callee33$(_context33) {
                  while (1) {
                    switch (_context33.prev = _context33.next) {
                      case 0:
                        self.logger.debug('contextType[%d] connect() entered for conn[%o]', self.contextType, conn.getId()); // If we were not given a networkSession, it most likely means something (an API token, Cert, etc) expired,
                        // so we need to purge them and re-acquire

                        if (!(isNull(networkSession) || isUndefined(networkSession))) {
                          _context33.next = 9;
                          break;
                        }

                        self.logger.debug('ctx.connect invoked with undefined networkSession');
                        ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);
                        ls.removeItem(zitiConstants.get().ZITI_IDENTITY_CERT);

                        if (!(self.contextType == contextTypes.ServiceWorkerType)) {
                          _context33.next = 7;
                          break;
                        }

                        return _context33.abrupt("return", reject('service worker cannot perform identity load; must be done from client'));

                      case 7:
                        _context33.next = 9;
                        return self._awaitIdentityLoadComplete().catch(function (err) {
                          self.logger.error(err);
                          return reject(err);
                        });

                      case 9:
                        conn.token = networkSession.token; // Get list of all Edge Router URLs where the Edge Router has a WS binding

                        // Get list of all Edge Router URLs where the Edge Router has a WS binding
                        edgeRouters = filter(networkSession.edgeRouters, function (o) {
                          return has(o, 'urls.ws');
                        }); // Something is wrong if we have no ws-enabled edge routers

                        if (!isEqual(edgeRouters.length, 0)) {
                          _context33.next = 13;
                          break;
                        }

                        return _context33.abrupt("return", reject(new Error('No Edge Routers with ws: binding were found')));

                      case 13:
                        //
                        self.logger.debug('contextType[%d] trying to acquire _connectMutex for conn[%o]', self.contextType, conn.getId());
                        _context33.next = 16;
                        return self._connectMutexWithTimeout.runExclusive( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee32() {
                          var pendingChannelConnects, channelConnects, channelConnectWithNearestEdgeRouter, channelWithNearestEdgeRouter;
                          return regeneratorRuntime.wrap(function _callee32$(_context32) {
                            while (1) {
                              switch (_context32.prev = _context32.next) {
                                case 0:
                                  self.logger.debug('contextType[%d] now own _connectMutex for conn[%o]', self.contextType, conn.getId());
                                  _context32.next = 3;
                                  return self._getPendingChannelConnects(conn, edgeRouters);

                                case 3:
                                  pendingChannelConnects = _context32.sent;
                                  self.logger.trace('pendingChannelConnects [%o]', pendingChannelConnects);
                                  _context32.next = 7;
                                  return Promise.all(pendingChannelConnects);

                                case 7:
                                  channelConnects = _context32.sent;
                                  self.logger.trace('channelConnects [%o]', channelConnects); // Select channel with nearest Edge Router. Heuristic: select one with earliest Hello-handshake completion timestamp

                                  // Select channel with nearest Edge Router. Heuristic: select one with earliest Hello-handshake completion timestamp
                                  channelConnectWithNearestEdgeRouter = minby(channelConnects, function (channelConnect) {
                                    return channelConnect.channel.helloCompletedTimestamp;
                                  });
                                  channelWithNearestEdgeRouter = channelConnectWithNearestEdgeRouter.channel;
                                  self.logger.debug('Channel [%d] has nearest Edge Router for conn[%o]', channelWithNearestEdgeRouter.getId(), conn.getId());

                                  channelWithNearestEdgeRouter._connections._saveConnection(conn);

                                  conn.setChannel(channelWithNearestEdgeRouter); // Initiate connection with Edge Router (creates Fabric session)

                                  _context32.next = 16;
                                  return channelWithNearestEdgeRouter.connect(conn);

                                case 16:
                                  if (!(conn.getState() == edge_protocol.conn_state.Connected)) {
                                    _context32.next = 20;
                                    break;
                                  }

                                  if (!conn.getEncrypted()) {
                                    _context32.next = 20;
                                    break;
                                  }

                                  _context32.next = 20;
                                  return channelWithNearestEdgeRouter.awaitConnectionCryptoEstablishComplete(conn);

                                case 20:
                                  self.logger.debug('contextType[%d] releasing _connectMutex for conn[%o]', self.contextType, conn.getId());

                                case 21:
                                case "end":
                                  return _context32.stop();
                              }
                            }
                          }, _callee32);
                        }))).catch(function (err) {
                          ziti._ctx.logger.error('contextType[%d] failed to acquire _connectMutex for conn[%o]: %o', self.contextType, conn.getId(), err);

                          reject(err);
                        });

                      case 16:
                        return _context33.abrupt("return", resolve());

                      case 17:
                      case "end":
                        return _context33.stop();
                    }
                  }
                }, _callee33);
              }));

              return function (_x39, _x40) {
                return _ref33.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context34.stop();
        }
      }
    }, _callee34, this);
  }));

  return function (_x37, _x38) {
    return _ref32.apply(this, arguments);
  };
}();
/**
 * Close specified ZitiConnection with Edge Router.
 * 
 * @param {ZitiConnection} conn
 * @returns {bool}
 */


ZitiContext.prototype.close = /*#__PURE__*/function () {
  var _ref35 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee36(conn) {
    return regeneratorRuntime.wrap(function _callee36$(_context36) {
      while (1) {
        switch (_context36.prev = _context36.next) {
          case 0:
            return _context36.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref36 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee35(resolve, reject) {
                var ch;
                return regeneratorRuntime.wrap(function _callee35$(_context35) {
                  while (1) {
                    switch (_context35.prev = _context35.next) {
                      case 0:
                        ch = conn.getChannel();
                        _context35.next = 3;
                        return ch.close(conn);

                      case 3:
                        ch._connections._deleteConnection(conn);

                        resolve();

                      case 5:
                      case "end":
                        return _context35.stop();
                    }
                  }
                }, _callee35);
              }));

              return function (_x42, _x43) {
                return _ref36.apply(this, arguments);
              };
            }()));

          case 1:
          case "end":
            return _context36.stop();
        }
      }
    }, _callee36);
  }));

  return function (_x41) {
    return _ref35.apply(this, arguments);
  };
}();

ZitiContext.prototype._getChannelForConnId = function (id) {
  var ch = find(this._channels, function (ch) {
    return !isUndefined(ch._connections._getConnection(id));
  });
  return ch;
};
/**
 * Determine if the given URL should be routed over Ziti.
 * 
 * @returns {bool}
 */


ZitiContext.prototype.shouldRouteOverZiti = /*#__PURE__*/function () {
  var _ref37 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee38(url) {
    var _this3 = this;

    var parsedURL, hostname, port, self;
    return regeneratorRuntime.wrap(function _callee38$(_context38) {
      while (1) {
        switch (_context38.prev = _context38.next) {
          case 0:
            parsedURL = new URL(url);
            hostname = parsedURL.hostname;
            port = parsedURL.port;

            if (port === '') {
              if (parsedURL.protocol === 'https:' || parsedURL.protocol === 'wss:') {
                port = 443;
              } else {
                port = 80;
              }
            }

            self = this;
            return _context38.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref38 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee37(resolve, reject) {
                var serviceName;
                return regeneratorRuntime.wrap(function _callee37$(_context37) {
                  while (1) {
                    switch (_context37.prev = _context37.next) {
                      case 0:
                        _context37.next = 2;
                        return _this3.getServiceNameByHostNameAndPort(hostname, port).catch(function (error) {
                          return reject(error);
                        });

                      case 2:
                        serviceName = _context37.sent;
                        return _context37.abrupt("return", resolve(serviceName));

                      case 4:
                      case "end":
                        return _context37.stop();
                    }
                  }
                }, _callee37);
              }));

              return function (_x45, _x46) {
                return _ref38.apply(this, arguments);
              };
            }()));

          case 6:
          case "end":
            return _context38.stop();
        }
      }
    }, _callee38, this);
  }));

  return function (_x44) {
    return _ref37.apply(this, arguments);
  };
}();
/**
 * Determine if the given URL should be handled via CORS Proxy.
 * 
 * @returns {bool}
 */


ZitiContext.prototype.shouldRouteOverCORSProxy = /*#__PURE__*/function () {
  var _ref39 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee40(url) {
    var parsedURL, hostname, port, corsHostsArray;
    return regeneratorRuntime.wrap(function _callee40$(_context40) {
      while (1) {
        switch (_context40.prev = _context40.next) {
          case 0:
            parsedURL = new URL(url);
            hostname = parsedURL.hostname;
            port = parsedURL.port;

            if (port === '') {
              if (parsedURL.protocol === 'https:' || parsedURL.protocol === 'wss:') {
                port = 443;
              } else {
                port = 80;
              }
            }

            corsHostsArray = zitiConfig.httpAgent.corsProxy.hosts.split(',');
            return _context40.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref40 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee39(resolve, reject) {
                var routeOverCORSProxy;
                return regeneratorRuntime.wrap(function _callee39$(_context39) {
                  while (1) {
                    switch (_context39.prev = _context39.next) {
                      case 0:
                        routeOverCORSProxy = false;
                        forEach(corsHostsArray, function (corsHost) {
                          var corsHostSplit = corsHost.split(':');

                          if (hostname === corsHostSplit[0] && port === parseInt(corsHostSplit[1], 10)) {
                            routeOverCORSProxy = true;
                          }
                        });
                        return _context39.abrupt("return", resolve(routeOverCORSProxy));

                      case 3:
                      case "end":
                        return _context39.stop();
                    }
                  }
                }, _callee39);
              }));

              return function (_x48, _x49) {
                return _ref40.apply(this, arguments);
              };
            }()));

          case 6:
          case "end":
            return _context40.stop();
        }
      }
    }, _callee40);
  }));

  return function (_x47) {
    return _ref39.apply(this, arguments);
  };
}();
/**
 * Determine if the given URL should be handled via DOM Proxy.
 * 
 * @returns {bool}
 */


ZitiContext.prototype.shouldRouteOverDOMProxy = /*#__PURE__*/function () {
  var _ref41 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee42(url) {
    var parsedURL, hostname, port, corsHostsArray;
    return regeneratorRuntime.wrap(function _callee42$(_context42) {
      while (1) {
        switch (_context42.prev = _context42.next) {
          case 0:
            parsedURL = new URL(url);
            hostname = parsedURL.hostname;
            port = parsedURL.port;

            if (port === '') {
              if (parsedURL.protocol === 'https:' || parsedURL.protocol === 'wss:') {
                port = 443;
              } else {
                port = 80;
              }
            }

            corsHostsArray = zitiConfig.httpAgent.domProxy.hosts.split(',');
            return _context42.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref42 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee41(resolve, reject) {
                var routeOverCORSProxy;
                return regeneratorRuntime.wrap(function _callee41$(_context41) {
                  while (1) {
                    switch (_context41.prev = _context41.next) {
                      case 0:
                        routeOverCORSProxy = false;
                        forEach(corsHostsArray, function (corsHost) {
                          var corsHostSplit = corsHost.split(':');

                          if (hostname === corsHostSplit[0] && port === parseInt(corsHostSplit[1], 10)) {
                            routeOverCORSProxy = true;
                          }
                        });
                        return _context41.abrupt("return", resolve(routeOverCORSProxy));

                      case 3:
                      case "end":
                        return _context41.stop();
                    }
                  }
                }, _callee41);
              }));

              return function (_x51, _x52) {
                return _ref42.apply(this, arguments);
              };
            }()));

          case 6:
          case "end":
            return _context42.stop();
        }
      }
    }, _callee42);
  }));

  return function (_x50) {
    return _ref41.apply(this, arguments);
  };
}();
/**
 * Return current value of the ztAPI
 *
 * @returns {undefined | string}
 */


ZitiContext.prototype.getZtAPI = function () {
  return this._ztAPI;
};
/**
 * Return current value of the ztWSAPI
 *
 * @returns {undefined | string}
 */


ZitiContext.prototype.getZtWSAPI = function () {
  return this._ztWSAPI;
};
/**
 * Return current value of the ztAPI
 *
 * @returns {undefined | string}
 */


ZitiContext.prototype.getTimeout = function () {
  return this._timeout;
};

ZitiContext.prototype.getNextChannelId = function () {
  this._channelSeq++;
  return this._channelSeq;
};

ZitiContext.prototype.getNextConnectionId = function () {
  this._connSeq++;
  return this._connSeq;
};

ZitiContext.prototype.getApiSessionToken = function () {
  return this._apiSession.token;
};

ZitiContext.prototype.getControllerVersion = function () {
  return this._controllerVersion;
};

ZitiContext.prototype.getServices = function () {
  return this._services;
};

ZitiContext.prototype.getServiceIdByName = function (name) {
  var service_id = result(find(this._services, function (obj) {
    return obj.name === name;
  }), 'id');
  this.logger.debug('service[%s] has id[%s]', name, service_id);
  return service_id;
};

ZitiContext.prototype.getServiceEncryptionRequiredByName = function (name) {
  var encryptionRequired = result(find(this._services, function (obj) {
    return obj.name === name;
  }), 'encryptionRequired');
  this.logger.debug('service[%s] has encryptionRequired[%o]', name, encryptionRequired);
  return encryptionRequired;
};
/**
 * Remain in lazy-sleepy loop until specified channel is connected.
 * 
 * @param {*} conn 
 */


ZitiContext.prototype.awaitControllerChannelConnectComplete = function (ch) {
  return new Promise(function (resolve) {
    (function waitForControllerChannelConnectComplete() {
      if (isEqual(ch.getState(), edge_protocol.conn_state.Initial) || isEqual(ch.getState(), edge_protocol.conn_state.Connecting)) {
        ch._ctx.logger.trace('waitForControllerChannelConnectComplete() ch[%d] still not yet connected', ch.getId());

        setTimeout(waitForControllerChannelConnectComplete, 100);
      } else {
        ch._ctx.logger.trace('controller ch[%d] is connected', ch.getId());

        return resolve();
      }
    })();
  });
};

ZitiContext.prototype.getControllerChannel = function () {
  var _this4 = this;

  return new Promise( /*#__PURE__*/function () {
    var _ref43 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee43(resolve) {
      var ch;
      return regeneratorRuntime.wrap(function _callee43$(_context43) {
        while (1) {
          switch (_context43.prev = _context43.next) {
            case 0:
              ch = _this4._channels.get(edgeRouter.hostname);

              if (isUndefined(ch)) {
                _context43.next = 8;
                break;
              }

              _this4.logger.debug('ch[%d] state[%d] found for ingress[%s]', ch.getId(), ch.getState(), edgeRouter.hostname);

              _context43.next = 5;
              return _this4.awaitChannelConnectComplete(ch);

            case 5:
              if (!isEqual(ch.getState(), edge_protocol.conn_state.Connected)) {
                _this4.logger.error('should not be here: ch[%d] has state[%d]', ch.getId(), ch.getState());
              }

              resolve(ch);
              return _context43.abrupt("return");

            case 8:
              // Create a Channel for this Edge Router
              ch = new ZitiChannel({
                ctx: _this4,
                edgeRouter: edgeRouter,
                session_token: _this4._apiSession.token,
                network_session_token: conn.token
              });
              ch.setState(edge_protocol.conn_state.Connecting);

              _this4.logger.debug('Created ch[%o] ', ch);

              _this4._channels.set(edgeRouter.hostname, ch);

              resolve(ch);

            case 13:
            case "end":
              return _context43.stop();
          }
        }
      }, _callee43);
    }));

    return function (_x53) {
      return _ref43.apply(this, arguments);
    };
  }());
};
/**
 * Remain in lazy-sleepy loop until specified channel is connected.
 * 
 * @param {*} channel 
 */


ZitiContext.prototype.awaitChannelConnectComplete = function (ch) {
  return new Promise(function (resolve) {
    (function waitForChannelConnectComplete() {
      if (isEqual(ch.getState(), edge_protocol.conn_state.Initial) || isEqual(ch.getState(), edge_protocol.conn_state.Connecting)) {
        ch._ctx.logger.trace('awaitChannelConnectComplete() ch[%d] still not yet connected', ch.getId());

        setTimeout(waitForChannelConnectComplete, 100);
      } else {
        ch._ctx.logger.trace('ch[%d] is connected', ch.getId());

        return resolve();
      }
    })();
  });
};

ZitiContext.prototype.getChannelByEdgeRouter = function (conn, edgeRouter) {
  var _this5 = this;

  return new Promise( /*#__PURE__*/function () {
    var _ref44 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee44(resolve) {
      var key, ch;
      return regeneratorRuntime.wrap(function _callee44$(_context44) {
        while (1) {
          switch (_context44.prev = _context44.next) {
            case 0:
              key = edgeRouter.hostname + '-' + conn.token;
              ch = _this5._channels.get(key);

              if (isUndefined(ch)) {
                _context44.next = 9;
                break;
              }

              _this5.logger.debug('ch[%d] state[%d] found for ingress[%s]', ch.getId(), ch.getState(), edgeRouter.hostname);

              _context44.next = 6;
              return _this5.awaitChannelConnectComplete(ch);

            case 6:
              if (!isEqual(ch.getState(), edge_protocol.conn_state.Connected)) {
                _this5.logger.error('should not be here: ch[%d] has state[%d]', ch.getId(), ch.getState());
              }

              resolve(ch);
              return _context44.abrupt("return");

            case 9:
              // Create a Channel for this Edge Router
              ch = new ZitiChannel({
                ctx: _this5,
                edgeRouter: edgeRouter,
                session_token: _this5._apiSession.token,
                network_session_token: conn.token
              });
              ch.setState(edge_protocol.conn_state.Connecting);

              _this5.logger.debug('Created ch[%o] ', ch);

              _this5._channels.set(key, ch);

              resolve(ch);

            case 14:
            case "end":
              return _context44.stop();
          }
        }
      }, _callee44);
    }));

    return function (_x54) {
      return _ref44.apply(this, arguments);
    };
  }());
};

ZitiContext.prototype.closeChannelByEdgeRouter = function (edgeRouter) {
  this._channels.delete(edgeRouter);
};

ZitiContext.prototype.closeAllChannels = function () {
  this._channels = new Map();
};

ZitiContext.prototype.getServiceIdByDNSHostName = function (name) {
  var service_id = result(find(this._services, function (obj) {
    return obj.dns.hostname === name;
  }), 'id');
  return service_id;
};

ZitiContext.prototype.getNetworkSessionByServiceId = /*#__PURE__*/function () {
  var _ref45 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee46(serviceID) {
    var self;
    return regeneratorRuntime.wrap(function _callee46$(_context46) {
      while (1) {
        switch (_context46.prev = _context46.next) {
          case 0:
            self = this;
            return _context46.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref46 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee45(resolve, reject) {
                var release;
                return regeneratorRuntime.wrap(function _callee45$(_context45) {
                  while (1) {
                    switch (_context45.prev = _context45.next) {
                      case 0:
                        _context45.next = 2;
                        return self._mutex.acquire();

                      case 2:
                        release = _context45.sent;

                        if (self._network_sessions.has(serviceID)) {
                          _context45.next = 17;
                          break;
                        }

                        _context45.next = 6;
                        return self.createNetworkSession(serviceID).catch(function (e) {
                          /* ignore */
                        });

                      case 6:
                        network_session = _context45.sent;

                        if (!isUndefined(network_session)) {
                          self.logger.debug('Created new network_session [%o] ', network_session);
                        }

                        self._network_sessions.set(serviceID, network_session);

                        _context45.t0 = ls;
                        _context45.t1 = zitiConstants.get().ZITI_NETWORK_SESSIONS;
                        _context45.t2 = self._network_sessions;
                        _context45.next = 14;
                        return ls.getWithExpiry(zitiConstants.get().ZITI_EXPIRY_TIME);

                      case 14:
                        _context45.t3 = _context45.sent;
                        _context45.next = 17;
                        return _context45.t0.setWithExpiry.call(_context45.t0, _context45.t1, _context45.t2, _context45.t3);

                      case 17:
                        release();
                        resolve(self._network_sessions.get(serviceID));

                      case 19:
                      case "end":
                        return _context45.stop();
                    }
                  }
                }, _callee45);
              }));

              return function (_x56, _x57) {
                return _ref46.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context46.stop();
        }
      }
    }, _callee46, this);
  }));

  return function (_x55) {
    return _ref45.apply(this, arguments);
  };
}();

ZitiContext.prototype.createNetworkSession = /*#__PURE__*/function () {
  var _ref47 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee48(id) {
    var _this6 = this;

    var self;
    return regeneratorRuntime.wrap(function _callee48$(_context48) {
      while (1) {
        switch (_context48.prev = _context48.next) {
          case 0:
            self = this;
            return _context48.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref48 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee47(resolve, reject) {
                var network_session;
                return regeneratorRuntime.wrap(function _callee47$(_context47) {
                  while (1) {
                    switch (_context47.prev = _context47.next) {
                      case 0:
                        _context47.next = 2;
                        return _this6._controllerClient.createSession({
                          body: {
                            serviceId: id
                          },
                          headers: {
                            // 'zt-session': this._apiSession.token,
                            'Content-Type': 'application/json'
                          }
                        });

                      case 2:
                        res = _context47.sent;

                        if (isUndefined(res.error)) {
                          _context47.next = 6;
                          break;
                        }

                        self.logger.error(res.error.message);
                        return _context47.abrupt("return", reject(res.error.message));

                      case 6:
                        network_session = res.data;
                        resolve(network_session);

                      case 8:
                      case "end":
                        return _context47.stop();
                    }
                  }
                }, _callee47);
              }));

              return function (_x59, _x60) {
                return _ref48.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context48.stop();
        }
      }
    }, _callee48, this);
  }));

  return function (_x58) {
    return _ref47.apply(this, arguments);
  };
}();