/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
'use strict';

require("core-js/modules/es.object.keys.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.array.filter.js");

require("core-js/modules/es.object.get-own-property-descriptor.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.object.get-own-property-descriptors.js");

require("core-js/modules/es.object.set-prototype-of.js");

require("core-js/modules/es.array.slice.js");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Stream = require('readable-stream');

function readStart(socket) {
  if (socket && !socket._paused && socket.readable) socket.resume();
}

function readStop(socket) {
  if (socket) socket.pause();
}
/* Abstract base class for ClientResponse. */


function IncomingMessage(socket) {
  var streamOptions;

  if (socket) {
    streamOptions = {
      highWaterMark: socket.readableHighWaterMark
    };
  }

  Stream.Readable.call(this, _objectSpread({
    autoDestroy: false
  }, streamOptions));
  this._readableState.readingMore = true;
  this.socket = socket;
  this.httpVersionMajor = null;
  this.httpVersionMinor = null;
  this.httpVersion = null;
  this.complete = false;
  this.headers = {};
  this.rawHeaders = [];
  this.trailers = {};
  this.rawTrailers = [];
  this.aborted = false;
  this.upgrade = null; // request (server) only

  this.url = '';
  this.method = null; // response (client) only

  this.statusCode = null;
  this.statusMessage = null;
  this.client = socket;
  this._consuming = false; // Flag for when we decide that this message cannot possibly be
  // read by the user, so there's no point continuing to handle it.

  this._dumped = false;
}

Object.setPrototypeOf(IncomingMessage.prototype, Stream.Readable.prototype);
Object.setPrototypeOf(IncomingMessage, Stream.Readable);
Object.defineProperty(IncomingMessage.prototype, 'connection', {
  get: function get() {
    return this.socket;
  },
  set: function set(val) {
    this.socket = val;
  }
});

IncomingMessage.prototype.setTimeout = function setTimeout(msecs, callback) {
  if (callback) this.on('timeout', callback);
  this.socket.setTimeout(msecs);
  return this;
};

IncomingMessage.prototype._read = function _read(n) {
  if (!this._consuming) {
    this._readableState.readingMore = false;
    this._consuming = true;
  } // We actually do almost nothing here, because the parserOnBody
  // function fills up our internal buffer directly.  However, we
  // do need to unpause the underlying socket so that it flows.


  if (this.socket.readable) readStart(this.socket);
}; // It's possible that the socket will be destroyed, and removed from
// any messages, before ever calling this.  In that case, just skip
// it, since something else is destroying this connection anyway.


IncomingMessage.prototype.destroy = function destroy(error) {
  // TODO(ronag): Implement in terms of _destroy
  this.destroyed = true;
  if (this.socket) this.socket.destroy(error);
  return this;
};

IncomingMessage.prototype._addHeaderLines = _addHeaderLines;

function _addHeaderLines(headers, n) {
  if (headers && headers.length) {
    var dest;

    if (this.complete) {
      this.rawTrailers = headers;
      dest = this.trailers;
    } else {
      this.rawHeaders = headers;
      dest = this.headers;
    }

    for (var i = 0; i < n; i += 2) {
      this._addHeaderLine(headers[i], headers[i + 1], dest);
    }
  }
} // This function is used to help avoid the lowercasing of a field name if it
// matches a 'traditional cased' version of a field name. It then returns the
// lowercased name to both avoid calling toLowerCase() a second time and to
// indicate whether the field was a 'no duplicates' field. If a field is not a
// 'no duplicates' field, a `0` byte is prepended as a flag. The one exception
// to this is the Set-Cookie header which is indicated by a `1` byte flag, since
// it is an 'array' field and thus is treated differently in _addHeaderLines().
// TODO: perhaps http_parser could be returning both raw and lowercased versions
// of known header names to avoid us having to call toLowerCase() for those
// headers.


function matchKnownFields(field, lowercased) {
  switch (field.length) {
    case 3:
      if (field === 'Age' || field === 'age') return 'age';
      break;

    case 4:
      if (field === 'Host' || field === 'host') return 'host';
      if (field === 'From' || field === 'from') return 'from';
      if (field === 'ETag' || field === 'etag') return 'etag';
      if (field === 'Date' || field === 'date') return "\0date";
      if (field === 'Vary' || field === 'vary') return "\0vary";
      break;

    case 6:
      if (field === 'Server' || field === 'server') return 'server';
      if (field === 'Cookie' || field === 'cookie') return "\x02cookie";
      if (field === 'Origin' || field === 'origin') return "\0origin";
      if (field === 'Expect' || field === 'expect') return "\0expect";
      if (field === 'Accept' || field === 'accept') return "\0accept";
      break;

    case 7:
      if (field === 'Referer' || field === 'referer') return 'referer';
      if (field === 'Expires' || field === 'expires') return 'expires';
      if (field === 'Upgrade' || field === 'upgrade') return "\0upgrade";
      break;

    case 8:
      if (field === 'Location' || field === 'location') return 'location';
      if (field === 'If-Match' || field === 'if-match') return "\0if-match";
      break;

    case 10:
      if (field === 'User-Agent' || field === 'user-agent') return 'user-agent';
      if (field === 'Set-Cookie' || field === 'set-cookie') return "\x01";
      if (field === 'Connection' || field === 'connection') return "\0connection";
      break;

    case 11:
      if (field === 'Retry-After' || field === 'retry-after') return 'retry-after';
      break;

    case 12:
      if (field === 'Content-Type' || field === 'content-type') return 'content-type';
      if (field === 'Max-Forwards' || field === 'max-forwards') return 'max-forwards';
      break;

    case 13:
      if (field === 'Authorization' || field === 'authorization') return 'authorization';
      if (field === 'Last-Modified' || field === 'last-modified') return 'last-modified';
      if (field === 'Cache-Control' || field === 'cache-control') return "\0cache-control";
      if (field === 'If-None-Match' || field === 'if-none-match') return "\0if-none-match";
      break;

    case 14:
      if (field === 'Content-Length' || field === 'content-length') return 'content-length';
      break;

    case 15:
      if (field === 'Accept-Encoding' || field === 'accept-encoding') return "\0accept-encoding";
      if (field === 'Accept-Language' || field === 'accept-language') return "\0accept-language";
      if (field === 'X-Forwarded-For' || field === 'x-forwarded-for') return "\0x-forwarded-for";
      break;

    case 16:
      if (field === 'Content-Encoding' || field === 'content-encoding') return "\0content-encoding";
      if (field === 'X-Forwarded-Host' || field === 'x-forwarded-host') return "\0x-forwarded-host";
      break;

    case 17:
      if (field === 'If-Modified-Since' || field === 'if-modified-since') return 'if-modified-since';
      if (field === 'Transfer-Encoding' || field === 'transfer-encoding') return "\0transfer-encoding";
      if (field === 'X-Forwarded-Proto' || field === 'x-forwarded-proto') return "\0x-forwarded-proto";
      break;

    case 19:
      if (field === 'Proxy-Authorization' || field === 'proxy-authorization') return 'proxy-authorization';
      if (field === 'If-Unmodified-Since' || field === 'if-unmodified-since') return 'if-unmodified-since';
      break;
  }

  if (lowercased) {
    return "\0" + field;
  }

  return matchKnownFields(field.toLowerCase(), true);
} // Add the given (field, value) pair to the message
//
// Per RFC2616, section 4.2 it is acceptable to join multiple instances of the
// same header with a ', ' if the header in question supports specification of
// multiple values this way. The one exception to this is the Cookie header,
// which has multiple values joined with a '; ' instead. If a header's values
// cannot be joined in either of these ways, we declare the first instance the
// winner and drop the second. Extended header fields (those beginning with
// 'x-') are always joined.


IncomingMessage.prototype._addHeaderLine = _addHeaderLine;

function _addHeaderLine(field, value, dest) {
  field = matchKnownFields(field);
  var flag = field.charCodeAt(0);

  if (flag === 0 || flag === 2) {
    field = field.slice(1); // Make a delimited list

    if (typeof dest[field] === 'string') {
      dest[field] += (flag === 0 ? ', ' : '; ') + value;
    } else {
      dest[field] = value;
    }
  } else if (flag === 1) {
    // Array header -- only Set-Cookie at the moment
    if (dest['set-cookie'] !== undefined) {
      dest['set-cookie'].push(value);
    } else {
      dest['set-cookie'] = [value];
    }
  } else if (dest[field] === undefined) {
    // Drop duplicates
    dest[field] = value;
  }
} // Call this instead of resume() if we want to just
// dump all the data to /dev/null


IncomingMessage.prototype._dump = function _dump() {
  if (!this._dumped) {
    this._dumped = true; // If there is buffered data, it may trigger 'data' events.
    // Remove 'data' event listeners explicitly.

    this.removeAllListeners('data');
    this.resume();
  }
};

module.exports = {
  IncomingMessage: IncomingMessage,
  readStart: readStart,
  readStop: readStop
};