/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
'use strict';

require("core-js/modules/es.array.slice.js");

require("core-js/modules/es.function.name.js");

require("core-js/modules/es.array.from.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/es.symbol.iterator.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.object.set-prototype-of.js");

require("core-js/modules/es.object.keys.js");

require("core-js/modules/es.array.join.js");

require("core-js/modules/es.regexp.to-string.js");

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var assert = require('assert');

var Stream = require('readable-stream');

var _require = require('./internal/http'),
    kOutHeaders = _require.kOutHeaders,
    utcDate = _require.utcDate,
    kNeedDrain = _require.kNeedDrain;

var _require2 = require('buffer'),
    Buffer = _require2.Buffer;

var common = require('./_http_common');

var checkIsHttpToken = common._checkIsHttpToken;
var checkInvalidHeaderChar = common._checkInvalidHeaderChar;

var _require3 = require('./internal/errors'),
    _require3$codes = _require3.codes,
    ERR_HTTP_HEADERS_SENT = _require3$codes.ERR_HTTP_HEADERS_SENT,
    ERR_HTTP_INVALID_HEADER_VALUE = _require3$codes.ERR_HTTP_INVALID_HEADER_VALUE,
    ERR_HTTP_TRAILER_INVALID = _require3$codes.ERR_HTTP_TRAILER_INVALID,
    ERR_INVALID_HTTP_TOKEN = _require3$codes.ERR_INVALID_HTTP_TOKEN,
    ERR_INVALID_ARG_TYPE = _require3$codes.ERR_INVALID_ARG_TYPE,
    ERR_INVALID_CHAR = _require3$codes.ERR_INVALID_CHAR,
    ERR_METHOD_NOT_IMPLEMENTED = _require3$codes.ERR_METHOD_NOT_IMPLEMENTED,
    ERR_STREAM_CANNOT_PIPE = _require3$codes.ERR_STREAM_CANNOT_PIPE,
    ERR_STREAM_ALREADY_FINISHED = _require3$codes.ERR_STREAM_ALREADY_FINISHED,
    ERR_STREAM_WRITE_AFTER_END = _require3$codes.ERR_STREAM_WRITE_AFTER_END,
    ERR_STREAM_NULL_VALUES = _require3$codes.ERR_STREAM_NULL_VALUES,
    ERR_STREAM_DESTROYED = _require3$codes.ERR_STREAM_DESTROYED;

var _require4 = require('./internal/util/types'),
    isUint8Array = _require4.isUint8Array;

var HIGH_WATER_MARK = 16 * 1024;
var CRLF = common.CRLF,
    debug = common.debug;
var kCorked = Symbol('corked');

function nop() {}

var RE_CONN_CLOSE = /(?:^|\W)close(?:$|\W)/i;
var RE_TE_CHUNKED = common.chunkExpression; // isCookieField performs a case-insensitive comparison of a provided string
// against the word "cookie." As of V8 6.6 this is faster than handrolling or
// using a case-insensitive RegExp.

function isCookieField(s) {
  return s.length === 6 && s.toLowerCase() === 'cookie';
}

function noopPendingOutput(amount) {}

function OutgoingMessage() {
  Stream.call(this); // Queue that holds all currently pending data, until the response will be
  // assigned to the socket (until it will its turn in the HTTP pipeline).

  this.outputData = []; // `outputSize` is an approximate measure of how much data is queued on this
  // response. `_onPendingData` will be invoked to update similar global
  // per-connection counter. That counter will be used to pause/unpause the
  // TCP socket and HTTP Parser and thus handle the backpressure.

  this.outputSize = 0;
  this.writable = true;
  this.destroyed = false;
  this._last = false;
  this.chunkedEncoding = false;
  this.shouldKeepAlive = true;
  this.useChunkedEncodingByDefault = true;
  this.sendDate = false;
  this._removedConnection = false;
  this._removedContLen = false;
  this._removedTE = false;
  this._contentLength = null;
  this._hasBody = true;
  this._trailer = '';
  this[kNeedDrain] = false;
  this.finished = false;
  this._headerSent = false;
  this[kCorked] = 0;
  this._closed = false;
  this.socket = null;
  this._header = null;
  this[kOutHeaders] = null;
  this._keepAliveTimeout = 0;
  this._onPendingData = noopPendingOutput;
}

Object.setPrototypeOf(OutgoingMessage.prototype, Stream.prototype);
Object.setPrototypeOf(OutgoingMessage, Stream);
Object.defineProperty(OutgoingMessage.prototype, 'writableFinished', {
  get: function get() {
    return this.finished && this.outputSize === 0 && (!this.socket || this.socket.writableLength === 0);
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'writableObjectMode', {
  get: function get() {
    return false;
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'writableLength', {
  get: function get() {
    return this.outputSize + (this.socket ? this.socket.writableLength : 0);
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'writableHighWaterMark', {
  get: function get() {
    return this.socket ? this.socket.writableHighWaterMark : HIGH_WATER_MARK;
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'writableCorked', {
  get: function get() {
    var corked = this.socket ? this.socket.writableCorked : 0;
    return corked + this[kCorked];
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'connection', {
  get: function get() {
    return this.socket;
  },
  set: function set(val) {
    this.socket = val;
  }
});

OutgoingMessage.prototype._renderHeaders = function _renderHeaders() {
  if (this._header) {
    throw new ERR_HTTP_HEADERS_SENT('render');
  }

  var headersMap = this[kOutHeaders];
  var headers = {};

  if (headersMap !== null) {
    var keys = Object.keys(headersMap); // Retain for(;;) loop for performance reasons
    // Refs: https://github.com/nodejs/node/pull/30958

    for (var i = 0, l = keys.length; i < l; i++) {
      var key = keys[i];
      headers[headersMap[key][0]] = headersMap[key][1];
    }
  }

  return headers;
};

OutgoingMessage.prototype.cork = function () {
  if (this.socket) {
    this.socket.cork();
  } else {
    this[kCorked]++;
  }
};

OutgoingMessage.prototype.uncork = function () {
  if (this.socket) {
    this.socket.uncork();
  } else if (this[kCorked]) {
    this[kCorked]--;
  }
};

OutgoingMessage.prototype.setTimeout = function setTimeout(msecs, callback) {
  if (callback) {
    this.on('timeout', callback);
  }

  if (!this.socket) {
    this.once('socket', function socketSetTimeoutOnConnect(socket) {
      socket.setTimeout(msecs);
    });
  } else {
    this.socket.setTimeout(msecs);
  }

  return this;
}; // It's possible that the socket will be destroyed, and removed from
// any messages, before ever calling this.  In that case, just skip
// it, since something else is destroying this connection anyway.


OutgoingMessage.prototype.destroy = function destroy(error) {
  if (this.destroyed) {
    return this;
  }

  this.destroyed = true;

  if (this.socket) {
    this.socket.destroy(error);
  } else {
    this.once('socket', function socketDestroyOnConnect(socket) {
      socket.destroy(error);
    });
  }

  return this;
}; // This abstract either writing directly to the socket or buffering it.


OutgoingMessage.prototype._send = function _send(data, encoding, callback) {
  if (!this._headerSent) {
    if (typeof data === 'string' && (encoding === 'utf8' || encoding === 'latin1' || !encoding)) {
      data = this._header + data;
    } else {
      var header = this._header;
      this.outputData.unshift({
        data: header,
        encoding: 'latin1',
        callback: null
      });
      this.outputSize += header.length;

      this._onPendingData(header.length);
    }

    this._headerSent = true;
  }

  return this._writeRaw(data, encoding, callback);
};

OutgoingMessage.prototype._writeRaw = _writeRaw;

function _writeRaw(data, encoding, callback) {
  var conn = this.socket;

  if (conn && conn.destroyed) {
    // The socket was destroyed. If we're still trying to write to it,
    // then we haven't gotten the 'close' event yet.
    return false;
  }

  if (typeof encoding === 'function') {
    callback = encoding;
    encoding = null;
  }

  if (conn && conn._httpMessage === this) {
    // There might be pending data in the this.output buffer.
    if (this.outputData.length) {
      this._flushOutput(conn);
    } // Directly write to socket.


    return conn.write(data, encoding, callback);
  } // Buffer, as long as we're not destroyed.


  this.outputData.push({
    data: data,
    encoding: encoding,
    callback: callback
  });
  this.outputSize += data.length;

  this._onPendingData(data.length);

  return this.outputSize < HIGH_WATER_MARK;
}

OutgoingMessage.prototype._storeHeader = _storeHeader;

function _storeHeader(firstLine, headers) {
  // firstLine in the case of request is: 'GET /index.html HTTP/1.1\r\n'
  // in the case of response it is: 'HTTP/1.1 200 OK\r\n'
  var state = {
    connection: false,
    contLen: false,
    te: false,
    date: false,
    expect: false,
    trailer: false,
    header: firstLine
  };

  if (headers) {
    if (headers === this[kOutHeaders]) {
      for (var key in headers) {
        var entry = headers[key];
        processHeader(this, state, entry[0], entry[1], false);
      }
    } else if (Array.isArray(headers)) {
      var _iterator = _createForOfIteratorHelper(headers),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _entry = _step.value;
          processHeader(this, state, _entry[0], _entry[1], true);
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    } else {
      for (var _key in headers) {
        if (Object.prototype.hasOwnProperty(headers, _key)) {
          processHeader(this, state, _key, headers[_key], true);
        }
      }
    }
  }

  var header = state.header; // Date header

  if (this.sendDate && !state.date) {
    header += 'Date: ' + utcDate() + CRLF;
  } // Force the connection to close when the response is a 204 No Content or
  // a 304 Not Modified and the user has set a "Transfer-Encoding: chunked"
  // header.
  //
  // RFC 2616 mandates that 204 and 304 responses MUST NOT have a body but
  // node.js used to send out a zero chunk anyway to accommodate clients
  // that don't have special handling for those responses.
  //
  // It was pointed out that this might confuse reverse proxies to the point
  // of creating security liabilities, so suppress the zero chunk and force
  // the connection to close.


  if (this.chunkedEncoding && (this.statusCode === 204 || this.statusCode === 304)) {
    // debug(this.statusCode + ' response should not use chunked encoding,' +
    // ' closing connection.');
    this.chunkedEncoding = false;
    this.shouldKeepAlive = false;
  } // keep-alive logic


  if (this._removedConnection) {
    this._last = true;
    this.shouldKeepAlive = false;
  } else if (!state.connection) {
    var shouldSendKeepAlive = this.shouldKeepAlive && (state.contLen || this.useChunkedEncodingByDefault || this.agent);

    if (shouldSendKeepAlive) {
      header += 'Connection: keep-alive\r\n';

      if (this._keepAliveTimeout) {
        var timeoutSeconds = Math.floor(this._keepAliveTimeout) / 1000;
        header += "Keep-Alive: timeout=".concat(timeoutSeconds, "\r\n");
      }
    } else {
      this._last = true;
      header += 'Connection: close\r\n';
    }
  }

  if (!state.contLen && !state.te) {
    if (!this._hasBody) {
      // Make sure we don't end the 0\r\n\r\n at the end of the message.
      this.chunkedEncoding = false;
    } else if (!this.useChunkedEncodingByDefault) {
      this._last = true;
    } else if (!state.trailer && !this._removedContLen && typeof this._contentLength === 'number') {
      header += 'Content-Length: ' + this._contentLength + CRLF;
    } else if (!this._removedTE) {
      header += 'Transfer-Encoding: chunked\r\n';
      this.chunkedEncoding = true;
    } else {// We should only be able to get here if both Content-Length and
      // Transfer-Encoding are removed by the user.
      // See: test/parallel/test-http-remove-header-stays-removed.js
      // debug('Both Content-Length and Transfer-Encoding are removed');
    }
  } // Test non-chunked message does not have trailer header set,
  // message will be terminated by the first empty line after the
  // header fields, regardless of the header fields present in the
  // message, and thus cannot contain a message body or 'trailers'.


  if (this.chunkedEncoding !== true && state.trailer) {
    throw new ERR_HTTP_TRAILER_INVALID();
  }

  this._header = header + CRLF;
  this._headerSent = false; // Wait until the first body chunk, or close(), is sent to flush,
  // UNLESS we're sending Expect: 100-continue.

  if (state.expect) this._send('');
}

function processHeader(self, state, key, value, validate) {
  if (validate) validateHeaderName(key);

  if (Array.isArray(value)) {
    if (value.length < 2 || !isCookieField(key)) {
      // Retain for(;;) loop for performance reasons
      // Refs: https://github.com/nodejs/node/pull/30958
      for (var i = 0; i < value.length; i++) {
        storeHeader(self, state, key, value[i], validate);
      }

      return;
    }

    value = value.join('; ');
  }

  storeHeader(self, state, key, value, validate);
}

function storeHeader(self, state, key, value, validate) {
  if (validate) validateHeaderValue(key, value);
  state.header += key + ': ' + value + CRLF;
  matchHeader(self, state, key, value);
}

function matchHeader(self, state, field, value) {
  if (field.length < 4 || field.length > 17) return;
  field = field.toLowerCase();

  switch (field) {
    case 'connection':
      state.connection = true;
      self._removedConnection = false;
      if (RE_CONN_CLOSE.test(value)) self._last = true;else self.shouldKeepAlive = true;
      break;

    case 'transfer-encoding':
      state.te = true;
      self._removedTE = false;
      if (RE_TE_CHUNKED.test(value)) self.chunkedEncoding = true;
      break;

    case 'content-length':
      state.contLen = true;
      self._removedContLen = false;
      break;

    case 'date':
    case 'expect':
    case 'trailer':
      state[field] = true;
      break;
  }
} // const validateHeaderName = hideStackFrames((name) => {


var validateHeaderName = function validateHeaderName(name) {
  if (typeof name !== 'string' || !name || !checkIsHttpToken(name)) {
    throw new ERR_INVALID_HTTP_TOKEN('Header name', name);
  }
}; // const validateHeaderValue = hideStackFrames((name, value) => {


var validateHeaderValue = function validateHeaderValue(name, value) {
  if (value === undefined) {
    throw new ERR_HTTP_INVALID_HEADER_VALUE(value, name);
  }

  if (checkInvalidHeaderChar(value)) {
    // debug('Header "%s" contains invalid characters', name);
    throw new ERR_INVALID_CHAR('header content', name);
  }
};

OutgoingMessage.prototype.setHeader = function setHeader(name, value) {
  if (this._header) {
    throw new ERR_HTTP_HEADERS_SENT('set');
  }

  validateHeaderName(name);
  validateHeaderValue(name, value);
  var headers = this[kOutHeaders];
  if (headers === null) this[kOutHeaders] = headers = Object.create(null);
  headers[name.toLowerCase()] = [name, value];
};

OutgoingMessage.prototype.getHeader = function getHeader(name) {
  // validateString(name, 'name');
  var headers = this[kOutHeaders];
  if (headers === null) return;
  var entry = headers[name.toLowerCase()];
  return entry && entry[1];
}; // Returns an array of the names of the current outgoing headers.


OutgoingMessage.prototype.getHeaderNames = function getHeaderNames() {
  return this[kOutHeaders] !== null ? Object.keys(this[kOutHeaders]) : [];
}; // Returns a shallow copy of the current outgoing headers.


OutgoingMessage.prototype.getHeaders = function getHeaders() {
  var headers = this[kOutHeaders];
  var ret = Object.create(null);

  if (headers) {
    var keys = Object.keys(headers); // Retain for(;;) loop for performance reasons
    // Refs: https://github.com/nodejs/node/pull/30958

    for (var i = 0; i < keys.length; ++i) {
      var key = keys[i];
      var val = headers[key][1];
      ret[key] = val;
    }
  }

  return ret;
};

OutgoingMessage.prototype.hasHeader = function hasHeader(name) {
  // validateString(name, 'name');
  return this[kOutHeaders] !== null && !!this[kOutHeaders][name.toLowerCase()];
};

OutgoingMessage.prototype.removeHeader = function removeHeader(name) {
  // validateString(name, 'name');
  if (this._header) {
    throw new ERR_HTTP_HEADERS_SENT('remove');
  }

  var key = name.toLowerCase();

  switch (key) {
    case 'connection':
      this._removedConnection = true;
      break;

    case 'content-length':
      this._removedContLen = true;
      break;

    case 'transfer-encoding':
      this._removedTE = true;
      break;

    case 'date':
      this.sendDate = false;
      break;
  }

  if (this[kOutHeaders] !== null) {
    delete this[kOutHeaders][key];
  }
};

OutgoingMessage.prototype._implicitHeader = function _implicitHeader() {
  throw new ERR_METHOD_NOT_IMPLEMENTED('_implicitHeader()');
};

Object.defineProperty(OutgoingMessage.prototype, 'headersSent', {
  configurable: true,
  enumerable: true,
  get: function get() {
    return !!this._header;
  }
});
Object.defineProperty(OutgoingMessage.prototype, 'writableEnded', {
  get: function get() {
    return this.finished;
  }
});
var crlf_buf = Buffer.from('\r\n');

OutgoingMessage.prototype.write = function write(chunk, encoding, callback) {
  if (typeof encoding === 'function') {
    callback = encoding;
    encoding = null;
  }

  var ret = write_(this, chunk, encoding, callback, false);
  if (!ret) this[kNeedDrain] = true;
  return ret;
};

function onError(msg, err, callback) {
  var triggerAsyncId = msg.socket ? msg.socket[async_id_symbol] : undefined; // defaultTriggerAsyncIdScope(triggerAsyncId,
  //                            process.nextTick,
  //                            emitErrorNt,
  //                            msg,
  //                            err,
  //                            callback);
}

function emitErrorNt(msg, err, callback) {
  callback(err);

  if (typeof msg.emit === 'function' && !msg._closed) {
    msg.emit('error', err);
  }
}

function write_(msg, chunk, encoding, callback, fromEnd) {
  if (typeof callback !== 'function') callback = nop;
  var len;

  if (chunk === null) {
    throw new ERR_STREAM_NULL_VALUES();
  } else if (typeof chunk === 'string') {
    len = Buffer.byteLength(chunk, encoding);
  } else if (isUint8Array(chunk)) {
    len = chunk.length;
  } else {
    throw new ERR_INVALID_ARG_TYPE('chunk', ['string', 'Buffer', 'Uint8Array'], chunk);
  }

  var err;

  if (msg.finished) {
    err = new ERR_STREAM_WRITE_AFTER_END();
  } else if (msg.destroyed) {
    err = new ERR_STREAM_DESTROYED('write');
  }

  if (err) {
    if (!msg.destroyed) {
      onError(msg, err, callback);
    } else {
      process.nextTick(callback, err);
    }

    return false;
  }

  if (!msg._header) {
    if (fromEnd) {
      msg._contentLength = len;
    }

    msg._implicitHeader();
  }

  if (!msg._hasBody) {
    // debug('This type of response MUST NOT have a body. ' +
    // 'Ignoring write() calls.');
    process.nextTick(callback);
    return true;
  }

  if (!fromEnd && msg.socket && !msg.socket.writableCorked) {
    msg.socket.cork();
    process.nextTick(connectionCorkNT, msg.socket);
  }

  var ret;

  if (msg.chunkedEncoding && chunk.length !== 0) {
    msg._send(len.toString(16), 'latin1', null);

    msg._send(crlf_buf, null, null);

    msg._send(chunk, encoding, null);

    ret = msg._send(crlf_buf, null, callback);
  } else {
    ret = msg._send(chunk, encoding, callback);
  } // debug('write ret = ' + ret);


  return ret;
}

function connectionCorkNT(conn) {
  conn.uncork();
}

OutgoingMessage.prototype.addTrailers = function addTrailers(headers) {
  this._trailer = '';
  var keys = Object.keys(headers);
  var isArray = Array.isArray(headers); // Retain for(;;) loop for performance reasons
  // Refs: https://github.com/nodejs/node/pull/30958

  for (var i = 0, l = keys.length; i < l; i++) {
    var field = void 0,
        value = void 0;
    var key = keys[i];

    if (isArray) {
      field = headers[key][0];
      value = headers[key][1];
    } else {
      field = key;
      value = headers[key];
    }

    if (typeof field !== 'string' || !field || !checkIsHttpToken(field)) {
      throw new ERR_INVALID_HTTP_TOKEN('Trailer name', field);
    }

    if (checkInvalidHeaderChar(value)) {
      // debug('Trailer "%s" contains invalid characters', field);
      throw new ERR_INVALID_CHAR('trailer content', field);
    }

    this._trailer += field + ': ' + value + CRLF;
  }
};

function onFinish(outmsg) {
  if (outmsg && outmsg.socket && outmsg.socket._hadError) return;
  outmsg.emit('finish');
}

OutgoingMessage.prototype.end = function end(chunk, encoding, callback) {
  if (typeof chunk === 'function') {
    callback = chunk;
    chunk = null;
    encoding = null;
  } else if (typeof encoding === 'function') {
    callback = encoding;
    encoding = null;
  }

  if (this.socket) {
    this.socket.cork();
  }

  if (chunk) {
    write_(this, chunk, encoding, null, true);
  } else if (this.finished) {
    if (typeof callback === 'function') {
      if (!this.writableFinished) {
        this.on('finish', callback);
      } else {
        callback(new ERR_STREAM_ALREADY_FINISHED('end'));
      }
    }

    return this;
  } else if (!this._header) {
    this._contentLength = 0;

    this._implicitHeader();
  }

  if (typeof callback === 'function') this.once('finish', callback);
  var finish = onFinish.bind(undefined, this);

  if (this._hasBody && this.chunkedEncoding) {
    this._send('0\r\n' + this._trailer + '\r\n', 'latin1', finish);
  } else {
    // Force a flush, HACK.
    this._send('', 'latin1', finish);
  }

  if (this.socket) {
    // Fully uncork connection on end().
    // this.socket._writableState.corked = 1;
    this.socket.uncork();
  }

  this[kCorked] = 0;
  this.finished = true; // There is the first message on the outgoing queue, and we've sent
  // everything to the socket.
  // debug('outgoing message end.');

  if (this.outputData.length === 0 && this.socket && this.socket._httpMessage === this) {
    this._finish();
  }

  return this;
};

OutgoingMessage.prototype._finish = function _finish() {
  assert(this.socket);
  this.emit('prefinish');
}; // This logic is probably a bit confusing. Let me explain a bit:
//
// In both HTTP servers and clients it is possible to queue up several
// outgoing messages. This is easiest to imagine in the case of a client.
// Take the following situation:
//
//    req1 = client.request('GET', '/');
//    req2 = client.request('POST', '/');
//
// When the user does
//
//   req2.write('hello world\n');
//
// it's possible that the first request has not been completely flushed to
// the socket yet. Thus the outgoing messages need to be prepared to queue
// up data internally before sending it on further to the socket's queue.
//
// This function, outgoingFlush(), is called by both the Server and Client
// to attempt to flush any pending messages out to the socket.


OutgoingMessage.prototype._flush = function _flush() {
  var socket = this.socket;

  if (socket && socket.writable) {
    // There might be remaining data in this.output; write it out
    var ret = this._flushOutput(socket);

    if (this.finished) {
      // This is a queue to the server or client to bring in the next this.
      this._finish();
    } else if (ret && this[kNeedDrain]) {
      this[kNeedDrain] = false;
      this.emit('drain');
    }
  }
};

OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
  while (this[kCorked]) {
    this[kCorked]--;
    socket.cork();
  }

  var outputLength = this.outputData.length;
  if (outputLength <= 0) return undefined;
  var outputData = this.outputData;
  socket.cork();
  var ret; // Retain for(;;) loop for performance reasons
  // Refs: https://github.com/nodejs/node/pull/30958

  for (var i = 0; i < outputLength; i++) {
    var _outputData$i = outputData[i],
        data = _outputData$i.data,
        encoding = _outputData$i.encoding,
        callback = _outputData$i.callback;
    ret = socket.write(data, encoding, callback);
  }

  socket.uncork();
  this.outputData = [];

  this._onPendingData(-this.outputSize);

  this.outputSize = 0;
  return ret;
};

OutgoingMessage.prototype.flushHeaders = function flushHeaders() {
  if (!this._header) {
    this._implicitHeader();
  } // Force-flush the headers.


  this._send('');
};

OutgoingMessage.prototype.pipe = function pipe() {
  // OutgoingMessage should be write-only. Piping from it is disabled.
  this.emit('error', new ERR_STREAM_CANNOT_PIPE());
}; // OutgoingMessage.prototype[EE.captureRejectionSymbol] =
// function(err, event) {
//   this.destroy(err);
// };


module.exports = {
  validateHeaderName: validateHeaderName,
  validateHeaderValue: validateHeaderValue,
  OutgoingMessage: OutgoingMessage
};