"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.number.constructor.js");

require("core-js/modules/es.array-buffer.is-view.js");

require("core-js/modules/es.array-buffer.constructor.js");

require("core-js/modules/es.array-buffer.slice.js");

require("core-js/modules/es.array.concat.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.array.slice.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var _require = require('readable-stream'),
    Readable = _require.Readable,
    Writable = _require.Writable,
    Transform = _require.Transform,
    Duplex = _require.Duplex,
    pipeline = _require.pipeline,
    finished = _require.finished;

var BUFFER = Symbol('buffer');
var TYPE = Symbol('type');
/**
 * Expose `Blob`.
 */

module.exports = Blob;
/**
 * Initialize a new `Blob`.
 *
 * @api public
 */

function Blob() {
  this[TYPE] = '';
  var blobParts = arguments[0];
  var options = arguments[1];
  var buffers = [];
  var size = 0;

  if (blobParts) {
    var a = blobParts;
    var length = Number(a.length);

    for (var i = 0; i < length; i++) {
      var element = a[i];
      var buffer = void 0;

      if (element instanceof Buffer) {
        buffer = element;
      } else if (ArrayBuffer.isView(element)) {
        buffer = Buffer.from(element.buffer, element.byteOffset, element.byteLength);
      } else if (element instanceof ArrayBuffer) {
        buffer = Buffer.from(element);
      } else if (element instanceof Blob) {
        buffer = element[BUFFER];
      } else {
        buffer = Buffer.from(typeof element === 'string' ? element : String(element));
      }

      size += buffer.length;
      buffers.push(buffer);
    }
  }

  this[BUFFER] = Buffer.concat(buffers);
  var type = options && options.type !== undefined && String(options.type).toLowerCase();

  if (type && !/[^\u0020-\u007E]/.test(type)) {
    this[TYPE] = type;
  }

  var ctx = mixin(this);
  return ctx;
}
/**
 * Mixin the prototype properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */


function mixin(obj) {
  // for (const key in HttpRequest.prototype) {
  //   if (Object.prototype.hasOwnProperty.call(HttpRequest.prototype, key))
  // 	obj[key] = HttpRequest.prototype[key];
  // }
  return obj;
}

Blob.prototype.BUFFER = BUFFER;

Blob.prototype.size = function () {
  return this[BUFFER].length;
};

Blob.prototype.type = function () {
  return this[TYPE];
};

Blob.prototype.text = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          return _context.abrupt("return", Promise.resolve(this[BUFFER].toString()));

        case 1:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this);
}));
Blob.prototype.arrayBuffer = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
  var buf, ab;
  return regeneratorRuntime.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          buf = this[BUFFER];
          ab = buf.buffer.slice(buf.byteOffset, buf.byteOffset + buf.byteLength);
          return _context2.abrupt("return", Promise.resolve(ab));

        case 3:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2, this);
}));

Blob.prototype.stream = function () {
  // const readable = new ReadableStream();
  var readable = new Duplex();

  readable._read = function () {};

  readable.push(this[BUFFER]);
  readable.push(null);
  return readable;
};

Blob.prototype.slice = function () {
  var size = this.size;
  var start = arguments[0];
  var end = arguments[1];
  var relativeStart, relativeEnd;

  if (start === undefined) {
    relativeStart = 0;
  } else if (start < 0) {
    relativeStart = Math.max(size + start, 0);
  } else {
    relativeStart = Math.min(start, size);
  }

  if (end === undefined) {
    relativeEnd = size;
  } else if (end < 0) {
    relativeEnd = Math.max(size + end, 0);
  } else {
    relativeEnd = Math.min(end, size);
  }

  var span = Math.max(relativeEnd - relativeStart, 0);
  var buffer = this[BUFFER];
  var slicedBuffer = buffer.slice(relativeStart, relativeStart + span);
  var blob = new Blob([], {
    type: arguments[2]
  });
  blob[BUFFER] = slicedBuffer;
  return blob;
};