"use strict";

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.symbol.iterator.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/es.promise.js");

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.array.concat.js");

require("core-js/modules/es.array.join.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.replace.js");

require("core-js/modules/es.function.name.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/web.dom-collections.for-each.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var util = require('util');

var path = require('path');

var http = require('http');

var https = require('https');

var parseUrl = require('url').parse;

var fs = require('fs'); // var Stream = require('stream').Stream;


var _require = require('stream-browserify'),
    Readable = _require.Readable,
    Writable = _require.Writable,
    Transform = _require.Transform,
    Duplex = _require.Duplex,
    pipeline = _require.pipeline,
    finished = _require.finished;

var mime = require('mime-types');

var asynckit = require('asynckit');

var populate = require('./populate.js');

var MultiStream = require('multistream');

var ZitiFileReaderStream = require('./file-reader-stream');

var isUndefined = require('lodash.isundefined');
/**
 * Expose `ZitiFormData`.
 */


module.exports = ZitiFormData;
/**
 * Create readable "multipart/form-data" streams.
 * Can be used to submit forms
 * and file uploads to other web applications.
 *
 * @constructor
 * @param {Object} options - Properties to be added/overriden for ZitiFormData and CombinedStream
 */

function ZitiFormData(options) {
  if (!(this instanceof ZitiFormData)) {
    return new ZitiFormData(options);
  }

  this._overheadLength = 0;
  this._valueLength = 0;
  this._valuesToMeasure = [];
  this._streams = [];
  options = options || {
    pauseStreams: false
  };

  for (var option in options) {
    this[option] = options[option];
  }
}

ZitiFormData.LINE_BREAK = '\r\n';
ZitiFormData.DEFAULT_CONTENT_TYPE = 'application/octet-stream';

ZitiFormData.prototype.append = function (field, value, options) {
  options = options || {}; // allow filename as single option

  if (typeof options == 'string') {
    options = {
      filename: options
    };
  } // all that streamy business can't handle numbers


  if (typeof value == 'number') {
    value = '' + value;
  }

  if (Array.isArray(value)) {
    // Please convert your array into string the way web server expects it
    this._error(new Error('Arrays are not supported.'));

    return;
  }

  var header = this._multiPartHeader(field, value, options); // var footer = this._multiPartFooter();


  var footer = undefined;

  if (options.lastkey) {
    footer = ZitiFormData.LINE_BREAK + this._lastBoundary();
  }

  var hdrStream = new Readable();
  hdrStream.push(header);
  hdrStream.push(null);

  this._streams.push(hdrStream);

  if (value instanceof File) {
    options.knownLength = value.size;

    this._streams.push(ZitiFileReaderStream(value, {
      chunkSize: 1024 * 8
    }));

    var lbStream = new Readable();
    lbStream.push(ZitiFormData.LINE_BREAK);
    lbStream.push(null);

    this._streams.push(lbStream);
  } else {
    options.knownLength = value.length;
    var vStream = new Readable();
    vStream.push(value);
    vStream.push(ZitiFormData.LINE_BREAK);
    vStream.push(null);

    this._streams.push(vStream);
  }

  if (!isUndefined(footer)) {
    var ftrStream = new Readable();
    ftrStream.push(footer);
    ftrStream.push(null);

    this._streams.push(ftrStream);
  } // pass along options.knownLength


  this._trackLength(header, value, options);
};

ZitiFormData.prototype._trackLength = function (header, value, options) {
  var valueLength = 0; // used w/ getLengthSync(), when length is known.
  // e.g. for streaming directly from a remote server,
  // w/ a known file a size, and not wanting to wait for
  // incoming file to finish to get its size.

  if (options.knownLength !== null) {
    valueLength += +options.knownLength;
  } else if (Buffer.isBuffer(value)) {
    valueLength = value.length;
  } else if (typeof value === 'string') {
    valueLength = Buffer.byteLength(value);
  }

  this._valueLength += valueLength; // @check why add CRLF? does this account for custom/multiple CRLFs?

  this._overheadLength += Buffer.byteLength(header) + ZitiFormData.LINE_BREAK.length; // empty or either doesn't have path or not an http response or not a stream
  // if (!value || ( !value.path && !(value.readable && value.hasOwnProperty('httpVersion')) && !(value instanceof Stream))) {
  // return;
  // }
  // no need to bother with the length

  if (!options.knownLength) {
    this._valuesToMeasure.push(value);
  }
};

ZitiFormData.prototype._lengthRetriever = function (value, callback) {
  if (value.hasOwnProperty('fd')) {
    // take read range into a account
    // `end` = Infinity –> read file till the end
    //
    // TODO: Looks like there is bug in Node fs.createReadStream
    // it doesn't respect `end` options without `start` options
    // Fix it when node fixes it.
    // https://github.com/joyent/node/issues/7819
    if (value.end != undefined && value.end != Infinity && value.start != undefined) {
      // when end specified
      // no need to calculate range
      // inclusive, starts with 0
      callback(null, value.end + 1 - (value.start ? value.start : 0)); // not that fast snoopy
    } else {
      // still need to fetch file size from fs
      fs.stat(value.path, function (err, stat) {
        var fileSize;

        if (err) {
          callback(err);
          return;
        } // update final size based on the range options


        fileSize = stat.size - (value.start ? value.start : 0);
        callback(null, fileSize);
      });
    } // or http response

  } else if (value.hasOwnProperty('httpVersion')) {
    callback(null, +value.headers['content-length']); // or request stream http://github.com/mikeal/request
  } else if (value.hasOwnProperty('httpModule')) {
    // wait till response come back
    value.on('response', function (response) {
      value.pause();
      callback(null, +response.headers['content-length']);
    });
    value.resume(); // something else
  } else {
    callback('Unknown stream');
  }
};

ZitiFormData.prototype._multiPartHeader = function (field, value, options) {
  // custom header specified (as string)?
  // it becomes responsible for boundary
  // (e.g. to handle extra CRLFs on .NET servers)
  if (typeof options.header == 'string') {
    return options.header;
  }

  var contentDisposition = this._getContentDisposition(value, options);

  var contentType = this._getContentType(value, options);

  var contents = '';
  var headers = {
    // add custom disposition as third element or keep it two elements if not
    'Content-Disposition': ['form-data', 'name="' + field + '"'].concat(contentDisposition || []),
    // if no content type. allow it to be empty array
    'Content-Type': [].concat(contentType || [])
  }; // allow custom headers.

  if (_typeof(options.header) == 'object') {
    populate(headers, options.header);
  }

  var header;

  for (var prop in headers) {
    if (!headers.hasOwnProperty(prop)) continue;
    header = headers[prop]; // skip nullish headers.

    if (header === null) {
      continue;
    } // convert all headers to arrays.


    if (!Array.isArray(header)) {
      header = [header];
    } // add non-empty headers.


    if (header.length) {
      contents += prop + ': ' + header.join('; ') + ZitiFormData.LINE_BREAK;
    }
  }

  return '--' + this.getBoundary() + ZitiFormData.LINE_BREAK + contents + ZitiFormData.LINE_BREAK;
};

ZitiFormData.prototype._getContentDisposition = function (value, options) {
  var filename, contentDisposition;

  if (typeof options.filepath === 'string') {
    // custom filepath for relative paths
    filename = path.normalize(options.filepath).replace(/\\/g, '/');
  } else if (options.filename || value.name || value.path) {
    // custom filename take precedence
    // formidable and the browser add a name property
    // fs- and request- streams have path property
    filename = path.basename(options.filename || value.name || value.path);
  } else if (value.readable && value.hasOwnProperty('httpVersion')) {
    // or try http response
    filename = path.basename(value.client._httpMessage.path || '');
  }

  if (filename) {
    contentDisposition = 'filename="' + filename + '"';
  }

  return contentDisposition;
};

ZitiFormData.prototype._getContentType = function (value, options) {
  // use custom content-type above all
  var contentType = options.contentType; // or try `name` from formidable, browser

  if (!contentType && value.name) {
    contentType = mime.lookup(value.name);
  } // or try `path` from fs-, request- streams


  if (!contentType && value.path) {
    contentType = mime.lookup(value.path);
  } // or if it's http-reponse


  if (!contentType && value.readable && value.hasOwnProperty('httpVersion')) {
    contentType = value.headers['content-type'];
  } // or guess it from the filepath or filename


  if (!contentType && (options.filepath || options.filename)) {
    contentType = mime.lookup(options.filepath || options.filename);
  } // fallback to the default content type if `value` is not simple value


  if (!contentType && _typeof(value) == 'object') {
    contentType = ZitiFormData.DEFAULT_CONTENT_TYPE;
  }

  return contentType;
};

ZitiFormData.prototype._multiPartFooter = function () {
  return function (next) {
    var footer = ZitiFormData.LINE_BREAK;
    var lastPart = this._streams.length === 0;

    if (lastPart) {
      footer += this._lastBoundary();
    }

    next(footer);
  }.bind(this);
};

ZitiFormData.prototype._lastBoundary = function () {
  return '--' + this.getBoundary() + '--' + ZitiFormData.LINE_BREAK;
};

ZitiFormData.prototype.getHeaders = function (userHeaders) {
  var header;
  var formHeaders = {
    'content-type': 'multipart/form-data; boundary=' + this.getBoundary()
  };

  for (header in userHeaders) {
    if (userHeaders.hasOwnProperty(header)) {
      formHeaders[header.toLowerCase()] = userHeaders[header];
    }
  }

  return formHeaders;
};

ZitiFormData.prototype.setBoundary = function (boundary) {
  this._boundary = boundary;
};

ZitiFormData.prototype.getBoundary = function () {
  if (!this._boundary) {
    this._generateBoundary();
  }

  return this._boundary;
};

ZitiFormData.prototype.getStream = function () {
  return new MultiStream(this._streams);
};

ZitiFormData.prototype.getBuffer = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
  var dataBuffer, boundary, i, len, zfr, fileBuffer;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          dataBuffer = new Buffer.alloc(0);
          boundary = this.getBoundary(); // Create the form content. Add Line breaks to the end of data.

          i = 0, len = this._streams.length;

        case 3:
          if (!(i < len)) {
            _context.next = 22;
            break;
          }

          if (!(typeof this._streams[i] !== 'function')) {
            _context.next = 19;
            break;
          }

          if (!Buffer.isBuffer(this._streams[i])) {
            _context.next = 9;
            break;
          }

          dataBuffer = Buffer.concat([dataBuffer, this._streams[i]]);
          _context.next = 18;
          break;

        case 9:
          if (!(this._streams[i] instanceof File)) {
            _context.next = 17;
            break;
          }

          zfr = new ZitiFileReader(this._streams[i]);
          _context.next = 13;
          return zfr.getBuffer();

        case 13:
          fileBuffer = _context.sent;
          dataBuffer = Buffer.concat([dataBuffer, fileBuffer]);
          _context.next = 18;
          break;

        case 17:
          dataBuffer = Buffer.concat([dataBuffer, Buffer.from(this._streams[i])]);

        case 18:
          // Add break after content.
          if (typeof this._streams[i] !== 'string' || this._streams[i].substring(2, boundary.length + 2) !== boundary) {
            dataBuffer = Buffer.concat([dataBuffer, Buffer.from(ZitiFormData.LINE_BREAK)]);
          }

        case 19:
          i++;
          _context.next = 3;
          break;

        case 22:
          return _context.abrupt("return", Buffer.concat([dataBuffer, Buffer.from(this._lastBoundary())]));

        case 23:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this);
}));

ZitiFormData.prototype._generateBoundary = function () {
  // This generates a 50 character boundary similar to those used by Firefox.
  // They are optimized for boyer-moore parsing.
  var boundary = '--------------------------';

  for (var i = 0; i < 24; i++) {
    boundary += Math.floor(Math.random() * 10).toString(16);
  }

  this._boundary = boundary;
}; // Note: getLengthSync DOESN'T calculate streams length
// As workaround one can calculate file size manually
// and add it as knownLength option


ZitiFormData.prototype.getLengthSync = function () {
  var knownLength = this._overheadLength + this._valueLength; // Don't get confused, there are 3 "internal" streams for each keyval pair
  // so it basically checks if there is any value added to the form

  if (this._streams.length) {
    knownLength += this._lastBoundary().length;
  } // https://github.com/form-data/form-data/issues/40


  if (!this.hasKnownLength()) {
    // Some async length retrievers are present
    // therefore synchronous length calculation is false.
    // Please use getLength(callback) to get proper length
    this._error(new Error('Cannot calculate proper length in synchronous way.'));
  }

  return knownLength;
}; // Public API to check if length of added values is known
// https://github.com/form-data/form-data/issues/196
// https://github.com/form-data/form-data/issues/262


ZitiFormData.prototype.hasKnownLength = function () {
  var hasKnownLength = true;

  if (this._valuesToMeasure.length) {
    hasKnownLength = false;
  }

  return hasKnownLength;
};

ZitiFormData.prototype.getLength = function (cb) {
  var knownLength = this._overheadLength + this._valueLength;

  if (this._streams.length) {
    knownLength += this._lastBoundary().length;
  }

  if (!this._valuesToMeasure.length) {
    process.nextTick(cb.bind(this, null, knownLength));
    return;
  }

  asynckit.parallel(this._valuesToMeasure, this._lengthRetriever, function (err, values) {
    if (err) {
      cb(err);
      return;
    }

    values.forEach(function (length) {
      knownLength += length;
    });
    cb(null, knownLength);
  });
};

ZitiFormData.prototype.submit = function (params, cb) {
  var request,
      options,
      defaults = {
    method: 'post'
  }; // parse provided url if it's string
  // or treat it as options object

  if (typeof params == 'string') {
    params = parseUrl(params);
    options = populate({
      port: params.port,
      path: params.pathname,
      host: params.hostname,
      protocol: params.protocol
    }, defaults); // use custom params
  } else {
    options = populate(params, defaults); // if no port provided use default one

    if (!options.port) {
      options.port = options.protocol == 'https:' ? 443 : 80;
    }
  } // put that good code in getHeaders to some use


  options.headers = this.getHeaders(params.headers); // https if specified, fallback to http in any other case

  if (options.protocol == 'https:') {
    request = https.request(options);
  } else {
    request = http.request(options);
  } // get content length and fire away


  this.getLength(function (err, length) {
    if (err && err !== 'Unknown stream') {
      this._error(err);

      return;
    } // add content length


    if (length) {
      request.setHeader('Content-Length', length);
    }

    this.pipe(request);

    if (cb) {
      var onResponse;

      var callback = function callback(error, responce) {
        request.removeListener('error', callback);
        request.removeListener('response', onResponse);
        return cb.call(this, error, responce);
      };

      onResponse = callback.bind(this, null);
      request.on('error', callback);
      request.on('response', onResponse);
    }
  }.bind(this));
  return request;
};

ZitiFormData.prototype._error = function (err) {
  if (!this.error) {
    this.error = err;
    this.pause();
    this.emit('error', err);
  }
};

ZitiFormData.prototype.toString = function () {
  return '[object ZitiFormData]';
};