"use strict";

require("core-js/modules/es.symbol.iterator.js");

require("core-js/modules/es.promise.js");

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/web.url.js");

require("core-js/modules/es.regexp.constructor.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.regexp.to-string.js");

require("core-js/modules/es.string.replace.js");

require("core-js/modules/es.string.search.js");

require("core-js/modules/es.array.includes.js");

require("core-js/modules/es.string.includes.js");

require("core-js/modules/es.object.assign.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * request.js
 *
 * HttpRequest class
 *
 * All spec algorithm step numbers are based on https://fetch.spec.whatwg.org/commit-snapshots/ae716822cb3a61843226cd090eefc6589446c1d2/.
 */
var Cookies = require('js-cookie');

var HttpHeaders = require('./headers.js');

var HttpBody = require('./body');

var ZitiFormData = require('./form-data');

var ls = require('../utils/localstorage');

var zitiConstants = require('../constants');

var isNull = require('lodash.isnull');

var clone = HttpBody.clone;

var pjson = require('../../package.json');

var isUndefined = require('lodash.isundefined');

var _split = require('lodash.split');

var forEach = require('lodash.foreach');

var INTERNALS = Symbol('HttpRequest internals');
/**
 * Check if a value is an instance of HttpRequest.
 *
 * @param   Mixed   input
 * @return  Boolean
 */

function isRequest(input) {
  return _typeof(input) === 'object' && _typeof(input[INTERNALS]) === 'object';
}
/**
 * Parse a URL.
 *
 * @param   Mixed   url
 * @return  Object
 */


function parseURL(url) {
  var parsedUrl = new URL(url);
  return parsedUrl;
}
/**
 * Expose `HttpRequest`.
 */


module.exports = HttpRequest;
/**
 * Initialize a new `HttpRequest`.
 *
 * @api public
 */

function HttpRequest(serviceNameOrConn, input) {
  var init = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var serviceName;
  var conn;

  if (_typeof(serviceNameOrConn) == 'object') {
    conn = serviceNameOrConn;
  } else if (typeof serviceNameOrConn == 'string') {
    serviceName = serviceNameOrConn;
  } else {
    throw new Error('first paramater is unsupported type');
  }

  if (!isRequest(input)) {
    if (input && input.href) {
      // in order to support Node.js' Url objects; though WHATWG's URL objects
      // will fall into this branch also (since their `toString()` will return
      // `href` property anyway)
      parsedURL = parseURL(input.href);
    } else {
      // coerce input to a string before attempting to parse
      parsedURL = parseURL("".concat(input));
    }

    input = {};
  } else {
    parsedURL = parseURL(input.url);
  }

  var method = init.method || input.method || 'GET';
  method = method.toUpperCase();

  if (isUndefined(init.body)) {
    init.body = null;
  }

  if ((init.body !== null || isRequest(input) && input.body !== null) && (method === 'GET' || method === 'HEAD')) {
    throw new Error('HttpRequest with GET/HEAD method cannot have body');
  }

  var inputBody = init.body !== null ? init.body : isRequest(input) && input.body !== null ? clone(input) : null;
  HttpBody.call(this, inputBody, {
    timeout: init.timeout || input.timeout || 0,
    size: init.size || input.size || 0
  });
  var headers = new HttpHeaders(init.headers || input.headers || {});

  if (this.body instanceof ZitiFormData) {
    inputBody = this.body;
  }

  if (inputBody !== null && !headers.has('Content-Type')) {
    var contentType = this.extractContentType(inputBody);

    if (contentType) {
      headers.append('Content-Type', contentType);
    }
  }

  this[INTERNALS] = {
    serviceName: serviceName,
    conn: conn,
    method: method,
    redirect: init.redirect || input.redirect || 'follow',
    headers: headers,
    parsedURL: parsedURL
  };
  var ctx = mixin(this);
  return ctx;
}
/**
 * Mixin the prototype properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */


function mixin(obj) {
  for (var key in HttpRequest.prototype) {
    if (Object.prototype.hasOwnProperty.call(HttpRequest.prototype, key)) obj[key] = HttpRequest.prototype[key];
  }

  return obj;
}

HttpRequest.prototype.getServiceName = function () {
  return this[INTERNALS].serviceName;
};

HttpRequest.prototype.getConn = function () {
  return this[INTERNALS].conn;
};

HttpRequest.prototype.getMethod = function () {
  return this[INTERNALS].method;
};

HttpRequest.prototype.getHeaders = function () {
  return this[INTERNALS].headers;
};

HttpRequest.prototype.getRedirect = function () {
  return this[INTERNALS].redirect;
};

HttpRequest.prototype.getParsedURL = function () {
  return this[INTERNALS].parsedURL;
};

HttpRequest.prototype.getRequestOptions = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
  var parsedURL, headers, replace, re, cookieObject, cookieHdr, cookies, browserCookies, cookie, release, zitiCookies, _cookie, cookieHeaderValue, _cookie2, contentLengthValue, totalBytes, obj, key;

  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          parsedURL = this[INTERNALS].parsedURL;
          headers = this[INTERNALS].headers; // Transform all occurances of the HTTP Agent hostname back to the target hostname

          replace = zitiConfig.httpAgent.self.host;
          re = new RegExp(replace, "gi");
          parsedURL.href = parsedURL.href.replace(re, zitiConfig.httpAgent.target.host);
          parsedURL.search = parsedURL.search.replace(re, zitiConfig.httpAgent.target.host); // fetch step 1.3

          if (!headers.has('Accept')) {
            headers.set('Accept', '*/*');
          } // Basic fetch


          if (parsedURL.hostname) {
            _context.next = 9;
            break;
          }

          throw new TypeError('Only absolute URLs are supported');

        case 9:
          if (!parsedURL.protocol) {
            parsedURL.protocol = 'https:';
          }

          if (/^https?:$/.test(parsedURL.protocol)) {
            _context.next = 12;
            break;
          }

          throw new Error('Only HTTP(S) protocols are supported');

        case 12:
          if (parsedURL.port !== '' && parsedURL.port !== '80') {
            headers.set('Host', parsedURL.hostname + ":" + parsedURL.port);
          } else {
            headers.set('Host', parsedURL.hostname);
          }

          cookieObject = {}; // Obtain all Cookie KV pairs from the incoming Cookie header

          if (headers.has('Cookie')) {
            cookieHdr = headers.get('Cookie');
            cookies = _split(cookieHdr, ';');
            forEach(cookies, function (cookie) {
              var cookieKV = _split(cookie, '=');

              cookieObject[cookieKV[0]] = cookieKV[1];
            });
          } // Obtain all Cookie KV pairs from the browser Cookie cache


          browserCookies = Cookies.get();

          for (cookie in browserCookies) {
            if (browserCookies.hasOwnProperty(cookie)) {
              cookieObject[cookie] = browserCookies[cookie];

              if (cookie.includes('CSRF')) {
                headers.set('X-CSRF-Token', browserCookies[cookie]);
              }
            }
          } // Obtain all Cookie KV pairs from the Ziti Cookie cache


          _context.next = 19;
          return ziti._cookiemutex.acquire();

        case 19:
          release = _context.sent;
          _context.next = 22;
          return ls.getWithExpiry(zitiConstants.get().ZITI_COOKIES);

        case 22:
          zitiCookies = _context.sent;
          release();

          if (!isNull(zitiCookies)) {
            for (_cookie in zitiCookies) {
              if (zitiCookies.hasOwnProperty(_cookie)) {
                if (zitiCookies[_cookie] !== 'null') {
                  cookieObject[_cookie] = zitiCookies[_cookie];
                }
              }
            }
          } // set the Cookie header


          cookieHeaderValue = '';

          for (_cookie2 in cookieObject) {
            if (_cookie2 !== '') {
              if (cookieObject.hasOwnProperty(_cookie2)) {
                cookieHeaderValue += _cookie2 + '=' + cookieObject[_cookie2] + '; ';
              }
            }
          }

          if (cookieHeaderValue !== '') {
            headers.set('Cookie', cookieHeaderValue);
          } else {
            headers.delete('Cookie');
          } // HTTP-network-or-cache fetch steps 2.4-2.7


          contentLengthValue = null;

          if (this.body === null && /^(POST|PUT)$/i.test(this.getMethod())) {
            contentLengthValue = '0';
          }

          if (this.body !== null) {
            this.body.get;
            totalBytes = this.getTotalBytes(this.body);

            if (typeof totalBytes === 'number') {
              contentLengthValue = String(totalBytes);
            }
          }

          if (/^(POST|PUT)$/i.test(this.getMethod())) {
            if (typeof contentLengthValue == 'string') {
              headers.set('Content-Length', contentLengthValue); // headers.set('Transfer-Encoding', 'chunked');
            } else {
              // it must be a stream, so we go with chunked encoding instead of content length
              headers.set('Transfer-Encoding', 'chunked');
            }
          } // HTTP-network-or-cache fetch step 2.11


          if (!headers.has('User-Agent')) {
            headers.set('User-Agent', 'ziti-sdk-js/' + pjson.version);
          } // --- Disable gzip for now ---
          //
          // // HTTP-network-or-cache fetch step 2.15


          if (this.compress && !headers.has('Accept-Encoding')) {
            headers.set('Accept-Encoding', 'gzip,deflate');
          } // if (!headers.has('Connection')) {
          // 	headers.set('Connection', 'keep-alive');
          // }


          obj = Object.assign({}, {
            serviceName: this.getServiceName(),
            conn: this.getConn(),
            method: this.getMethod(),
            headers: headers,
            body: this.body
          });

          for (key in parsedURL) {
            obj[key] = parsedURL[key];
          }

          obj.path = obj.pathname;
          return _context.abrupt("return", obj);

        case 38:
        case "end":
          return _context.stop();
      }
    }
  }, _callee, this);
}));
HttpBody.mixIn(HttpRequest.prototype);
Object.defineProperties(HttpRequest.prototype, {
  method: {
    enumerable: true
  },
  url: {
    enumerable: true
  },
  headers: {
    enumerable: true
  },
  redirect: {
    enumerable: true
  },
  clone: {
    enumerable: true
  }
});