"use strict";

require("core-js/modules/es.number.constructor.js");

require("core-js/modules/es.symbol.js");

require("core-js/modules/es.symbol.description.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.symbol.iterator.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var url = require('url');

var ZitiSocket = require('./ziti-socket');
/**
 * Module exports.
 */


module.exports = ZitiAgent;
/**
 * Base HTTP "ZitiAgent" class. Emulates the node-core `http.Agent` class, but
 * implemented in a way that can be extended for additional functionality.
 *
 *
 * @api public
 */

function ZitiAgent(opts) {
  if (!(this instanceof ZitiAgent)) return new ZitiAgent(opts);
  if ('string' == typeof opts) opts = url.parse(opts); // Agent.call(this);

  this.proxy = opts;
  this.secure = this.proxy.protocol && this.proxy.protocol === 'https:'; // EventEmitter.call(this);
} // inherits(ZitiAgent, EventEmitter);

/**
 * Default port to connect to.
 */


ZitiAgent.prototype.defaultPort = 443;
/**
 * Called when creating a new HTTP request with this ZitiAgent instance.
 *
 * @api public
 */

ZitiAgent.prototype.addRequest = function (req, host, port, localAddress) {
  var opts;

  if (_typeof(host) == 'object') {
    // >= v0.11.x API
    opts = host;
  } else {
    // <= v0.10.x API
    opts = {
      host: host,
      port: port,
      localAddress: localAddress
    };
  } // hint to use "Connection: close"


  req.shouldKeepAlive = false; // create the `ZitiSocket` instance

  var info = {
    serviceName: opts.serviceName,
    conn: opts.conn,
    host: opts.hostname || opts.host,
    port: Number(opts.port) || this.defaultPort,
    localAddress: opts.localAddress,
    isWebSocket: opts.isWebSocket
  };
  this.createConnection(info, function (err, socket) {
    if (err) {
      req.emit('error', err);
    } else {
      req.onSocket(socket);
    }
  });
};
/**
 * Creates and returns a `ZitiSocket` instance to use for an HTTP request.
 *
 * @api public
 */


ZitiAgent.prototype.createConnection = function (opts, deferredFn) {
  var _this = this;

  this.deferredFn = deferredFn;

  var onSocketConnect = function onSocketConnect() {
    _this.deferredFn(null, _this.socket);
  };

  this.socket = new ZitiSocket(opts);
  this.socket.connect(opts);
  this.socket.once('connect', onSocketConnect);
};