"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/web.timers.js");

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.replace.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var flatOptions = require('flat-options');

var forge = require('node-forge');

var isUndefined = require('lodash.isundefined');

var isNull = require('lodash.isnull');

var consola = require('consola');

var MicroModal;
var modalMsg;

if (typeof window !== 'undefined') {
  MicroModal = require('micromodal');
  modalMsg = require('../ui/identity_modal/keypair_generation_msg');
}

var ZitiReporter = require('../utils/ziti-reporter');

var ls = require('../utils/localstorage');

var defaultOptions = require('./options');

var zitiConstants = require('../constants');

var identityModalCSS;
var keypairModalHTML;

if (typeof window !== 'undefined') {
  identityModalCSS = require('../ui/identity_modal/css');
  keypairModalHTML = require('../ui/identity_modal/keypair_generation_html');
}
/**
 * Expose `ZitiPKI`.
 */


module.exports = ZitiPKI;
/**
 * Initialize a new `ZitiPKI`.
 *
 * @api public
 */

function ZitiPKI(obj) {
  var ctx = mixin(obj);
  return ctx;
}
/**
 * Mixin the prototype properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */


function mixin(obj) {
  for (var key in ZitiPKI.prototype) {
    if (Object.prototype.hasOwnProperty.call(ZitiPKI.prototype, key)) obj[key] = ZitiPKI.prototype[key];
  }

  return obj;
}
/**
 * Initialize the Ziti PKI.
 * 
 * Tasks:
 * - validate options
 * - create logger if necessary
 *
 * @param {Object} [options]
 * @returns {nothing}   
 */


ZitiPKI.prototype.init = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(options) {
    var self;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            self = this;
            return _context2.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(resolve, reject) {
                var _options;

                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _options = flatOptions(options, defaultOptions);
                        self.ctx = _options.ctx;
                        self.logger = _options.logger;

                        if (isNull(self.logger)) {
                          self.logger = consola.create({
                            level: _options.logLevel,
                            reporters: [new ZitiReporter()],
                            defaults: {
                              additionalColor: 'white'
                            }
                          });
                          self.logger.wrapConsole();
                        }

                        resolve();

                      case 5:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x2, _x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();

ZitiPKI.prototype._haveKeypair = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
  var self;
  return regeneratorRuntime.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          self = this;
          return _context4.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(resolve, reject) {
              var privateKey, publicKey;
              return regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.next = 2;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY);

                    case 2:
                      privateKey = _context3.sent;
                      _context3.next = 5;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY);

                    case 5:
                      publicKey = _context3.sent;

                      if (isNull(privateKey) || isUndefined(privateKey) || isNull(publicKey) || isUndefined(publicKey)) {
                        resolve(false);
                      } else {
                        resolve(true);
                      }

                    case 7:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3);
            }));

            return function (_x4, _x5) {
              return _ref4.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4, this);
}));
/**
 * Generate the keypair for this browser
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */

ZitiPKI.prototype.generateKeyPair = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7() {
  var self, privateKeySize, stepInterval;
  return regeneratorRuntime.wrap(function _callee7$(_context7) {
    while (1) {
      switch (_context7.prev = _context7.next) {
        case 0:
          self = this;
          privateKeySize = 4096;
          stepInterval = 500; // step-time interval, in ms, for generating keypair

          return _context7.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(resolve, reject) {
              var haveKeys, startTime, endTime, state, step;
              return regeneratorRuntime.wrap(function _callee6$(_context6) {
                while (1) {
                  switch (_context6.prev = _context6.next) {
                    case 0:
                      _context6.next = 2;
                      return self._haveKeypair();

                    case 2:
                      haveKeys = _context6.sent;

                      if (!haveKeys) {
                        _context6.next = 7;
                        break;
                      }

                      self.logger.trace('Pre-existing KeyPair found; skipping new keypair generation');
                      resolve(false);
                      return _context6.abrupt("return");

                    case 7:
                      self.logger.info('Starting KeyPair Generation'); // if (typeof window !== 'undefined') {
                      //   identityModalCSS.inject();
                      //   keypairModalHTML.inject();
                      //   MicroModal.init({
                      //     onShow: modal => console.info(`${modal.id} is shown`), // [1]
                      //     onClose: modal => console.info(`${modal.id} is hidden`), // [2]
                      //     openTrigger: 'ziti-data-micromodal-trigger', // [3]
                      //     closeTrigger: 'data-custom-close', // [4]
                      //     openClass: 'is-open', // [5]
                      //     disableScroll: true, // [6]
                      //     disableFocus: false, // [7]
                      //     awaitOpenAnimation: false, // [8]
                      //     awaitCloseAnimation: true, // [9]
                      //     debugMode: false // [10]
                      //   });
                      //   MicroModal.show('ziti-keypair-modal');
                      //   modalMsg.setMessage('Please do not close this browser window.');
                      //   modalMsg.setProgress('Zero-Trust KeyPair creation in progress.');
                      // }

                      startTime = performance.now(); // Generate an RSA key pair, run for a few ms at a time on the main JS thread, so as not to completely block JS execution in browser.

                      // Generate an RSA key pair, run for a few ms at a time on the main JS thread, so as not to completely block JS execution in browser.
                      state = forge.pki.rsa.createKeyPairGenerationState(privateKeySize);

                      step = /*#__PURE__*/function () {
                        var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
                          var timeDiff, seconds, privatePEM, publicPEM;
                          return regeneratorRuntime.wrap(function _callee5$(_context5) {
                            while (1) {
                              switch (_context5.prev = _context5.next) {
                                case 0:
                                  if (forge.pki.rsa.stepKeyPairGenerationState(state, stepInterval)) {
                                    _context5.next = 9;
                                    break;
                                  }

                                  endTime = performance.now();
                                  timeDiff = endTime - startTime; //in ms 

                                  //in ms 
                                  timeDiff /= 1000; // strip the ms

                                  // strip the ms
                                  seconds = Math.round(timeDiff);

                                  if (seconds % 2 == 0) {
                                    self.logger.debug('Zero-Trust KeyPair creation in progress: elapsed[' + seconds + ' sec]');

                                    if (typeof window !== 'undefined') {
                                      modalMsg.setProgress('Zero-Trust KeyPair creation in progress: elapsed[' + seconds + ' sec]');
                                    }
                                  }

                                  setTimeout(step, 50);
                                  _context5.next = 29;
                                  break;

                                case 9:
                                  // Now that we have a keypair
                                  self._privateKey = state.keys.privateKey;
                                  self._publicKey = state.keys.publicKey;
                                  privatePEM = forge.pki.privateKeyToPem(self._privateKey);
                                  privatePEM = privatePEM.replace(/\\n/g, '\n');
                                  privatePEM = privatePEM.replace(/[\r]+/g, '');
                                  privatePEM = privatePEM.replace(/\n/g, '\x0a');
                                  _context5.next = 17;
                                  return ls.setWithExpiry(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY, privatePEM, new Date(8640000000000000));

                                case 17:
                                  publicPEM = forge.pki.publicKeyToPem(self._publicKey);
                                  publicPEM = publicPEM.replace(/\\n/g, '\n');
                                  publicPEM = publicPEM.replace(/[\r]+/g, '');
                                  publicPEM = publicPEM.replace(/\n/g, '\x0a');
                                  _context5.next = 23;
                                  return ls.setWithExpiry(zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY, publicPEM, new Date(8640000000000000));

                                case 23:
                                  endTime = performance.now();
                                  timeDiff = endTime - startTime; //in ms 

                                  //in ms 
                                  timeDiff /= 1000; // strip the ms

                                  // strip the ms
                                  seconds = Math.round(timeDiff);
                                  self.logger.info('KeyPair Generation COMPLETE... elapsed[' + seconds + ' sec]'); // if (typeof window !== 'undefined') {
                                  //   modalMsg.setProgress('KeyPair Generation COMPLETE... elapsed[' + seconds + ' sec]');
                                  //   modalMsg.setMessage('You may now REFRESH this browser window to load the application.');
                                  //   MicroModal.close('ziti-keypair-modal');
                                  // }

                                  // if (typeof window !== 'undefined') {
                                  //   modalMsg.setProgress('KeyPair Generation COMPLETE... elapsed[' + seconds + ' sec]');
                                  //   modalMsg.setMessage('You may now REFRESH this browser window to load the application.');
                                  //   MicroModal.close('ziti-keypair-modal');
                                  // }
                                  resolve(true);

                                case 29:
                                case "end":
                                  return _context5.stop();
                              }
                            }
                          }, _callee5);
                        }));

                        return function step() {
                          return _ref7.apply(this, arguments);
                        };
                      }();

                      setTimeout(step); // initiate async keypair generation

                    case 12:
                    case "end":
                      return _context6.stop();
                  }
                }
              }, _callee6);
            }));

            return function (_x6, _x7) {
              return _ref6.apply(this, arguments);
            };
          }()));

        case 4:
        case "end":
          return _context7.stop();
      }
    }
  }, _callee7, this);
}));
/**
 * Wait for keypair generation to complete before returning
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */

ZitiPKI.prototype.awaitKeyPairGenerationComplete = /*#__PURE__*/function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(bypassRenderingOfUI) {
    var self;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            self = this;
            return _context10.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(resolve, reject) {
                var haveKeys;
                return regeneratorRuntime.wrap(function _callee9$(_context9) {
                  while (1) {
                    switch (_context9.prev = _context9.next) {
                      case 0:
                        _context9.next = 2;
                        return self._haveKeypair();

                      case 2:
                        haveKeys = _context9.sent;

                        if (!haveKeys) {
                          _context9.next = 7;
                          break;
                        }

                        self.logger.info('Pre-existing KeyPair found; skipping wait for keypair generation completion');
                        resolve(false);
                        return _context9.abrupt("return");

                      case 7:
                        if (typeof window !== 'undefined') {
                          if (bypassRenderingOfUI) {
                            self.logger.info('bypassRenderingOfUI');
                          } else {
                            identityModalCSS.inject();
                            keypairModalHTML.inject();
                            MicroModal.init({
                              onShow: function onShow(modal) {
                                return console.info("".concat(modal.id, " is shown"));
                              },
                              // [1]
                              onClose: function onClose(modal) {
                                return console.info("".concat(modal.id, " is hidden"));
                              },
                              // [2]
                              openTrigger: 'ziti-data-micromodal-trigger',
                              // [3]
                              closeTrigger: 'data-custom-close',
                              // [4]
                              openClass: 'is-open',
                              // [5]
                              disableScroll: true,
                              // [6]
                              disableFocus: false,
                              // [7]
                              awaitOpenAnimation: false,
                              // [8]
                              awaitCloseAnimation: true,
                              // [9]
                              debugMode: false // [10]

                            });
                            MicroModal.show('ziti-keypair-modal');
                            modalMsg.setMessage('Please do not close this browser window.');
                            modalMsg.setProgress('Zero-Trust KeyPair creation in progress.');
                          }
                        }

                        (function () {
                          var _waitForKeyPairGenerationComplete = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8() {
                            var haveKeys;
                            return regeneratorRuntime.wrap(function _callee8$(_context8) {
                              while (1) {
                                switch (_context8.prev = _context8.next) {
                                  case 0:
                                    _context8.next = 2;
                                    return self._haveKeypair();

                                  case 2:
                                    haveKeys = _context8.sent;

                                    if (!haveKeys) {
                                      _context8.next = 6;
                                      break;
                                    }

                                    MicroModal.close('ziti-keypair-modal');
                                    return _context8.abrupt("return", resolve(true));

                                  case 6:
                                    setTimeout(waitForKeyPairGenerationComplete, 200);

                                  case 7:
                                  case "end":
                                    return _context8.stop();
                                }
                              }
                            }, _callee8);
                          }));

                          function waitForKeyPairGenerationComplete() {
                            return _waitForKeyPairGenerationComplete.apply(this, arguments);
                          }

                          return waitForKeyPairGenerationComplete;
                        })()();

                      case 9:
                      case "end":
                        return _context9.stop();
                    }
                  }
                }, _callee9);
              }));

              return function (_x9, _x10) {
                return _ref9.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10, this);
  }));

  return function (_x8) {
    return _ref8.apply(this, arguments);
  };
}();