"use strict";

require("regenerator-runtime/runtime.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.promise.js");

require("core-js/modules/web.timers.js");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Module dependencies.
 */
var flatOptions = require('flat-options');

var forge = require('node-forge');

var isUndefined = require('lodash.isundefined');

var isNull = require('lodash.isnull');

var consola = require('consola');

var MicroModal;
var modalMsg;
var identityModalLogin;
var identityModalKeypairDirectory;
var error;

if (typeof window !== 'undefined') {
  MicroModal = require('micromodal');
  modalMsg = require('../ui/identity_modal/keypair_generation_msg');
  identityModalLogin = require('./login');
  identityModalKeypairDirectory = require('./keypairDirectory');
  error = require('./error');
}

var ZitiReporter = require('../utils/ziti-reporter');

var ls = require('../utils/localstorage');

var defaultOptions = require('./options');

var zitiConstants = require('../constants');

var identityModalCSS;
var updbModalHTML;
var updbKeypairDirectoryModalHTML;

if (typeof window !== 'undefined') {
  identityModalCSS = require('../ui/identity_modal/css');
  updbModalHTML = require('../ui/identity_modal/updb_prompt_html');
  updbKeypairDirectoryModalHTML = require('../ui/identity_modal/updb_prompt_keypairDirectory_html');
  reloadingpageModalHTML = require('../ui/identity_modal/reloading_page_html');
}
/**
 * Expose `ZitiUPDB`.
 */


module.exports = ZitiUPDB;
/**
 * Initialize a new `ZitiUPDB`.
 *
 * @api public
 */

function ZitiUPDB(obj) {
  var ctx = mixin(obj);
  return ctx;
}
/**
 * Mixin the prototype properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */


function mixin(obj) {
  for (var key in ZitiUPDB.prototype) {
    if (Object.prototype.hasOwnProperty.call(ZitiUPDB.prototype, key)) obj[key] = ZitiUPDB.prototype[key];
  }

  return obj;
}
/**
 * Initialize the Ziti Context.
 * 
 * Tasks:
 * - validate options
 * - create logger if necessary
 *
 * @param {Object} [options]
 * @returns {nothing}   
 */


ZitiUPDB.prototype.init = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(options) {
    var self;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            self = this;
            return _context2.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(resolve, reject) {
                var _options;

                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        _options = flatOptions(options, defaultOptions);
                        self.ctx = _options.ctx;
                        self.logger = _options.logger;

                        if (isNull(self.logger)) {
                          self.logger = consola.create({
                            level: _options.logLevel,
                            reporters: [new ZitiReporter()],
                            defaults: {
                              additionalColor: 'white'
                            }
                          });
                          self.logger.wrapConsole();
                        }

                        resolve();

                      case 5:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              return function (_x2, _x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function (_x) {
    return _ref.apply(this, arguments);
  };
}();

ZitiUPDB.prototype._haveCreds = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
  var self;
  return regeneratorRuntime.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          self = this;
          return _context4.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(resolve, reject) {
              var username, password;
              return regeneratorRuntime.wrap(function _callee3$(_context3) {
                while (1) {
                  switch (_context3.prev = _context3.next) {
                    case 0:
                      _context3.next = 2;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_USERNAME);

                    case 2:
                      username = _context3.sent;
                      _context3.next = 5;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_PASSWORD);

                    case 5:
                      password = _context3.sent;

                      if (isNull(username) || isUndefined(username) || isNull(password) || isUndefined(password)) {
                        resolve(false);
                      } else {
                        self._loginFormValues = {
                          username: username,
                          password: password
                        };
                        resolve(true);
                      }

                    case 7:
                    case "end":
                      return _context3.stop();
                  }
                }
              }, _callee3);
            }));

            return function (_x4, _x5) {
              return _ref4.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4, this);
}));
ZitiUPDB.prototype._haveKeypair = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
  var self;
  return regeneratorRuntime.wrap(function _callee6$(_context6) {
    while (1) {
      switch (_context6.prev = _context6.next) {
        case 0:
          self = this;
          return _context6.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(resolve, reject) {
              var publicKey, privateKey;
              return regeneratorRuntime.wrap(function _callee5$(_context5) {
                while (1) {
                  switch (_context5.prev = _context5.next) {
                    case 0:
                      _context5.next = 2;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY);

                    case 2:
                      publicKey = _context5.sent;
                      _context5.next = 5;
                      return ls.get(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY);

                    case 5:
                      privateKey = _context5.sent;

                      if (isNull(publicKey) || isUndefined(publicKey) || isNull(privateKey) || isUndefined(privateKey)) {
                        resolve(false);
                      } else {
                        resolve(true);
                      }

                    case 7:
                    case "end":
                      return _context5.stop();
                  }
                }
              }, _callee5);
            }));

            return function (_x6, _x7) {
              return _ref6.apply(this, arguments);
            };
          }()));

        case 2:
        case "end":
          return _context6.stop();
      }
    }
  }, _callee6, this);
}));
/**
 * Prompt the user for their creds
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */

ZitiUPDB.prototype.promptForCreds = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8() {
  var self;
  return regeneratorRuntime.wrap(function _callee8$(_context8) {
    while (1) {
      switch (_context8.prev = _context8.next) {
        case 0:
          self = this;
          self.logger.info('Starting Creds Prompt');

          if (typeof window !== 'undefined') {
            if (!self.identityModalInjected) {
              identityModalCSS.inject();
              updbModalHTML.inject();
              self.identityModalInjected = true;
            }

            this._loginFormValues = undefined;
            identityModalLogin.injectButtonHandler( /*#__PURE__*/function () {
              var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(results) {
                var expiry;
                return regeneratorRuntime.wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        self._loginFormValues = results;
                        self.logger.debug('Login Form cb(): results [%o]', results); // Save the username/password, but ensure it vanishes in 5 minutes

                        expiry = new Date();
                        expiry.setMinutes(expiry.getMinutes() + 5);
                        _context7.next = 6;
                        return ls.setWithExpiry(zitiConstants.get().ZITI_IDENTITY_USERNAME, self._loginFormValues.username, expiry);

                      case 6:
                        _context7.next = 8;
                        return ls.setWithExpiry(zitiConstants.get().ZITI_IDENTITY_PASSWORD, self._loginFormValues.password, expiry);

                      case 8:
                        _context7.next = 10;
                        return ls.removeItem(zitiConstants.get().ZITI_API_SESSION_TOKEN);

                      case 10:
                        _context7.next = 12;
                        return ls.removeItem(zitiConstants.get().ZITI_CLIENT_CERT_PEM);

                      case 12:
                      case "end":
                        return _context7.stop();
                    }
                  }
                }, _callee7);
              }));

              return function (_x8) {
                return _ref8.apply(this, arguments);
              };
            }());
            MicroModal.init({
              onShow: function onShow(modal) {
                return console.info("".concat(modal.id, " is shown"));
              },
              // [1]
              onClose: function onClose(modal) {
                return console.info("".concat(modal.id, " is hidden"));
              },
              // [2]
              openTrigger: 'ziti-data-micromodal-trigger',
              // [3]
              closeTrigger: 'data-custom-close',
              // [4]
              openClass: 'is-open',
              // [5]
              disableScroll: true,
              // [6]
              disableFocus: false,
              // [7]
              awaitOpenAnimation: false,
              // [8]
              awaitCloseAnimation: true,
              // [9]
              debugMode: false // [10]

            });
            MicroModal.show('ziti-updb-modal');
            self.modalShown = true;
          }

        case 3:
        case "end":
          return _context8.stop();
      }
    }
  }, _callee8, this);
}));
/**
 * Prompt the user for a dir to save keypair into
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */

ZitiUPDB.prototype.promptForKeypairDirectory = /*#__PURE__*/function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(source) {
    var self, text;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            self = this;

            if (source === zitiConstants.get().ZITI_IDENTITY_KEYPAIR_OBTAIN_FROM_FS) {
              text = 'Select Folder for Keypair storage.';
            } else if (source === zitiConstants.get().ZITI_IDENTITY_KEYPAIR_OBTAIN_FROM_IDB) {
              text = 'New KeyPair generation is complete.<br/><br/>Select Folder to write new Keypair into.';
            } else {
              text = 'huh?';
            }

            self.logger.info('Starting keypairDirectory Prompt');

            if (typeof window !== 'undefined') {
              // if (!self.keypairDirectoryModalInjected) {
              identityModalCSS.inject();
              updbKeypairDirectoryModalHTML.inject(text); // self.keypairDirectoryModalInjected = true;
              // }

              this._keypairPresent = undefined;
              identityModalKeypairDirectory.injectButtonHandler(self, source, /*#__PURE__*/function () {
                var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(result) {
                  return regeneratorRuntime.wrap(function _callee9$(_context9) {
                    while (1) {
                      switch (_context9.prev = _context9.next) {
                        case 0:
                          self._keypairPresent = result;
                          self.logger.debug('keypairDirectory Form cb(): results [%o]', self._keypairPresent);
                          MicroModal.close('ziti-updb-modal-keypairDirectory');
                          self.modalShown = false;

                        case 4:
                        case "end":
                          return _context9.stop();
                      }
                    }
                  }, _callee9);
                }));

                return function (_x10) {
                  return _ref10.apply(this, arguments);
                };
              }());
              MicroModal.init({
                onShow: function onShow(modal) {
                  return console.info("".concat(modal.id, " is shown"));
                },
                // [1]
                onClose: function onClose(modal) {
                  return console.info("".concat(modal.id, " is hidden"));
                },
                // [2]
                openTrigger: 'ziti-data-micromodal-trigger',
                // [3]
                closeTrigger: 'data-custom-close',
                // [4]
                openClass: 'is-open',
                // [5]
                disableScroll: true,
                // [6]
                disableFocus: false,
                // [7]
                awaitOpenAnimation: false,
                // [8]
                awaitCloseAnimation: true,
                // [9]
                debugMode: false // [10]

              });
              MicroModal.show('ziti-updb-modal-keypairDirectory');
              self.modalShown = true;
            }

          case 4:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10, this);
  }));

  return function (_x9) {
    return _ref9.apply(this, arguments);
  };
}();
/**
 * Display 'Reloading Page'
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */


ZitiUPDB.prototype.relodingPage = function () {
  var self = this;

  if (typeof window !== 'undefined') {
    identityModalCSS.inject();
    reloadingpageModalHTML.inject();
    MicroModal.init({
      onShow: function onShow(modal) {
        return console.info("".concat(modal.id, " is shown"));
      },
      // [1]
      onClose: function onClose(modal) {
        return console.info("".concat(modal.id, " is hidden"));
      },
      // [2]
      openTrigger: 'ziti-data-micromodal-trigger',
      // [3]
      closeTrigger: 'data-custom-close',
      // [4]
      openClass: 'is-open',
      // [5]
      disableScroll: true,
      // [6]
      disableFocus: false,
      // [7]
      awaitOpenAnimation: false,
      // [8]
      awaitCloseAnimation: true,
      // [9]
      debugMode: false // [10]

    });
    MicroModal.show('ziti-reloadingpage-modal');
  }
};
/**
 * Prompt the user for their creds
 * 
 * @params  {nothing}   
 * @returns {nothing}   
 */


ZitiUPDB.prototype.closeLoginForm = function () {
  if (this.modalShown) {
    MicroModal.close('ziti-updb-modal');
    this.modalShown = false;
  }
};
/**
 * Return a Promise that will resolve as soon as we have acquired login creds from the UI.
 *
 * @returns {Promise}   
 */


ZitiUPDB.prototype.awaitLoginFormComplete = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12() {
  var self;
  return regeneratorRuntime.wrap(function _callee12$(_context12) {
    while (1) {
      switch (_context12.prev = _context12.next) {
        case 0:
          this.logger.debug('ZitiUPDB.awaitLoginFormComplete() starting');
          self = this;
          return _context12.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref12 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(resolve, reject) {
              return regeneratorRuntime.wrap(function _callee11$(_context11) {
                while (1) {
                  switch (_context11.prev = _context11.next) {
                    case 0:
                      self._loginFormValues = undefined;
                      self.promptForCreds();

                      (function waitForLoginFormComplete() {
                        if (self._loginFormValues) {
                          return resolve(self._loginFormValues);
                        }

                        self.logger.trace('ZitiUPDB.awaitLoginFormComplete() _loginFormValues still not present');
                        setTimeout(waitForLoginFormComplete, 250);
                      })();

                    case 3:
                    case "end":
                      return _context11.stop();
                  }
                }
              }, _callee11);
            }));

            return function (_x11, _x12) {
              return _ref12.apply(this, arguments);
            };
          }()));

        case 3:
        case "end":
          return _context12.stop();
      }
    }
  }, _callee12, this);
}));
/**
 * Return a Promise that will resolve as soon as we have acquired login creds from the UI
 * that are acceptable to the Ziti Controller.
 *
 * @returns {Promise}   
 */

ZitiUPDB.prototype.awaitCredentialsAndAPISession = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14() {
  var _this = this;

  var self;
  return regeneratorRuntime.wrap(function _callee14$(_context14) {
    while (1) {
      switch (_context14.prev = _context14.next) {
        case 0:
          this.logger.debug('ZitiUPDB.awaitCredentialsAndAPISession() starting');
          self = this;
          return _context14.abrupt("return", new Promise( /*#__PURE__*/function () {
            var _ref14 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13(resolve, reject) {
              var loginFormValues;
              return regeneratorRuntime.wrap(function _callee13$(_context13) {
                while (1) {
                  switch (_context13.prev = _context13.next) {
                    case 0:
                      _context13.next = 2;
                      return self.awaitLoginFormComplete();

                    case 2:
                      loginFormValues = _context13.sent;
                      // await user creds input
                      self.ctx.setLoginFormValues(loginFormValues);
                      _context13.next = 6;
                      return self.ctx.getFreshAPISession();

                    case 6:
                      validCreds = _context13.sent;

                      if (!validCreds) {
                        error.setMessage('ERROR: Invalid credentials');
                      }

                    case 8:
                      if (!validCreds) {
                        _context13.next = 0;
                        break;
                      }

                    case 9:
                      _this.logger.debug('ZitiUPDB.awaitCredentialsAndAPISession() now have valid creds, closing login form');

                      self.closeLoginForm();
                      resolve();

                    case 12:
                    case "end":
                      return _context13.stop();
                  }
                }
              }, _callee13);
            }));

            return function (_x13, _x14) {
              return _ref14.apply(this, arguments);
            };
          }()));

        case 3:
        case "end":
          return _context14.stop();
      }
    }
  }, _callee14, this);
}));
/**
 * Return a Promise that will resolve as soon as we have acquired keypair dir location from the UI.
 *
 * @returns {Promise}   
 */

ZitiUPDB.prototype.awaitKeypair = /*#__PURE__*/function () {
  var _ref15 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee16(source) {
    var self;
    return regeneratorRuntime.wrap(function _callee16$(_context16) {
      while (1) {
        switch (_context16.prev = _context16.next) {
          case 0:
            this.logger.debug('ZitiUPDB.awaitKeypair() starting');
            self = this;
            return _context16.abrupt("return", new Promise( /*#__PURE__*/function () {
              var _ref16 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee15(resolve, reject) {
                return regeneratorRuntime.wrap(function _callee15$(_context15) {
                  while (1) {
                    switch (_context15.prev = _context15.next) {
                      case 0:
                        self._keypairPresent = undefined;
                        self.promptForKeypairDirectory(source);

                        (function waitForKeypairDirectoryFormComplete() {
                          if (self._keypairPresent == zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY_FILE_NOT_FOUND) {
                            return resolve(zitiConstants.get().ZITI_IDENTITY_PUBLIC_KEY_FILE_NOT_FOUND);
                          }

                          if (self._keypairPresent == zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY_FILE_NOT_FOUND) {
                            return resolve(zitiConstants.get().ZITI_IDENTITY_PRIVATE_KEY_FILE_NOT_FOUND);
                          }

                          if (self._keypairPresent == zitiConstants.get().ZITI_IDENTITY_KEYPAIR_FOUND) {
                            return resolve(zitiConstants.get().ZITI_IDENTITY_KEYPAIR_FOUND);
                          }

                          self.logger.trace('ZitiUPDB.awaitLoginFormComplete() _keypairPresent still not true');
                          setTimeout(waitForKeypairDirectoryFormComplete, 500);
                        })();

                      case 3:
                      case "end":
                        return _context15.stop();
                    }
                  }
                }, _callee15);
              }));

              return function (_x16, _x17) {
                return _ref16.apply(this, arguments);
              };
            }()));

          case 3:
          case "end":
            return _context16.stop();
        }
      }
    }, _callee16, this);
  }));

  return function (_x15) {
    return _ref15.apply(this, arguments);
  };
}();