"use strict";

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.array-buffer.slice.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.typed-array.uint8-array.js");

require("core-js/modules/es.typed-array.copy-within.js");

require("core-js/modules/es.typed-array.every.js");

require("core-js/modules/es.typed-array.fill.js");

require("core-js/modules/es.typed-array.filter.js");

require("core-js/modules/es.typed-array.find.js");

require("core-js/modules/es.typed-array.find-index.js");

require("core-js/modules/es.typed-array.for-each.js");

require("core-js/modules/es.typed-array.includes.js");

require("core-js/modules/es.typed-array.index-of.js");

require("core-js/modules/es.typed-array.iterator.js");

require("core-js/modules/es.typed-array.join.js");

require("core-js/modules/es.typed-array.last-index-of.js");

require("core-js/modules/es.typed-array.map.js");

require("core-js/modules/es.typed-array.reduce.js");

require("core-js/modules/es.typed-array.reduce-right.js");

require("core-js/modules/es.typed-array.reverse.js");

require("core-js/modules/es.typed-array.set.js");

require("core-js/modules/es.typed-array.slice.js");

require("core-js/modules/es.typed-array.some.js");

require("core-js/modules/es.typed-array.sort.js");

require("core-js/modules/es.typed-array.subarray.js");

require("core-js/modules/es.typed-array.to-locale-string.js");

require("core-js/modules/es.typed-array.to-string.js");

require("core-js/modules/es.regexp.exec.js");

require("core-js/modules/es.string.split.js");

require("core-js/modules/es.string.trim.js");

require("core-js/modules/es.regexp.to-string.js");

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
var asn1js = require('asn1js');

var pkijs = require("pkijs");

var Certificate = pkijs.Certificate;
/**
 *	Convert base64 string to buffer
 *
 * @param {string} b64str
 */

exports.base64StringToArrayBuffer = function (b64str) {
  var byteStr = atob(b64str);
  var bytes = new Uint8Array(byteStr.length);

  for (var i = 0; i < byteStr.length; i++) {
    bytes[i] = byteStr.charCodeAt(i);
  }

  return bytes.buffer;
};
/**
 *	Convert PEM string to binary
 *
 * @param {string} pem
 */


exports.convertPemToBinary = function (pem) {
  var lines = pem.split('\n');
  var encoded = '';

  for (var i = 0; i < lines.length; i++) {
    if (lines[i].trim().length > 0 && lines[i].indexOf('-BEGIN RSA PRIVATE KEY-') < 0 && lines[i].indexOf('-BEGIN RSA PUBLIC KEY-') < 0 && lines[i].indexOf('-BEGIN PUBLIC KEY-') < 0 && lines[i].indexOf('-BEGIN CERTIFICATE-') < 0 && lines[i].indexOf('-BEGIN PRIVATE KEY-') < 0 && lines[i].indexOf('-END PRIVATE KEY-') < 0 && lines[i].indexOf('-END CERTIFICATE-') < 0 && lines[i].indexOf('-END PUBLIC KEY-') < 0 && lines[i].indexOf('-END RSA PRIVATE KEY-') < 0 && lines[i].indexOf('-END RSA PUBLIC KEY-') < 0) {
      encoded += lines[i].trim();
    }
  }

  return exports.base64StringToArrayBuffer(encoded);
};
/**
 *	Convert buffer to Certificate
 *
 * @param {Buffer} certificateBuffer
 */


exports.convertBinaryToCertificate = function (certificateBuffer) {
  var asn1 = asn1js.fromBER(certificateBuffer);

  if (asn1.offset === -1) {
    console.log("Can not parse binary data");
  }

  var certificate = new Certificate({
    schema: asn1.result
  });
  return certificate;
};
/**
 *	Convert PEM to Certificate
 *
 * @param {string} pem
 */


exports.convertPemToCertificate = function (pem) {
  return exports.convertBinaryToCertificate(exports.convertPemToBinary(pem));
};
/**
 *	Convert buffer to Certificate
 *
 * @param {string} pem
 */


exports.printCertificate = function (certificate) {
  console.log(certificate);
  console.log('Certificate Serial Number');
  console.log('Certificate Issuance');
  console.log(certificate.notBefore.value.toString());
  console.log('Certificate Expiry');
  console.log(certificate.notAfter.value.toString());
  console.log(certificate.issuer);
};
/**
 *	Return time (in millis) for when Certificate expires
 *
 * @param {Buffer} certificateBuffer
 */


exports.getExpiryTimeFromCertificate = function (certificate) {
  return certificate.notAfter.toSchema().toDate().getTime();
};
/**
 *	Return time (human-readable) for when Certificate expires
 *
 * @param {Buffer} certificateBuffer
 */


exports.getExpiryStringFromCertificate = function (certificate) {
  return certificate.notAfter.toSchema().toDate().toString();
};
/**
 *	Return time (in millis) for when Certificate becomes usable
 *
 * @param {Buffer} certificateBuffer
 */


exports.getBecomesUsableTimeFromCertificate = function (certificate) {
  return certificate.notBefore.toSchema().toDate().getTime();
};
/**
 *	Return time (human-readable) for when Certificate becomes usable
 *
 * @param {Buffer} certificateBuffer
 */


exports.getBecomesUsableStringFromCertificate = function (certificate) {
  return certificate.notBefore.toSchema().toDate().toString();
};