"use strict";

require("core-js/modules/es.array.iterator.js");

require("core-js/modules/es.map.js");

require("core-js/modules/es.object.to-string.js");

require("core-js/modules/es.string.iterator.js");

require("core-js/modules/web.dom-collections.iterator.js");

require("core-js/modules/web.dom-collections.for-each.js");

require("core-js/modules/es.array.concat.js");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/*
Copyright 2019-2020 Netfoundry, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/**
 * Class for manage pending requests.
 * @private
 */
var PromiseController = require('promise-controller');

var promiseFinally = require('promise.prototype.finally');

module.exports = /*#__PURE__*/function () {
  function Requests() {
    _classCallCheck(this, Requests);

    this._items = new Map();
  }
  /**
   * Creates new request and stores it in the list.
   *
   * @param {String|Number} requestId
   * @param {Function} fn
   * @param {Number} timeout
   * @returns {Promise}
   */


  _createClass(Requests, [{
    key: "create",
    value: function create(requestId, fn, timeout) {
      this._rejectExistingRequest(requestId);

      return this._createNewRequest(requestId, fn, timeout);
    }
  }, {
    key: "resolve",
    value: function resolve(requestId, data) {
      if (requestId && this._items.has(requestId)) {
        this._items.get(requestId).resolve(data);
      }
    }
  }, {
    key: "rejectAll",
    value: function rejectAll(error) {
      this._items.forEach(function (request) {
        return request.isPending ? request.reject(error) : null;
      });
    }
  }, {
    key: "_rejectExistingRequest",
    value: function _rejectExistingRequest(requestId) {
      var existingRequest = this._items.get(requestId);

      if (existingRequest && existingRequest.isPending) {
        existingRequest.reject(new Error("WebSocket request is replaced, id: ".concat(requestId)));
      }
    }
  }, {
    key: "_createNewRequest",
    value: function _createNewRequest(requestId, fn, timeout) {
      var _this = this;

      var request = new PromiseController({
        timeout: timeout,
        timeoutReason: "WebSocket request was rejected by timeout (".concat(timeout, " ms). RequestId: ").concat(requestId)
      });

      this._items.set(requestId, request);

      return promiseFinally(request.call(fn), function () {
        return _this._deleteRequest(requestId, request);
      });
    }
  }, {
    key: "_deleteRequest",
    value: function _deleteRequest(requestId, request) {
      // this check is important when request was replaced
      if (this._items.get(requestId) === request) {
        this._items.delete(requestId);
      }
    }
  }]);

  return Requests;
}();